@@ -18,7 +18,7 @@
  * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2018-2022 IBM Corporation.  All rights reserved.
- * Copyright (c) 2021-2024 Nanook Consulting  All rights reserved.
+ * Copyright (c) 2021-2025 Nanook Consulting  All rights reserved.
  * Copyright (c) 2022-2024 Triad National Security, LLC. All rights
  *                         reserved.
  * $COPYRIGHT$
@@ -121,6 +121,7 @@ static struct option ompioptions[] = {
     PMIX_OPTION_DEFINE(PRTE_CLI_FWD_SIGNALS, PMIX_ARG_REQD),
     PMIX_OPTION_DEFINE(PRTE_CLI_RUN_AS_ROOT, PMIX_ARG_NONE),
     PMIX_OPTION_DEFINE(PRTE_CLI_REPORT_CHILD_SEP, PMIX_ARG_NONE),
+    PMIX_OPTION_DEFINE(PRTE_CLI_HETERO_NODES, PMIX_ARG_NONE),
 
     /* debug options */
     PMIX_OPTION_DEFINE(PRTE_CLI_XTERM, PMIX_ARG_REQD),
@@ -158,6 +159,15 @@ static struct option ompioptions[] = {
     PMIX_OPTION_DEFINE(PRTE_CLI_PRELOAD_FILES, PMIX_ARG_REQD),
     PMIX_OPTION_SHORT_DEFINE(PRTE_CLI_PRELOAD_BIN, PMIX_ARG_NONE, 's'),
     PMIX_OPTION_SHORT_DEFINE(PRTE_CLI_FWD_ENVAR, PMIX_ARG_REQD, 'x'),
+#ifdef PMIX_CLI_PREPEND_ENVAR
+    PMIX_OPTION_DEFINE(PMIX_CLI_PREPEND_ENVAR, PMIX_ARG_REQD),
+#endif
+#ifdef PMIX_CLI_APPEND_ENVAR
+    PMIX_OPTION_DEFINE(PMIX_CLI_APPEND_ENVAR, PMIX_ARG_REQD),
+#endif
+#ifdef PMIX_CLI_UNSET_ENVAR
+    PMIX_OPTION_DEFINE(PMIX_CLI_UNSET_ENVAR, PMIX_ARG_REQD),
+#endif
     PMIX_OPTION_DEFINE(PRTE_CLI_WDIR, PMIX_ARG_REQD),
     PMIX_OPTION_DEFINE("wd", PMIX_ARG_REQD),
     PMIX_OPTION_DEFINE(PRTE_CLI_PATH, PMIX_ARG_REQD),
@@ -168,6 +178,10 @@ static struct option ompioptions[] = {
     PMIX_OPTION_DEFINE(PRTE_CLI_DEFAULT_HOSTFILE, PMIX_ARG_REQD),
     PMIX_OPTION_SHORT_DEFINE(PRTE_CLI_HOST, PMIX_ARG_REQD, 'H'),
     PMIX_OPTION_DEFINE(PRTE_CLI_FWD_ENVIRON, PMIX_ARG_OPTIONAL),
+    PMIX_OPTION_DEFINE(PRTE_CLI_MEM_ALLOC_KIND, PMIX_ARG_REQD),
+    PMIX_OPTION_DEFINE(PRTE_CLI_GPU_SUPPORT, PMIX_ARG_REQD),
+    PMIX_OPTION_DEFINE(PRTE_CLI_APP_PREFIX, PMIX_ARG_REQD),
+    PMIX_OPTION_DEFINE(PRTE_CLI_NO_APP_PREFIX, PMIX_ARG_NONE),
 
     /* placement options */
     /* Mapping options */
@@ -198,7 +212,9 @@ static struct option ompioptions[] = {
     /* mpiexec mandated form launch key parameters - MPI 4.0 */
     PMIX_OPTION_DEFINE("initial-errhandler", PMIX_ARG_REQD),
     /* mpiexec mandated form launch key parameters  - MPI 4.1*/
-    PMIX_OPTION_DEFINE("memory-alloc-kinds", PMIX_ARG_REQD),
+    PMIX_OPTION_DEFINE(PRTE_CLI_MEM_ALLOC_KIND, PMIX_ARG_REQD),
+    /* GPU support - on/off */
+    PMIX_OPTION_DEFINE(PRTE_CLI_GPU_SUPPORT, PMIX_ARG_REQD),
 
     /* Display Commumication Protocol : MPI_Init */
     PMIX_OPTION_DEFINE("display-comm", PMIX_ARG_NONE),
@@ -263,7 +279,7 @@ static void set_classpath_jar_file(prte_pmix_app_t *app, int index, char *jarfil
         char *fmt = ':' == app->app.argv[index][strlen(app->app.argv[index]-1)]
                     ? "%s%s/%s" : "%s:%s/%s";
         char *str;
-        asprintf(&str, fmt, app->app.argv[index], ompi_install_dirs_libdir, jarfile);
+        pmix_asprintf(&str, fmt, app->app.argv[index], ompi_install_dirs_libdir, jarfile);
         free(app->app.argv[index]);
         app->app.argv[index] = str;
     }
@@ -313,9 +329,9 @@ static int setup_app(prte_pmix_app_t *app)
             if (NULL == strstr(app->app.argv[i], ompi_install_dirs_libdir)) {
                 /* doesn't appear to - add it to be safe */
                 if (':' == app->app.argv[i][strlen(app->app.argv[i]-1)]) {
-                    asprintf(&value, "-Djava.library.path=%s%s", dptr, ompi_install_dirs_libdir);
+                    pmix_asprintf(&value, "-Djava.library.path=%s%s", dptr, ompi_install_dirs_libdir);
                 } else {
-                    asprintf(&value, "-Djava.library.path=%s:%s", dptr, ompi_install_dirs_libdir);
+                    pmix_asprintf(&value, "-Djava.library.path=%s:%s", dptr, ompi_install_dirs_libdir);
                 }
                 free(app->app.argv[i]);
                 app->app.argv[i] = value;
@@ -326,7 +342,7 @@ static int setup_app(prte_pmix_app_t *app)
 
     if (!found) {
         /* need to add it right after the java command */
-        asprintf(&value, "-Djava.library.path=%s", ompi_install_dirs_libdir);
+        pmix_asprintf(&value, "-Djava.library.path=%s", ompi_install_dirs_libdir);
         pmix_argv_insert_element(&app->app.argv, 1, value);
         free(value);
     }
@@ -348,7 +364,7 @@ static int setup_app(prte_pmix_app_t *app)
             }
             free(value);
             /* always add the local directory */
-            asprintf(&value, "%s:%s", app->app.cwd, app->app.argv[i+1]);
+            pmix_asprintf(&value, "%s:%s", app->app.cwd, app->app.argv[i+1]);
             free(app->app.argv[i+1]);
             app->app.argv[i+1] = value;
             break;
@@ -370,7 +386,7 @@ static int setup_app(prte_pmix_app_t *app)
                 }
                 free(value);
                 /* always add the local directory */
-                (void)asprintf(&value, "%s:%s", app->app.cwd, app->app.argv[1]);
+                pmix_asprintf(&value, "%s:%s", app->app.cwd, app->app.argv[1]);
                 free(app->app.argv[1]);
                 app->app.argv[1] = value;
                 pmix_argv_insert_element(&app->app.argv, 1, "-cp");
@@ -390,7 +406,7 @@ static int setup_app(prte_pmix_app_t *app)
             /* check for mpi.jar */
             value = pmix_os_path(false, ompi_install_dirs_libdir, "mpi.jar", NULL);
             if (access(value, F_OK ) != -1) {
-                (void)asprintf(&str2, "%s:%s", str, value);
+                pmix_asprintf(&str2, "%s:%s", str, value);
                 free(str);
                 str = str2;
             }
@@ -602,6 +618,8 @@ static int parse_cli(char **argv, pmix_cli_result_t *results,
         }
     }
 
+    // check for the --stream-buffering option
+
     if (NULL != results->tail) {
         /* search for the leader of the tail */
         for (n=0; NULL != argv[n]; n++) {
@@ -623,7 +641,7 @@ static int convert_deprecated_cli(pmix_cli_result_t *results,
 {
     char *option, *p1, *p2, *tmp, *tmp2, *output;
     int rc = PRTE_SUCCESS;
-    pmix_cli_item_t *opt, *nxt;
+    pmix_cli_item_t *opt, *nxt, *opt2;
     bool warn;
 
     if (silent) {
@@ -1040,6 +1058,32 @@ static int convert_deprecated_cli(pmix_cli_result_t *results,
                 }
             }
             PMIX_CLI_REMOVE_DEPRECATED(results, opt);
+        } else if (0 == strcmp(option, "stream-buffering")) {
+            uint16_t u16;
+
+            u16 = strtol(opt->values[0], NULL, 10);
+            // if the value is 0, then we need to direct the output to be ":raw"
+            if (0 == u16) {
+                opt2 = pmix_cmd_line_get_param(results, PRTE_CLI_OUTPUT);
+                if (NULL == opt2) {
+                    // add the output directive
+                    opt2 = PMIX_NEW(pmix_cli_item_t);
+                    opt2->key = strdup(PRTE_CLI_OUTPUT);
+                    PMIX_ARGV_APPEND_NOSIZE_COMPAT(&opt2->values, ":raw");
+                    pmix_list_append(&results->instances, &opt2->super);
+                } else {
+                    // see if the "raw" modifier is already present
+                    p1 = strstr(opt2->values[0], "raw");
+                    if (NULL == p1) {
+                        /* it isn't - need to add it. Since qualifiers are separated
+                         * from the directive by a ':' as well as from each other,
+                         * we can just append ":raw" to the string */
+                        pmix_asprintf(&p2, "%s:raw", opt2->values[0]);
+                        free(opt2->values[0]);
+                        opt2->values[0] = p2;
+                    }
+                }
+            }
         }
     }
 
@@ -1603,7 +1647,7 @@ static int parse_env(char **srcenv, char ***dstenv,
         }
     }
 
-    if (NULL != (opt = pmix_cmd_line_get_param(results, "memory-alloc-kinds"))) {
+    if (NULL != (opt = pmix_cmd_line_get_param(results, PRTE_CLI_MEM_ALLOC_KIND))) {
         rc = check_cache(&cache, &cachevals, "mpi_memory_alloc_kinds", opt->values[0]);
         if (PRTE_SUCCESS != rc) {
             PMIX_ARGV_FREE_COMPAT(cache);
@@ -1779,36 +1823,6 @@ static int parse_env(char **srcenv, char ***dstenv,
     }
     PMIX_ARGV_FREE_COMPAT(envlist);
 
-    /* now look for -x options - not allowed to conflict with a -mca option */
-    if (NULL != (opt = pmix_cmd_line_get_param(results, "x"))) {
-        for (i = 0; NULL != opt->values[i]; ++i) {
-            /* the value is the envar */
-            p1 = opt->values[i];
-            /* if there is an '=' in it, then they are setting a value */
-            if (NULL != (p2 = strchr(p1, '='))) {
-                *p2 = '\0';
-                ++p2;
-            } else {
-                p2 = getenv(p1);
-                if (NULL == p2) {
-                    continue;
-                }
-            }
-            /* not allowed to duplicate anything from an MCA param on the cmd line */
-            rc = check_cache_noadd(&cache, &cachevals, p1, p2);
-            if (PRTE_SUCCESS != rc) {
-                PMIX_ARGV_FREE_COMPAT(cache);
-                PMIX_ARGV_FREE_COMPAT(cachevals);
-                PMIX_ARGV_FREE_COMPAT(xparams);
-                PMIX_ARGV_FREE_COMPAT(xvals);
-                return rc;
-            }
-            /* cache this for later inclusion */
-            PMIX_ARGV_APPEND_NOSIZE_COMPAT(&xparams, p1);
-            PMIX_ARGV_APPEND_NOSIZE_COMPAT(&xvals, p2);
-        }
-    }
-
     /* process the resulting cache into the dstenv */
     if (NULL != cache) {
         for (i = 0; NULL != cache[i]; i++) {
