@@ -19,7 +19,7 @@
  * Copyright (c) 2014-2019 Research Organization for Information Science
  *                         and Technology (RIST).  All rights reserved.
  * Copyright (c) 2020      IBM Corporation.  All rights reserved.
- * Copyright (c) 2021-2024 Nanook Consulting  All rights reserved.
+ * Copyright (c) 2021-2025 Nanook Consulting  All rights reserved.
  * Copyright (c) 2024      Triad National Security, LLC. All rights
  *                         reserved.
  * $COPYRIGHT$
@@ -80,7 +80,7 @@ static void _query(int sd, short args, void *cbdata)
     prte_node_t *node, *ndptr;
     int j, k, rc;
     size_t m, n, p;
-    uint32_t key, nodeid, sessionid = UINT32_MAX;
+    uint32_t key, nodeid;
     char **nspaces, *hostname, *uri;
     char *cmdline;
     char **ans, *tmp;
@@ -92,7 +92,7 @@ static void _query(int sd, short args, void *cbdata)
     prte_proc_t *proct;
     pmix_proc_t *proc;
     size_t sz;
-    PRTE_HIDE_UNUSED_PARAMS(sd, args, sessionid);
+    PRTE_HIDE_UNUSED_PARAMS(sd, args);
 
     PMIX_ACQUIRE_OBJECT(cd);
 
@@ -121,6 +121,13 @@ static void _query(int sd, short args, void *cbdata)
                                          : "(not a string)"));
 
                 if (PMIX_CHECK_KEY(&q->qualifiers[n], PMIX_NSPACE)) {
+                    // the nspace could be NULL, indicating that this is a
+                    // wildcard request - if so, then ignore it here
+                    if (NULL == q->qualifiers[n].value.data.string ||
+                        0 == strlen(q->qualifiers[n].value.data.string)) {
+                        PMIX_LOAD_NSPACE(jobid, NULL);
+                        continue;
+                    }
                     /* Never trust the namespace string that is provided.
                      * First check to see if we know about this namespace. If
                      * not then return an error. If so then continue on.
@@ -145,7 +152,7 @@ static void _query(int sd, short args, void *cbdata)
                         goto done;
                     }
 
-                    PMIX_LOAD_NSPACE(jobid, q->qualifiers[n].value.data.string);
+                    PMIX_LOAD_NSPACE(jobid, jdata->nspace);
                     if (PMIX_NSPACE_INVALID(jobid)) {
                         ret = PMIX_ERR_BAD_PARAM;
                         goto done;
@@ -188,10 +195,6 @@ static void _query(int sd, short args, void *cbdata)
 
                 } else if (PMIX_CHECK_KEY(&q->qualifiers[n], PMIX_PSET_NAME)) {
                     psetname = q->qualifiers[n].value.data.string;
-
-                } else if (PMIX_CHECK_KEY(&q->qualifiers[n], PMIX_SESSION_ID)) {
-                    PMIX_VALUE_GET_NUMBER(rc, &q->qualifiers[n].value, sessionid, uint32_t);
-
                 }
 
             }
@@ -315,7 +318,10 @@ static void _query(int sd, short args, void *cbdata)
                 }
                 /* add our findings to the results */
                 PMIX_INFO_LIST_CONVERT(rc, cache, &dry);
-                if (PMIX_SUCCESS != rc) {
+                if (PMIX_SUCCESS != rc && PMIX_ERR_EMPTY != rc) {
+                    // if the array is empty, then there is nothing wrong - we
+                    // simply didn't find any runnning jobs
+                    // otherwise, report the error and abort
                     PMIX_ERROR_LOG(rc);
                     PMIX_INFO_LIST_RELEASE(cache);
                     goto done;
@@ -394,7 +400,7 @@ static void _query(int sd, short args, void *cbdata)
                 }
 
             } else if (0 == strcmp(q->keys[n], PMIX_HWLOC_XML_V2)) {
-                /* we cannot provide it if we are using v1.x */
+               /* we cannot provide it if we are using v1.x */
 #if HWLOC_API_VERSION >= 0x20000
                 if (NULL != prte_hwloc_topology) {
                     char *xmlbuffer = NULL;
@@ -594,18 +600,19 @@ static void _query(int sd, short args, void *cbdata)
                     PMIX_ARGV_APPEND_NOSIZE_COMPAT(&ans, ps->name);
                 }
                 if (NULL == ans) {
-                    ret = PMIX_ERR_NOT_FOUND;
-                    goto done;
+                    tmp = NULL;;
                 } else {
                     tmp = PMIX_ARGV_JOIN_COMPAT(ans, ',');
                     PMIX_ARGV_FREE_COMPAT(ans);
                     ans = NULL;
-                    PMIX_INFO_LIST_ADD(rc, results, PMIX_QUERY_PSET_NAMES, tmp, PMIX_STRING);
+                }
+                PMIX_INFO_LIST_ADD(rc, results, PMIX_QUERY_PSET_NAMES, tmp, PMIX_STRING);
+                if (NULL != tmp) {
                     free(tmp);
-                    if (PMIX_SUCCESS != rc) {
-                        PMIX_ERROR_LOG(rc);
-                        goto done;
-                    }
+                }
+                if (PMIX_SUCCESS != rc) {
+                    PMIX_ERROR_LOG(rc);
+                    goto done;
                 }
 
             } else if (0 == strcmp(q->keys[n], PMIX_QUERY_PSET_MEMBERSHIP)) {
@@ -702,9 +709,7 @@ static void _query(int sd, short args, void *cbdata)
                 /* cycle thru the job and create an entry for each proc */
                 PMIX_DATA_ARRAY_CONSTRUCT(&dry, grp->num_members, PMIX_PROC);
                 proc = (pmix_proc_t *) dry.array;
-                for (p = 0; p < grp->num_members; p++) {
-                    PMIX_LOAD_PROCID(&proc[p], grp->members[p].nspace, grp->members[p].rank);
-                }
+                memcpy(proc, grp->members, grp->num_members * sizeof(pmix_proc_t));
                 PMIX_INFO_LIST_ADD(rc, results, PMIX_QUERY_GROUP_MEMBERSHIP, &dry, PMIX_DATA_ARRAY);
                 PMIX_DATA_ARRAY_DESTRUCT(&dry);
                 if (PMIX_SUCCESS != rc) {
@@ -763,8 +768,8 @@ static void _query(int sd, short args, void *cbdata)
                     if (0 != hwloc_topology_export_xmlbuffer(topo->topo, &str, &len, 0)) {
                         continue;
                     }
-                    PMIX_INFO_LIST_ADD(rc, nodelist, PMIX_HWLOC_XML_V2, str, PMIX_STRING);
 #endif
+                    PMIX_INFO_LIST_ADD(rc, nodelist, PMIX_HWLOC_XML_V2, str, PMIX_STRING);
                     free(str);
                 }
                 /* convert list to array */
@@ -844,8 +849,132 @@ static void _query(int sd, short args, void *cbdata)
                     goto done;
                 }
 #endif
+
+#ifdef PMIX_QUERY_RESOLVE_PEERS
+            } else if (0 == strcmp(q->keys[n], PMIX_QUERY_RESOLVE_PEERS)) {
+                char *nm;
+                pmix_list_t procs;
+                int idx;
+                prte_proc_t *p2;
+                bool found;
+                // must at least have given us a hostname
+                if (NULL == hostname) {
+                    ret = PMIX_ERR_BAD_PARAM;
+                    goto done;
+                }
+                /* does the name refer to me? */
+                if (prte_check_host_is_local(hostname)) {
+                    nm = prte_process_info.nodename;
+                } else {
+                    nm = hostname;
+                }
+                PMIX_CONSTRUCT(&procs, pmix_list_t);
+                // could ask for info on all jobs, so have to check for that case
+                for (k = 1; k < prte_job_data->size; k++) {
+                    jdata = (prte_job_t *) pmix_pointer_array_get_item(prte_job_data, k);
+                    if (NULL == jdata) {
+                        continue;
+                    }
+                    if (NULL == jdata->map) {
+                        continue;
+                    }
+                    if (!PMIX_NSPACE_INVALID(jobid) &&
+                        !PMIX_CHECK_NSPACE(jobid, jdata->nspace)) {
+                        continue;
+                    }
+                    // see if this job has any procs on the indicated node
+                    for (j=0; j < jdata->map->nodes->size; j++) {
+                        node = (prte_node_t *) pmix_pointer_array_get_item(jdata->map->nodes, j);
+                        if (NULL == node) {
+                            continue;
+                        }
+                        if (0 != strcmp(node->name, nm)) {
+                            if (NULL == node->aliases) {
+                                continue;
+                            }
+                            found = false;
+                            for (p = 0; NULL != node->aliases[p]; p++) {
+                                if (0 == strcmp(nm, node->aliases[p])) {
+                                    /* this is the node! */
+                                    found = true;
+                                    break;
+                                }
+                            }
+                            if (!found) {
+                                continue;
+                            }
+                        }
+                        // we want the procs from this node
+                        for (idx=0; idx < node->procs->size; idx++) {
+                            proct = (prte_proc_t*)pmix_pointer_array_get_item(node->procs, idx);
+                            if (NULL == proct) {
+                                continue;
+                            }
+                            if (PMIX_CHECK_NSPACE(jdata->nspace, proct->name.nspace)) {
+                                pmix_list_append(&procs, &proct->super);
+                            }
+                        }
+                    }
+                }
+                sz = pmix_list_get_size(&procs);
+                if (0 < sz) {
+                    PMIX_PROC_CREATE(proc, sz);
+                    idx = 0;
+                    PMIX_LIST_FOREACH_SAFE(proct, p2, &procs, prte_proc_t) {
+                        memcpy(&proc[idx], &proct->name, sizeof(pmix_proc_t));
+                        pmix_list_remove_item(&procs, &proct->super);
+                        ++idx;
+                    }
+                } else {
+                    proc = NULL;
+                }
+                dry.type = PMIX_PROC;
+                dry.array = proc;
+                dry.size = sz;
+                PMIX_INFO_LIST_ADD(rc, results, PMIX_QUERY_RESOLVE_PEERS, &dry, PMIX_DATA_ARRAY);
+                if (NULL != proc) {
+                    free(proc);
+                }
+                PMIX_DESTRUCT(&procs);
+#endif
+
+#ifdef PMIX_QUERY_RESOLVE_NODE
+            } else if (0 == strcmp(q->keys[n], PMIX_QUERY_RESOLVE_NODE)) {
+                char **nodes = NULL, *nodelist;
+                // could ask for info on all jobs, so have to check for that case
+                for (k = 1; k < prte_job_data->size; k++) {
+                    jdata = (prte_job_t *) pmix_pointer_array_get_item(prte_job_data, k);
+                    if (NULL == jdata) {
+                        continue;
+                    }
+                    if (NULL == jdata->map) {
+                        continue;
+                    }
+                    if (!PMIX_NSPACE_INVALID(jobid) &&
+                        !PMIX_CHECK_NSPACE(jobid, jdata->nspace)) {
+                        continue;
+                    }
+                    // assemble the nodes
+                    for (j=0; j < jdata->map->nodes->size; j++) {
+                        node = (prte_node_t *) pmix_pointer_array_get_item(jdata->map->nodes, j);
+                        if (NULL == node) {
+                            continue;
+                        }
+                        PMIx_Argv_append_unique_nosize(&nodes, node->name);
+                    }
+                }
+                nodelist = PMIx_Argv_join(nodes, ',');
+                PMIx_Argv_free(nodes);
+                PMIX_INFO_LIST_ADD(rc, results, PMIX_QUERY_RESOLVE_NODE, nodelist, PMIX_STRING);
+#endif
+
             } else {
-                fprintf(stderr, "Query for unrecognized attribute: %s\n", q->keys[n]);
+                pmix_output_verbose(2, prte_pmix_server_globals.output,
+                                    "%s Query for unrecognized attribute: %s",
+                                    PRTE_NAME_PRINT(PRTE_PROC_MY_NAME),
+                                    PMIx_Get_attribute_name(q->keys[n]));
+                ret = PMIX_ERR_NOT_SUPPORTED;
+                goto done;
             }
         } // for
     }     // for
@@ -858,16 +987,18 @@ done:
         ret = rc;
     }
     PMIX_INFO_LIST_RELEASE(results);
-    if (PMIX_ERR_EMPTY == rc) {
-        ret = PMIX_ERR_NOT_FOUND;
-    } else if (PMIX_SUCCESS == ret) {
-        if (0 == dry.size) {
+    if (PMIX_ERR_NOT_SUPPORTED != ret) {
+        if (PMIX_ERR_EMPTY == rc) {
             ret = PMIX_ERR_NOT_FOUND;
-        } else {
-            if (dry.size < cd->ninfo) {
-                ret = PMIX_QUERY_PARTIAL_SUCCESS;
+        } else if (PMIX_SUCCESS == ret) {
+            if (0 == dry.size) {
+                ret = PMIX_ERR_NOT_FOUND;
             } else {
-                ret = PMIX_SUCCESS;
+                if (dry.size < cd->ninfo) {
+                    ret = PMIX_QUERY_PARTIAL_SUCCESS;
+                } else {
+                    ret = PMIX_SUCCESS;
+                }
             }
         }
     }
