@@ -19,7 +19,7 @@
  *                         and Technology (RIST).  All rights reserved.
  * Copyright (c) 2020      Geoffroy Vallee. All rights reserved.
  * Copyright (c) 2020      IBM Corporation.  All rights reserved.
- * Copyright (c) 2021-2024 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2025 Nanook Consulting  All rights reserved.
  * Copyright (c) 2021      Amazon.com, Inc. or its affiliates.  All Rights
  *                         reserved.
  * Copyright (c) 2022-2023 Triad National Security, LLC. All rights
@@ -250,18 +250,16 @@ static char *pmix_getline(FILE *fp)
 int main(int argc, char *argv[])
 {
     int rc = 1, i;
-    char *param, *timeoutenv, *tpath, *cptr;
+    char *param, *tpath, *cptr;
     prte_pmix_lock_t lock;
-    pmix_list_t apps;
+    pmix_list_t apps, jobdata;
     prte_pmix_app_t *app;
     pmix_info_t *iptr, *iptr2, info;
     pmix_status_t ret;
-    bool flag;
     size_t n, ninfo, param_len;
     pmix_app_t *papps;
     size_t napps;
     mylock_t mylock;
-    uint32_t ui32;
     char **pargv, **split;
     int pargc;
     prte_job_t *jdata;
@@ -280,9 +278,11 @@ int main(int argc, char *argv[])
     pmix_cli_result_t results;
     pmix_cli_item_t *opt;
     FILE *fp;
+    prte_info_item_t *iprteinfo;
 
     /* init the globals */
     PMIX_CONSTRUCT(&apps, pmix_list_t);
+    PMIX_CONSTRUCT(&jobdata, pmix_list_t);
     if (NULL == (param = getenv("PRTE_BASENAME"))) {
         prte_tool_basename = pmix_basename(argv[0]);
     } else {
@@ -582,11 +582,18 @@ int main(int argc, char *argv[])
         prte_leave_session_attached = true;
     }
 
+    // check for hetero nodes
+    if (pmix_cmd_line_is_taken(&results, PRTE_CLI_HETERO_NODES)) {
+        prte_hetero_nodes = true;
+    }
+
     /* detach from controlling terminal
      * otherwise, remain attached so output can get to us
      */
     if (pmix_cmd_line_is_taken(&results, PRTE_CLI_DAEMONIZE)) {
-        pipe(wait_pipe);
+        if (0 > pipe(wait_pipe)) {
+            return PRTE_ERROR;
+        }
         prte_state_base.parent_fd = wait_pipe[1];
         prte_daemon_init_callback(NULL, wait_dvm);
         close(wait_pipe[0]);
@@ -635,7 +642,7 @@ int main(int argc, char *argv[])
     if (!pmix_cmd_line_is_taken(&results, PRTE_CLI_DAEMONIZE)) {
         /* see if they want to run an application - let's parse
          * the cmd line to get it */
-        rc = prte_parse_locals(schizo, &apps, pargv, &hostfiles, &hosts);
+        rc = prte_parse_locals(schizo, &apps, pargv, &hostfiles, &hosts, &jobdata);
         // not-found => no app given
         if (PRTE_SUCCESS != rc && PRTE_ERR_NOT_FOUND != rc) {
             PRTE_UPDATE_EXIT_STATUS(rc);
@@ -756,13 +763,37 @@ int main(int argc, char *argv[])
         goto DONE;
     }
 
-    /* Did the user specify a prefix, or want prefix by default? */
-    opt = pmix_cmd_line_get_param(&results, PRTE_CLI_PREFIX);
-    if (NULL != opt || want_prefix_by_default) {
-        if (NULL != opt) {
-            param = strdup(opt->values[0]);
-        } else {
-            /* --enable-prun-prefix-default was given to prun */
+    /* check if we were given multiple prefixes on the cmd line,
+     * and if so, are any of them different from each other. The
+     * app parser puts every prefix it finds, regardless of app_context,
+     * onto the jobdata list */
+    param = NULL;
+    PMIX_LIST_FOREACH(iprteinfo, &jobdata, prte_info_item_t) {
+        if (PMIx_Check_key(iprteinfo->info.key, "PRTE_PREFIX")) {
+            if (NULL == param) {
+                param = strdup(iprteinfo->info.value.data.string);
+            } else if (0 != strcmp(param, iprteinfo->info.value.data.string)) {
+                    // we have non-matching prefixes
+                    pmix_show_help("help-plm-base.txt", "multiple-prefixes", true,
+                                   prte_tool_basename, PRTE_CLI_PREFIX,
+                                   PRTE_CLI_PREFIX, "PRRTE", PRTE_CLI_PREFIX,
+                                   param, iprteinfo->info.value.data.string);
+                    PRTE_UPDATE_EXIT_STATUS(PRTE_ERR_FATAL);
+                    goto DONE;
+            }
+        }
+    }
+
+    /* Did the user specify a prefix, or want prefix by default? We
+     * already checked for any prefixes on the cmd line, so we can
+     * just use that result */
+    if (NULL != param || want_prefix_by_default) {
+        if (NULL == param) {
+            /* --enable-prte-prefix-default was given at time of configure.
+             * NOTE: if the PRTE_PREFIX envar was set, then the installdirs
+             * framework would have picked that value up during init and
+             * populated the prte_install_dirs struct accordingly, so we
+             * do not need to explicitly check the environment here */
             param = strdup(prte_install_dirs.prefix);
         }
         /* "Parse" the param, aka remove superfluous path_sep. */
@@ -778,9 +809,24 @@ int main(int argc, char *argv[])
                 break;
             }
         }
-        prte_set_attribute(&dapp->attributes, PRTE_APP_PREFIX_DIR, PRTE_ATTR_GLOBAL,
-                           param, PMIX_STRING);
-        free(param);
+    } else if (NULL != (cptr = getenv("PRTE_PREFIX"))) {
+        /* need to cover the case where "want prefix by default" is not
+         * given, but PRTE_PREFIX was added to the environment prior
+         * to actually invoking prte */
+        param = strdup(cptr);
+        /* "Parse" the param, aka remove superfluous path_sep. */
+        param_len = strlen(param);
+        while (0 == strcmp(PRTE_PATH_SEP, &(param[param_len - 1]))) {
+            param[param_len - 1] = '\0';
+            param_len--;
+            if (0 == param_len) {
+                /* We get here if we removed all PATH_SEP's and end up
+                   with an empty string.  In this case, the prefix is
+                   just a single PATH_SEP. */
+                strncpy(param, PRTE_PATH_SEP, sizeof(param) - 1);
+                break;
+            }
+        }
     } else {
         /* Check if called with fully-qualified path to prte.
            (Note: Put this second so can override with --prefix (above). */
@@ -806,11 +852,103 @@ int main(int argc, char *argv[])
                 free(tmp_basename);
             }
             if (NULL != tpath) {
-                prte_set_attribute(&dapp->attributes, PRTE_APP_PREFIX_DIR, PRTE_ATTR_GLOBAL,
-                                   tpath, PMIX_STRING);
+                param = tpath;
             }
         }
     }
+    if (NULL != param) {
+        // add the directive to the daemon job object
+        if (prte_get_attribute(&jdata->attributes, PRTE_JOB_PREFIX, (void **) &cptr, PMIX_STRING)) {
+            // already have a prefix directory entry - see if they are the same
+            if (0 != strcmp(cptr, param)) {
+                pmix_show_help("help-plm-base.txt", "multiple-prrte-prefixes", true,
+                               prte_tool_basename, prte_tool_basename,
+                               prte_tool_basename, param, cptr);
+                free(param);
+                free(cptr);
+                PRTE_UPDATE_EXIT_STATUS(PRTE_ERR_FATAL);
+                goto DONE;
+            }
+            free(cptr);
+        }
+        prte_set_attribute(&jdata->attributes, PRTE_JOB_PREFIX, PRTE_ATTR_GLOBAL,
+                           param, PMIX_STRING);
+        free(param);
+    }
+
+    /* do the same for PMIx prefixes for the daemons. Note that we don't
+     * have a prefix-by-default setting here because that only applies
+     * to PRRTE itself. If the PMIx library used by PRRTE was configured
+     * with that option, we have no way of knowing about it, so there is
+     * nothing we can do. Likewise, the use of an absolute path for PRRTE
+     * doesn't say anything about the location of the PMIx library */
+    param = NULL;
+    PMIX_LIST_FOREACH(iprteinfo, &jobdata, prte_info_item_t) {
+        if (PMIx_Check_key(iprteinfo->info.key, PMIX_PREFIX)) {
+            if (NULL == param) {
+                param = strdup(iprteinfo->info.value.data.string);
+            } else if (0 != strcmp(param, iprteinfo->info.value.data.string)) {
+                    // we have non-matching prefixes
+                    pmix_show_help("help-plm-base.txt", "multiple-prefixes", true,
+                                   prte_tool_basename, PRTE_CLI_PMIX_PREFIX,
+                                   PRTE_CLI_PMIX_PREFIX, "PMIx", PRTE_CLI_PMIX_PREFIX,
+                                   param, iprteinfo->info.value.data.string);
+                    PRTE_UPDATE_EXIT_STATUS(PRTE_ERR_FATAL);
+                    goto DONE;
+            }
+        }
+    }
+    /* Did the user specify a prefix? We checked for any prefixes on the
+     * cmd line above, so we can just use that result */
+    if (NULL != param) {
+        /* "Parse" the param, aka remove superfluous path_sep. */
+        param_len = strlen(param);
+        while (0 == strcmp(PRTE_PATH_SEP, &(param[param_len - 1]))) {
+            param[param_len - 1] = '\0';
+            param_len--;
+            if (0 == param_len) {
+                /* We get here if we removed all PATH_SEP's and end up
+                   with an empty string.  In this case, the prefix is
+                   just a single PATH_SEP. */
+                strncpy(param, PRTE_PATH_SEP, sizeof(param) - 1);
+                break;
+            }
+        }
+    } else if (NULL != (cptr = getenv("PMIX_PREFIX"))) {
+        param = strdup(cptr);
+        /* "Parse" the param, aka remove superfluous path_sep. */
+        param_len = strlen(param);
+        while (0 == strcmp(PRTE_PATH_SEP, &(param[param_len - 1]))) {
+            param[param_len - 1] = '\0';
+            param_len--;
+            if (0 == param_len) {
+                /* We get here if we removed all PATH_SEP's and end up
+                   with an empty string.  In this case, the prefix is
+                   just a single PATH_SEP. */
+                strncpy(param, PRTE_PATH_SEP, sizeof(param) - 1);
+                break;
+            }
+        }
+    }
+    if (NULL != param) {
+        // add the directive to the daemon job object
+        if (prte_get_attribute(&jdata->attributes, PRTE_JOB_PMIX_PREFIX, (void **) &cptr, PMIX_STRING)) {
+            // already have a prefix directory entry - see if they are the same
+            if (0 != strcmp(cptr, param)) {
+                pmix_show_help("help-plm-base.txt", "multiple-pmix-prefixes", true,
+                               prte_tool_basename, prte_tool_basename,
+                               param, cptr);
+                free(param);
+                free(cptr);
+                PRTE_UPDATE_EXIT_STATUS(PRTE_ERR_FATAL);
+                goto DONE;
+            }
+            free(cptr);
+        }
+        prte_set_attribute(&jdata->attributes, PRTE_JOB_PMIX_PREFIX, PRTE_ATTR_GLOBAL,
+                           param, PMIX_STRING);
+        free(param);
+    }
 
     /* apply any provided runtime options to the DVM itself */
     opt = pmix_cmd_line_get_param(&results, PRTE_CLI_RTOS);
@@ -1004,136 +1142,11 @@ int main(int argc, char *argv[])
         PMIX_VALUE_RELEASE(val);
     }
 
-    /* pass the personality */
-    PMIX_INFO_LIST_ADD(ret, jinfo, PMIX_PERSONALITY, personality, PMIX_STRING);
-
-    /* get display options */
-    opt = pmix_cmd_line_get_param(&results, PRTE_CLI_DISPLAY);
-    if (NULL != opt) {
-        ret = prte_schizo_base_parse_display(opt, jinfo);
-        if (PRTE_SUCCESS != ret) {
-            PRTE_UPDATE_EXIT_STATUS(PRTE_ERR_FATAL);
-            goto DONE;
-        }
+    ret = prte_prun_parse_common_cli(jinfo, &results, schizo, &apps);
+    if (PRTE_SUCCESS != ret) {
+        goto DONE;
     }
 
-    /* get output options */
-    opt = pmix_cmd_line_get_param(&results, PRTE_CLI_OUTPUT);
-    if (NULL != opt) {
-        ret = prte_schizo_base_parse_output(opt, jinfo);
-        if (PRTE_SUCCESS != ret) {
-            PRTE_UPDATE_EXIT_STATUS(PRTE_ERR_FATAL);
-            goto DONE;
-        }
-    }
-
-    /* check for runtime options */
-    opt = pmix_cmd_line_get_param(&results, PRTE_CLI_RTOS);
-    if (NULL != opt) {
-        PMIX_INFO_LIST_ADD(ret, jinfo, PMIX_RUNTIME_OPTIONS, opt->values[0], PMIX_STRING);
-    }
-
-    /* check what user wants us to do with stdin */
-    opt = pmix_cmd_line_get_param(&results, PRTE_CLI_STDIN);
-    if (NULL != opt) {
-        PMIX_INFO_LIST_ADD(ret, jinfo, PMIX_STDIN_TGT, opt->values[0], PMIX_STRING);
-    }
-
-    opt = pmix_cmd_line_get_param(&results, PRTE_CLI_MAPBY);
-    if (NULL != opt) {
-        PMIX_INFO_LIST_ADD(ret, jinfo, PMIX_MAPBY, opt->values[0], PMIX_STRING);
-    }
-
-    /* if the user specified a ranking policy, then set it */
-    opt = pmix_cmd_line_get_param(&results, PRTE_CLI_RANKBY);
-    if (NULL != opt) {
-        PMIX_INFO_LIST_ADD(ret, jinfo, PMIX_RANKBY, opt->values[0], PMIX_STRING);
-    }
-
-    /* if the user specified a binding policy, then set it */
-    opt = pmix_cmd_line_get_param(&results, PRTE_CLI_BINDTO);
-    if (NULL != opt) {
-        PMIX_INFO_LIST_ADD(ret, jinfo, PMIX_BINDTO, opt->values[0], PMIX_STRING);
-    }
-
-    /* check for an exec agent */
-   opt = pmix_cmd_line_get_param(&results, PRTE_CLI_EXEC_AGENT);
-    if (NULL != opt) {
-        PMIX_INFO_LIST_ADD(ret, jinfo, PMIX_EXEC_AGENT, opt->values[0], PMIX_STRING);
-    }
-
-    /* mark if recovery was enabled */
-    if (pmix_cmd_line_is_taken(&results, PRTE_CLI_ENABLE_RECOVERY)) {
-        PMIX_INFO_LIST_ADD(ret, jinfo, PMIX_JOB_RECOVERABLE, NULL, PMIX_BOOL);
-    }
-    /* record the max restarts */
-    opt = pmix_cmd_line_get_param(&results, PRTE_CLI_MAX_RESTARTS);
-    if (NULL != opt) {
-        ui32 = strtol(opt->values[0], NULL, 10);
-        PMIX_LIST_FOREACH(app, &apps, prte_pmix_app_t)
-        {
-            PMIX_INFO_LIST_ADD(ret, app->info, PMIX_MAX_RESTARTS, &ui32, PMIX_UINT32);
-        }
-    }
-    /* if continuous operation was specified */
-    if (pmix_cmd_line_is_taken(&results, PRTE_CLI_CONTINUOUS)) {
-        /* mark this job as continuously operating */
-        PMIX_INFO_LIST_ADD(ret, jinfo, PMIX_JOB_CONTINUOUS, NULL, PMIX_BOOL);
-    }
-#ifdef PMIX_ABORT_NONZERO_EXIT
-    /* if ignore non-zero exit was specified */
-    if (pmix_cmd_line_is_taken(&results, PRTE_CLI_TERM_NONZERO)) {
-        PMIX_INFO_LIST_ADD(ret, jinfo, PMIX_ABORT_NONZERO_EXIT, NULL, PMIX_BOOL);
-    }
-#endif
-    /* if stop-on-exec was specified */
-    if (pmix_cmd_line_is_taken(&results, PRTE_CLI_STOP_ON_EXEC)) {
-        PMIX_INFO_LIST_ADD(ret, jinfo, PMIX_DEBUG_STOP_ON_EXEC, NULL, PMIX_BOOL);
-    }
-
-    /* check for a job timeout specification, to be provided in seconds
-     * as that is what MPICH used
-     */
-    timeoutenv = NULL;
-    opt = pmix_cmd_line_get_param(&results, PRTE_CLI_TIMEOUT);
-    if (NULL != opt || NULL != (timeoutenv = getenv("MPIEXEC_TIMEOUT"))) {
-        if (NULL != timeoutenv) {
-            i = strtol(timeoutenv, NULL, 10);
-            /* both cannot be present, or they must agree */
-            if (NULL != opt) {
-                n = strtol(opt->values[0], NULL, 10);
-                if (i != (int)n) {
-                    pmix_show_help("help-prun.txt", "prun:timeoutconflict", false,
-                                   prte_tool_basename, n, timeoutenv);
-                    PRTE_UPDATE_EXIT_STATUS(1);
-                    goto DONE;
-                }
-            }
-        } else {
-            i = strtol(opt->values[0], NULL, 10);
-        }
-        PMIX_INFO_LIST_ADD(ret, jinfo, PMIX_JOB_TIMEOUT, &i, PMIX_INT);
-    }
-    if (pmix_cmd_line_is_taken(&results, PRTE_CLI_STACK_TRACES)) {
-        PMIX_INFO_LIST_ADD(ret, jinfo, PMIX_TIMEOUT_STACKTRACES, NULL, PMIX_BOOL);
-    }
-    if (pmix_cmd_line_is_taken(&results, PRTE_CLI_REPORT_STATE)) {
-        PMIX_INFO_LIST_ADD(ret, jinfo, PMIX_TIMEOUT_REPORT_STATE, NULL, PMIX_BOOL);
-    }
-    opt = pmix_cmd_line_get_param(&results, PRTE_CLI_SPAWN_TIMEOUT);
-    if (NULL != opt) {
-        i = strtol(opt->values[0], NULL, 10);
-        PMIX_INFO_LIST_ADD(ret, jinfo, PMIX_SPAWN_TIMEOUT, &i, PMIX_INT);
-    }
-    opt = pmix_cmd_line_get_param(&results, PRTE_CLI_DO_NOT_AGG_HELP);
-    if (NULL != opt) {
-        flag = false;
-        PMIX_INFO_LIST_ADD(ret, jinfo, PMIX_LOG_AGG, &flag, PMIX_BOOL);
-    }
-
-    /* give the schizo components a chance to add to the job info */
-    schizo->job_info(&results, jinfo);
-
     /* convert the job info into an array */
     PMIX_INFO_LIST_CONVERT(ret, jinfo, &darray);
     if (PMIX_ERR_EMPTY == ret) {
@@ -1408,7 +1421,7 @@ static int prep_singleton(const char *name)
     app->app = strdup(jdata->nspace);
     app->num_procs = 1;
     PMIX_ARGV_APPEND_NOSIZE_COMPAT(&app->argv, app->app);
-    getcwd(cwd, sizeof(cwd));
+    pmix_getcwd(cwd, sizeof(cwd));
     app->cwd = strdup(cwd);
     pmix_pointer_array_set_item(jdata->apps, 0, app);
     jdata->num_apps = 1;
