@@ -9,7 +9,7 @@
  *                         reserved.
  * Copyright (c) 2015-2018 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
- * Copyright (c) 2016      FUJITSU LIMITED.  All rights reserved.
+ * Copyright (c) 2016-2018 FUJITSU LIMITED.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -29,6 +29,7 @@
 #define OMPI_DATATYPE_INTERNAL_H
 
 #include "opal/datatype/opal_datatype_internal.h"
+#include "ompi/datatype/ompi_datatype.h"
 
 /*
  * This is the OMPI-layered numbering of ALL supported MPI types
@@ -46,67 +47,76 @@
 #define OMPI_DATATYPE_MPI_FLOAT                   0x09
 #define OMPI_DATATYPE_MPI_DOUBLE                  0x0A
 #define OMPI_DATATYPE_MPI_LONG_DOUBLE             0x0B
-#define OMPI_DATATYPE_MPI_COMPLEX8                0x0C
-#define OMPI_DATATYPE_MPI_COMPLEX16               0x0D
-#define OMPI_DATATYPE_MPI_COMPLEX32               0x0E
-#define OMPI_DATATYPE_MPI_WCHAR                   0x0F
-#define OMPI_DATATYPE_MPI_PACKED                  0x10
+#define OMPI_DATATYPE_MPI_COMPLEX4                0x0C
+#define OMPI_DATATYPE_MPI_COMPLEX8                0x0D
+#define OMPI_DATATYPE_MPI_COMPLEX16               0x0E
+#define OMPI_DATATYPE_MPI_COMPLEX32               0x0F
+#define OMPI_DATATYPE_MPI_WCHAR                   0x10
+#define OMPI_DATATYPE_MPI_PACKED                  0x11
 
-#define OMPI_DATATYPE_MPI_BOOL                    0x11
+#define OMPI_DATATYPE_MPI_BOOL                    0x12
 
-#define OMPI_DATATYPE_MPI_LOGICAL                 0x12
-#define OMPI_DATATYPE_MPI_CHARACTER               0x13
-#define OMPI_DATATYPE_MPI_INTEGER                 0x14
-#define OMPI_DATATYPE_MPI_REAL                    0x15
-#define OMPI_DATATYPE_MPI_DOUBLE_PRECISION        0x16
+#define OMPI_DATATYPE_MPI_LOGICAL                 0x13
+#define OMPI_DATATYPE_MPI_CHARACTER               0x14
+#define OMPI_DATATYPE_MPI_INTEGER                 0x15
+#define OMPI_DATATYPE_MPI_REAL                    0x16
+#define OMPI_DATATYPE_MPI_DOUBLE_PRECISION        0x17
 
 /*
- * Derived datatypes supposely contiguous
+ * Derived datatypes supposedly contiguous
  */
-#define OMPI_DATATYPE_MPI_COMPLEX                 0x17   /* Was COMPLEX_FLOAT */
-#define OMPI_DATATYPE_MPI_DOUBLE_COMPLEX          0x18   /* Was COMPLEX_DOUBLE */
-#define OMPI_DATATYPE_MPI_LONG_DOUBLE_COMPLEX     0x19   /* Was COMPLEX_LONG_DOUBLE */
-#define OMPI_DATATYPE_MPI_2INT                    0x1A
-#define OMPI_DATATYPE_MPI_2INTEGER                0x1B
-#define OMPI_DATATYPE_MPI_2REAL                   0x1C
-#define OMPI_DATATYPE_MPI_2DBLPREC                0x1D
-#define OMPI_DATATYPE_MPI_2COMPLEX                0x1E
-#define OMPI_DATATYPE_MPI_2DOUBLE_COMPLEX         0x1F
+#define OMPI_DATATYPE_MPI_COMPLEX                 0x18   /* Was COMPLEX_FLOAT */
+#define OMPI_DATATYPE_MPI_DOUBLE_COMPLEX          0x19   /* Was COMPLEX_DOUBLE */
+#define OMPI_DATATYPE_MPI_LONG_DOUBLE_COMPLEX     0x1A   /* Was COMPLEX_LONG_DOUBLE */
+#define OMPI_DATATYPE_MPI_2INT                    0x1B
+#define OMPI_DATATYPE_MPI_2INTEGER                0x1C
+#define OMPI_DATATYPE_MPI_2REAL                   0x1D
+#define OMPI_DATATYPE_MPI_2DBLPREC                0x1E
+#define OMPI_DATATYPE_MPI_2COMPLEX                0x1F
+#define OMPI_DATATYPE_MPI_2DOUBLE_COMPLEX         0x20
 /*
  * Derived datatypes which will definitively be non contiguous on some architectures.
  */
-#define OMPI_DATATYPE_MPI_FLOAT_INT               0x20
-#define OMPI_DATATYPE_MPI_DOUBLE_INT              0x21
-#define OMPI_DATATYPE_MPI_LONG_DOUBLE_INT         0x22
-#define OMPI_DATATYPE_MPI_LONG_INT                0x23
-#define OMPI_DATATYPE_MPI_SHORT_INT               0x24
+#define OMPI_DATATYPE_MPI_FLOAT_INT               0x21
+#define OMPI_DATATYPE_MPI_DOUBLE_INT              0x22
+#define OMPI_DATATYPE_MPI_LONG_DOUBLE_INT         0x23
+#define OMPI_DATATYPE_MPI_LONG_INT                0x24
+#define OMPI_DATATYPE_MPI_SHORT_INT               0x25
 /*
  * Datatypes from the MPI 2.2 standard
  */
-#define OMPI_DATATYPE_MPI_AINT                    0x25
-#define OMPI_DATATYPE_MPI_OFFSET                  0x26
-#define OMPI_DATATYPE_MPI_C_BOOL                  0x27
-#define OMPI_DATATYPE_MPI_C_COMPLEX               0x28
-#define OMPI_DATATYPE_MPI_C_FLOAT_COMPLEX         0x29
-#define OMPI_DATATYPE_MPI_C_DOUBLE_COMPLEX        0x2A
-#define OMPI_DATATYPE_MPI_C_LONG_DOUBLE_COMPLEX   0x2B
+#define OMPI_DATATYPE_MPI_AINT                    0x26
+#define OMPI_DATATYPE_MPI_OFFSET                  0x27
+#define OMPI_DATATYPE_MPI_C_BOOL                  0x28
+#define OMPI_DATATYPE_MPI_C_COMPLEX               0x29
+#define OMPI_DATATYPE_MPI_C_FLOAT_COMPLEX         0x2A
+#define OMPI_DATATYPE_MPI_C_DOUBLE_COMPLEX        0x2B
+#define OMPI_DATATYPE_MPI_C_LONG_DOUBLE_COMPLEX   0x2C
 
-#define OMPI_DATATYPE_MPI_LB                      0x2C
-#define OMPI_DATATYPE_MPI_UB                      0x2D
+#define OMPI_DATATYPE_MPI_LB                      0x2D
+#define OMPI_DATATYPE_MPI_UB                      0x2E
 
 /*
  * Datatypes from the MPI 3.0 standard
  */
-#define OMPI_DATATYPE_MPI_COUNT                   0x2E
+#define OMPI_DATATYPE_MPI_COUNT                   0x2F
+
+/*
+ * Datatypes proposed to the MPI Forum in June 2017 for proposal in
+ * the MPI 4.0 standard. As of February 2019, it is not accepted yet.
+ * See https://github.com/mpi-forum/mpi-issues/issues/65
+ */
+#define OMPI_DATATYPE_MPI_SHORT_FLOAT             0x30
+#define OMPI_DATATYPE_MPI_C_SHORT_FLOAT_COMPLEX   0x31
 
 /*
  * Datatypes that have a different external32 length.
  */
-#define OMPI_DATATYPE_MPI_LONG                    0x2F
-#define OMPI_DATATYPE_MPI_UNSIGNED_LONG           0x30
+#define OMPI_DATATYPE_MPI_LONG                    0x32
+#define OMPI_DATATYPE_MPI_UNSIGNED_LONG           0x33
 
 /* This should __ALWAYS__ stay last  */
-#define OMPI_DATATYPE_MPI_UNAVAILABLE             0x31
+#define OMPI_DATATYPE_MPI_UNAVAILABLE             0x34
 
 
 #define OMPI_DATATYPE_MPI_MAX_PREDEFINED          (OMPI_DATATYPE_MPI_UNAVAILABLE+1)
@@ -123,27 +133,14 @@
  * of being redefined as independent types, they will be made synonyms to
  * the most basic type.
  */
-#if SIZEOF_CHAR == 1
+#if SIZEOF_CHAR != 1
+#error Open MPI asserts that sizeof(char) is 1. This seems not to be the case here. Please report on Open MPI github (see issue #11815)
+#endif
+
 #define OMPI_DATATYPE_MPI_CHAR                    OMPI_DATATYPE_MPI_INT8_T
 #define OMPI_DATATYPE_MPI_SIGNED_CHAR             OMPI_DATATYPE_MPI_INT8_T
 #define OMPI_DATATYPE_MPI_UNSIGNED_CHAR           OMPI_DATATYPE_MPI_UINT8_T
 #define OMPI_DATATYPE_MPI_BYTE                    OMPI_DATATYPE_MPI_UINT8_T
-#elif SIZEOF_CHAR == 2
-#define OMPI_DATATYPE_MPI_CHAR                    OMPI_DATATYPE_MPI_INT16_T
-#define OMPI_DATATYPE_MPI_SIGNED_CHAR             OMPI_DATATYPE_MPI_INT16_T
-#define OMPI_DATATYPE_MPI_UNSIGNED_CHAR           OMPI_DATATYPE_MPI_UINT16_T
-#define OMPI_DATATYPE_MPI_BYTE                    OMPI_DATATYPE_MPI_UINT16_T
-#elif SIZEOF_CHAR == 4
-#define OMPI_DATATYPE_MPI_CHAR                    OMPI_DATATYPE_MPI_INT32_T
-#define OMPI_DATATYPE_MPI_SIGNED_CHAR             OMPI_DATATYPE_MPI_INT32_T
-#define OMPI_DATATYPE_MPI_UNSIGNED_CHAR           OMPI_DATATYPE_MPI_UINT32_T
-#define OMPI_DATATYPE_MPI_BYTE                    OMPI_DATATYPE_MPI_UINT32_T
-#elif SIZEOF_CHAR == 8
-#define OMPI_DATATYPE_MPI_CHAR                    OMPI_DATATYPE_MPI_INT64_T
-#define OMPI_DATATYPE_MPI_SIGNED_CHAR             OMPI_DATATYPE_MPI_INT64_T
-#define OMPI_DATATYPE_MPI_UNSIGNED_CHAR           OMPI_DATATYPE_MPI_UINT64_T
-#define OMPI_DATATYPE_MPI_BYTE                    OMPI_DATATYPE_MPI_UINT64_T
-#endif
 
 #if SIZEOF_SHORT == 1
 #define OMPI_DATATYPE_MPI_SHORT                   OMPI_DATATYPE_MPI_INT8_T
@@ -324,6 +321,9 @@
 /* REAL */
 #if OMPI_SIZEOF_FORTRAN_REAL2 == OMPI_SIZEOF_FORTRAN_REAL
 #  define OMPI_DATATYPE_MPI_REAL2                 OMPI_DATATYPE_MPI_REAL
+#elif (defined(HAVE_SHORT_FLOAT) && OMPI_SIZEOF_FORTRAN_REAL2 == SIZEOF_SHORT_FLOAT) || \
+      (defined(HAVE_OPAL_SHORT_FLOAT_T) && OMPI_SIZEOF_FORTRAN_REAL2 == SIZEOF_OPAL_SHORT_FLOAT_T)
+#  define OMPI_DATATYPE_MPI_REAL2                 OMPI_DATATYPE_MPI_SHORT_FLOAT
 #elif OMPI_SIZEOF_FORTRAN_REAL2 == SIZEOF_FLOAT
 #  define OMPI_DATATYPE_MPI_REAL2                 OMPI_DATATYPE_MPI_FLOAT
 #elif OMPI_SIZEOF_FORTRAN_REAL2 == SIZEOF_DOUBLE
@@ -336,6 +336,9 @@
 
 #if OMPI_SIZEOF_FORTRAN_REAL4 == OMPI_SIZEOF_FORTRAN_REAL
 #  define OMPI_DATATYPE_MPI_REAL4                 OMPI_DATATYPE_MPI_REAL
+#elif (defined(HAVE_SHORT_FLOAT) && OMPI_SIZEOF_FORTRAN_REAL4 == SIZEOF_SHORT_FLOAT) || \
+      (defined(HAVE_OPAL_SHORT_FLOAT_T) && OMPI_SIZEOF_FORTRAN_REAL4 == SIZEOF_OPAL_SHORT_FLOAT_T)
+#  define OMPI_DATATYPE_MPI_REAL4                 OMPI_DATATYPE_MPI_SHORT_FLOAT
 #elif OMPI_SIZEOF_FORTRAN_REAL4 == SIZEOF_FLOAT
 #  define OMPI_DATATYPE_MPI_REAL4                 OMPI_DATATYPE_MPI_FLOAT
 #elif OMPI_SIZEOF_FORTRAN_REAL4 == SIZEOF_DOUBLE
@@ -348,6 +351,9 @@
 
 #if OMPI_SIZEOF_FORTRAN_REAL8 == OMPI_SIZEOF_FORTRAN_REAL
 #  define OMPI_DATATYPE_MPI_REAL8                 OMPI_DATATYPE_MPI_REAL
+#elif (defined(HAVE_SHORT_FLOAT) && OMPI_SIZEOF_FORTRAN_REAL8 == SIZEOF_SHORT_FLOAT) || \
+      (defined(HAVE_OPAL_SHORT_FLOAT_T) && OMPI_SIZEOF_FORTRAN_REAL8 == SIZEOF_OPAL_SHORT_FLOAT_T)
+#  define OMPI_DATATYPE_MPI_REAL8                 OMPI_DATATYPE_MPI_SHORT_FLOAT
 #elif OMPI_SIZEOF_FORTRAN_REAL8 == SIZEOF_FLOAT
 #  define OMPI_DATATYPE_MPI_REAL8                 OMPI_DATATYPE_MPI_FLOAT
 #elif OMPI_SIZEOF_FORTRAN_REAL8 == SIZEOF_DOUBLE
@@ -360,6 +366,9 @@
 
 #if OMPI_SIZEOF_FORTRAN_REAL16 == OMPI_SIZEOF_FORTRAN_REAL
 #  define OMPI_DATATYPE_MPI_REAL16                OMPI_DATATYPE_MPI_REAL
+#elif (defined(HAVE_SHORT_FLOAT) && OMPI_SIZEOF_FORTRAN_REAL16 == SIZEOF_SHORT_FLOAT) || \
+      (defined(HAVE_OPAL_SHORT_FLOAT_T) && OMPI_SIZEOF_FORTRAN_REAL16 == SIZEOF_OPAL_SHORT_FLOAT_T)
+#  define OMPI_DATATYPE_MPI_REAL16                 OMPI_DATATYPE_MPI_SHORT_FLOAT
 #elif OMPI_SIZEOF_FORTRAN_REAL16 == SIZEOF_FLOAT
 #  define OMPI_DATATYPE_MPI_REAL16                OMPI_DATATYPE_MPI_FLOAT
 #elif OMPI_SIZEOF_FORTRAN_REAL16 == SIZEOF_DOUBLE
@@ -374,6 +383,7 @@
  * C++ datatypes, these map to C datatypes.
  */
 #define OMPI_DATATYPE_MPI_CXX_BOOL                OMPI_DATATYPE_MPI_C_BOOL
+#define OMPI_DATATYPE_MPI_CXX_SHORT_FLOAT_COMPLEX OMPI_DATATYPE_MPI_C_SHORT_FLOAT_COMPLEX
 #define OMPI_DATATYPE_MPI_CXX_FLOAT_COMPLEX       OMPI_DATATYPE_MPI_C_FLOAT_COMPLEX
 #define OMPI_DATATYPE_MPI_CXX_DOUBLE_COMPLEX      OMPI_DATATYPE_MPI_C_DOUBLE_COMPLEX
 #define OMPI_DATATYPE_MPI_CXX_LONG_DOUBLE_COMPLEX OMPI_DATATYPE_MPI_C_LONG_DOUBLE_COMPLEX
@@ -398,7 +408,7 @@ extern const ompi_datatype_t* ompi_datatype_basicDatatypes[OMPI_DATATYPE_MPI_MAX
     .d_f_to_c_index = -1,                                                            \
     .d_keyhash = NULL,                                                               \
     .args = NULL,                                                                    \
-    .packed_description = NULL,                                                      \
+    .packed_description = 0,                                                         \
     .name = "MPI_" # NAME
 
 #define OMPI_DATATYPE_INITIALIZER_UNAVAILABLE(FLAGS)                                 \
@@ -430,9 +440,9 @@ extern const ompi_datatype_t* ompi_datatype_basicDatatypes[OMPI_DATATYPE_MPI_MAX
     OMPI_DATATYPE_INIT_PREDEFINED_BASIC_TYPE( UNAVAILABLE, NAME, FLAGS )
 
 /*
- * Initilization for these types is deferred until runtime.
+ * Initialization for these types is deferred until runtime.
  *
- * Using this macro implies that at this point not all informations needed
+ * Using this macro implies that at this point not all information needed
  * to fill up the datatype are known. We fill them with zeros and then later
  * when the datatype engine will be initialized we complete with the
  * correct information. This macro should be used for all composed types.
@@ -496,27 +506,10 @@ extern const ompi_datatype_t* ompi_datatype_basicDatatypes[OMPI_DATATYPE_MPI_MAX
 #define OMPI_DATATYPE_INITIALIZER_INT64_T             OPAL_DATATYPE_INITIALIZER_INT8
 #define OMPI_DATATYPE_INITIALIZER_UINT64_T            OPAL_DATATYPE_INITIALIZER_UINT8
 
-#if SIZEOF_CHAR == 1
 #define OMPI_DATATYPE_INITIALIZER_CHAR                OPAL_DATATYPE_INITIALIZER_INT1
 #define OMPI_DATATYPE_INITIALIZER_UNSIGNED_CHAR       OPAL_DATATYPE_INITIALIZER_UINT1
 #define OMPI_DATATYPE_INITIALIZER_SIGNED_CHAR         OPAL_DATATYPE_INITIALIZER_INT1
 #define OMPI_DATATYPE_INITIALIZER_BYTE                OPAL_DATATYPE_INITIALIZER_UINT1
-#elif SIZEOF_CHAR == 2
-#define OMPI_DATATYPE_INITIALIZER_CHAR                OPAL_DATATYPE_INITIALIZER_INT2
-#define OMPI_DATATYPE_INITIALIZER_UNSIGNED_CHAR       OPAL_DATATYPE_INITIALIZER_UINT2
-#define OMPI_DATATYPE_INITIALIZER_SIGNED_CHAR         OPAL_DATATYPE_INITIALIZER_INT2
-#define OMPI_DATATYPE_INITIALIZER_BYTE                OPAL_DATATYPE_INITIALIZER_UINT2
-#elif SIZEOF_CHAR == 4
-#define OMPI_DATATYPE_INITIALIZER_CHAR                OPAL_DATATYPE_INITIALIZER_INT4
-#define OMPI_DATATYPE_INITIALIZER_UNSIGNED_CHAR       OPAL_DATATYPE_INITIALIZER_UINT4
-#define OMPI_DATATYPE_INITIALIZER_SIGNED_CHAR         OPAL_DATATYPE_INITIALIZER_INT4
-#define OMPI_DATATYPE_INITIALIZER_BYTE                OPAL_DATATYPE_INITIALIZER_UINT4
-#elif SIZEOF_CHAR == 8
-#define OMPI_DATATYPE_INITIALIZER_CHAR                OPAL_DATATYPE_INITIALIZER_INT8
-#define OMPI_DATATYPE_INITIALIZER_UNSIGNED_CHAR       OPAL_DATATYPE_INITIALIZER_UINT8
-#define OMPI_DATATYPE_INITIALIZER_SIGNED_CHAR         OPAL_DATATYPE_INITIALIZER_INT8
-#define OMPI_DATATYPE_INITIALIZER_BYTE                OPAL_DATATYPE_INITIALIZER_UINT8
-#endif
 
 #if SIZEOF_SHORT == 2
 #define OMPI_DATATYPE_INITIALIZER_SHORT               OPAL_DATATYPE_INITIALIZER_INT2
@@ -543,8 +536,6 @@ extern const ompi_datatype_t* ompi_datatype_basicDatatypes[OMPI_DATATYPE_MPI_MAX
 #define OMPI_DATATYPE_INITIALIZER_LONG                OPAL_DATATYPE_INITIALIZER_LONG
 #define OMPI_DATATYPE_INITIALIZER_UNSIGNED_LONG       OPAL_DATATYPE_INITIALIZER_UNSIGNED_LONG
 
-
-#if HAVE_LONG_LONG
 #if SIZEOF_LONG_LONG == 4
 #define OMPI_DATATYPE_INITIALIZER_LONG_LONG_INT       OPAL_DATATYPE_INITIALIZER_INT4
 #define OMPI_DATATYPE_INITIALIZER_UNSIGNED_LONG_LONG  OPAL_DATATYPE_INITIALIZER_UINT4
@@ -556,12 +547,25 @@ extern const ompi_datatype_t* ompi_datatype_basicDatatypes[OMPI_DATATYPE_MPI_MAX
 #define OMPI_DATATYPE_INITIALIZER_UNSIGNED_LONG_LONG  OPAL_DATATYPE_INITIALIZER_UINT16
 #endif
 
-#else /* HAVE_LONG_LONG */
-
-#define OMPI_DATATYPE_INITIALIZER_LONG_LONG_INT       OPAL_DATATYPE_INIT_UNAVAILABLE (LONG_LONG_INT, OMPI_DATATYPE_FLAG_DATA_C)
-#define OMPI_DATATYPE_INITIALIZER_UNSIGNED_LONG_LONG  OPAL_DATATYPE_INIT_UNAVAILABLE (UNSIGNED_LONG_LONG, OMPI_DATATYPE_FLAG_DATA_C)
-
-#endif /* HAVE_LONG_LONG */
+#if defined(HAVE_SHORT_FLOAT)
+#if SIZEOF_SHORT_FLOAT == 2
+#define OMPI_DATATYPE_INITIALIZER_SHORT_FLOAT         OPAL_DATATYPE_INITIALIZER_FLOAT2
+#elif SIZEOF_SHORT_FLOAT == 4
+#define OMPI_DATATYPE_INITIALIZER_SHORT_FLOAT         OPAL_DATATYPE_INITIALIZER_FLOAT4
+#elif SIZEOF_SHORT_FLOAT == 8
+#define OMPI_DATATYPE_INITIALIZER_SHORT_FLOAT         OPAL_DATATYPE_INITIALIZER_FLOAT8
+#endif
+#elif defined(HAVE_OPAL_SHORT_FLOAT_T) /* HAVE_SHORT_FLOAT */
+#if SIZEOF_OPAL_SHORT_FLOAT_T == 2
+#define OMPI_DATATYPE_INITIALIZER_SHORT_FLOAT         OPAL_DATATYPE_INITIALIZER_FLOAT2
+#elif SIZEOF_OPAL_SHORT_FLOAT_T == 4
+#define OMPI_DATATYPE_INITIALIZER_SHORT_FLOAT         OPAL_DATATYPE_INITIALIZER_FLOAT4
+#elif SIZEOF_OPAL_SHORT_FLOAT_T == 8
+#define OMPI_DATATYPE_INITIALIZER_SHORT_FLOAT         OPAL_DATATYPE_INITIALIZER_FLOAT8
+#endif
+#else /* HAVE_SHORT_FLOAT */
+#define OMPI_DATATYPE_INITIALIZER_SHORT_FLOAT         OPAL_DATATYPE_INITIALIZER_UNAVAILABLE
+#endif /* HAVE_SHORT_FLOAT */
 
 #if SIZEOF_FLOAT == 2
 #define OMPI_DATATYPE_INITIALIZER_FLOAT               OPAL_DATATYPE_INITIALIZER_FLOAT2
@@ -571,7 +575,6 @@ extern const ompi_datatype_t* ompi_datatype_basicDatatypes[OMPI_DATATYPE_MPI_MAX
 #define OMPI_DATATYPE_INITIALIZER_FLOAT               OPAL_DATATYPE_INITIALIZER_FLOAT8
 #endif
 
-
 #if SIZEOF_DOUBLE == 4
 #define OMPI_DATATYPE_INITIALIZER_DOUBLE              OPAL_DATATYPE_INITIALIZER_FLOAT4
 #elif SIZEOF_DOUBLE == 8
@@ -582,8 +585,6 @@ extern const ompi_datatype_t* ompi_datatype_basicDatatypes[OMPI_DATATYPE_MPI_MAX
 #define OMPI_DATATYPE_INITIALIZER_DOUBLE              OPAL_DATATYPE_INITIALIZER_FLOAT16
 #endif
 
-
-#if HAVE_LONG_DOUBLE
 #if SIZEOF_LONG_DOUBLE == 4
 #define OMPI_DATATYPE_INITIALIZER_LONG_DOUBLE         OPAL_DATATYPE_INITIALIZER_FLOAT4
 #elif SIZEOF_LONG_DOUBLE == 8
@@ -594,18 +595,13 @@ extern const ompi_datatype_t* ompi_datatype_basicDatatypes[OMPI_DATATYPE_MPI_MAX
 #define OMPI_DATATYPE_INITIALIZER_LONG_DOUBLE         OPAL_DATATYPE_INITIALIZER_FLOAT16
 #endif
 
-#else /* HAVE_LONG_DOUBLE */
-
-#define OMPI_DATATYPE_INITIALIZER_LONG_DOUBLE         OMPI_DATATYPE_INIT_UNAVAILABLE(LONG_DOUBLE, OMPI_DATATYPE_FLAG_DATA_C)
-
-#endif  /* HAVE_LONG_DOUBLE */
-
 #define OMPI_DATATYPE_INITIALIZER_PACKED              OPAL_DATATYPE_INITIALIZER_UINT1
 
 #define OMPI_DATATYPE_INITIALIZER_BOOL                OPAL_DATATYPE_INITIALIZER_BOOL
 
 #define OMPI_DATATYPE_INITIALIZER_WCHAR               OPAL_DATATYPE_INITIALIZER_WCHAR
 
+#define OMPI_DATATYPE_INITIALIZER_C_SHORT_FLOAT_COMPLEX OPAL_DATATYPE_INITIALIZER_SHORT_FLOAT_COMPLEX
 #define OMPI_DATATYPE_INITIALIZER_C_FLOAT_COMPLEX       OPAL_DATATYPE_INITIALIZER_FLOAT_COMPLEX
 #define OMPI_DATATYPE_INITIALIZER_C_DOUBLE_COMPLEX      OPAL_DATATYPE_INITIALIZER_DOUBLE_COMPLEX
 #define OMPI_DATATYPE_INITIALIZER_C_LONG_DOUBLE_COMPLEX OPAL_DATATYPE_INITIALIZER_LONG_DOUBLE_COMPLEX
@@ -617,7 +613,7 @@ extern const ompi_datatype_t* ompi_datatype_basicDatatypes[OMPI_DATATYPE_MPI_MAX
 #define OMPI_DATATYPE_FIRST_TYPE                      OPAL_DATATYPE_MAX_PREDEFINED
 
 /*
- * Derived datatypes supposely contiguous
+ * Derived datatypes supposedly contiguous
  */
 #define OMPI_DATATYPE_2INT                            (OMPI_DATATYPE_FIRST_TYPE+6)
 #define OMPI_DATATYPE_2INTEGER                        (OMPI_DATATYPE_FIRST_TYPE+7)
