@@ -1,5 +1,6 @@
 /*
  * Copyright (c) 2009-2018 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2018      Amazon.com, Inc. or its affiliates.  All Rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -15,6 +16,7 @@
 #include <unistd.h>
 
 #include "opal/runtime/opal.h"
+#include "opal/util/printf.h"
 #include "opal/mca/dl/base/base.h"
 
 #if !OPAL_HAVE_DL_SUPPORT
@@ -115,11 +117,11 @@ static int do_test(void)
     /* If we're using libltdl, it will find the .la file and may
        discover that it needs to open the actual file in the .libs
        directory.  If we're not using libltdl, then we won't know
-       about the magic .la file / .libs directory.  Hueristic: if we
+       about the magic .la file / .libs directory.  Heuristic: if we
        get here, manually prefix the filename with .libs/ and try
        again. */
     char *rel_filename;
-    asprintf(&rel_filename, ".libs/%s", filename);
+    opal_asprintf(&rel_filename, ".libs/%s", filename);
     if (NULL == rel_filename) {
         return 1;
     }
