@@ -18,6 +18,7 @@
  *                         reserved.
  * Copyright (c) 2015-2016 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2020      Triad National Security, LLC. All rights
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -201,7 +202,7 @@ int ompi_group_incl(ompi_group_t* group, int n, const int *ranks, ompi_group_t *
         len[2] = ompi_group_calc_sporadic ( n ,ranks );
         len[3] = ompi_group_calc_bmap     ( n , group->grp_proc_count ,ranks );
 
-        /* determin minimum length */
+        /* determine minimum length */
         method = ompi_group_minloc ( len, 4 );
     }
 #endif
@@ -606,3 +607,36 @@ int ompi_group_count_local_peers (ompi_group_t *group)
 
     return local_peers;
 }
+
+int ompi_group_to_proc_name_array (ompi_group_t *group, opal_process_name_t **name_array, size_t *name_array_size)
+{
+    opal_process_name_t *array = calloc (group->grp_proc_count, sizeof (array[0]));
+
+    if (NULL == array) {
+        return OMPI_ERR_OUT_OF_RESOURCE;
+    }
+
+    for (int i = 0 ; i < group->grp_proc_count ; ++i) {
+        array[i] = ompi_group_get_proc_name (group, i);
+    }
+
+    *name_array = array;
+    *name_array_size = group->grp_proc_count;
+
+    return OMPI_SUCCESS;
+}
+
+bool ompi_group_overlap (const ompi_group_t *group1, const ompi_group_t *group2)
+{
+    for (int i = 0 ; i < group1->grp_proc_count ; ++i) {
+        opal_process_name_t proc1 = ompi_group_get_proc_name (group1, i);
+        for (int j = 0 ; j < group2->grp_proc_count ; ++j) {
+            opal_process_name_t proc2 = ompi_group_get_proc_name (group2, j);
+            if (0 == opal_compare_proc (proc1, proc2)) {
+                return true;
+            }
+        }
+    }
+
+    return false;
+}
