@@ -2,7 +2,8 @@
  * Copyright (c) 2018-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2020      Bull S.A.S. All rights reserved.
+ * Copyright (c) 2022      IBM Corporation. All rights reserved
+ * Copyright (c) 2020-2022 Bull S.A.S. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -39,6 +40,7 @@
 #include "ompi/mca/coll/base/coll_base_functions.h"
 #include "coll_han_trigger.h"
 #include "ompi/mca/coll/han/coll_han_dynamic.h"
+#include "coll_han_algorithms.h"
 
 /*
  * Today;
@@ -169,6 +171,27 @@ struct mca_coll_han_allgather_s {
 };
 typedef struct mca_coll_han_allgather_s mca_coll_han_allgather_t;
 
+typedef struct mca_coll_han_op_up_low_module_name_t {
+    char* han_op_up_module_name;
+    char* han_op_low_module_name;
+} mca_coll_han_op_up_low_module_name_t;
+
+/**
+ * The only reason we need to keep these around is because our MCA system does
+ * not support MCA variables that do not point to existing variables (aka. where
+ * mbv_storage does not exists until the completion of the application). Thus,
+ * we need to keep track of the storage for all variables, even the ones we
+ * only use to translated into a string.
+ */
+typedef struct mca_coll_han_op_module_name_t {
+    mca_coll_han_op_up_low_module_name_t bcast;
+    mca_coll_han_op_up_low_module_name_t reduce;
+    mca_coll_han_op_up_low_module_name_t allreduce;
+    mca_coll_han_op_up_low_module_name_t allgather;
+    mca_coll_han_op_up_low_module_name_t gather;
+    mca_coll_han_op_up_low_module_name_t scatter;
+} mca_coll_han_op_module_name_t;
+
 /**
  * Structure to hold the han coll component.  First it holds the
  * base coll component, and then holds a bunch of
@@ -177,12 +200,13 @@ typedef struct mca_coll_han_allgather_s mca_coll_han_allgather_t;
  */
 typedef struct mca_coll_han_component_t {
     /** Base coll component */
-    mca_coll_base_component_2_0_0_t super;
+    mca_coll_base_component_2_4_0_t super;
 
     /** MCA parameter: Priority of this component */
     int han_priority;
     /* whether output the log message */
     int han_output;
+    int han_output_verbose; /* activation level of coll han verbosity */
     /* segment size for bcast */
     uint32_t han_bcast_segsize;
     /* up level module for bcast */
@@ -213,11 +237,15 @@ typedef struct mca_coll_han_component_t {
     uint32_t han_scatter_up_module;
     /* low level module for scatter */
     uint32_t han_scatter_low_module;
+    /* name of the modules */
+    mca_coll_han_op_module_name_t han_op_module_name;
     /* whether we need reproducible results
      * (but disables topological optimisations)
      */
     bool han_reproducible;
     bool use_simple_algorithm[COLLCOUNT];
+    int use_algorithm[COLLCOUNT];
+    int use_algorithm_param[COLLCOUNT]; // MCA parmeter id for algo, to know if user provided
 
     /* Dynamic configuration rules */
     bool use_dynamic_file_rules;
@@ -226,7 +254,11 @@ typedef struct mca_coll_han_component_t {
     /* Dynamic rules from file */
     mca_coll_han_dynamic_rules_t dynamic_rules;
     /* Dynamic rules from mca parameter */
-    COMPONENT_T mca_rules[COLLCOUNT][NB_TOPO_LVL];
+    COMPONENT_T mca_sub_components[COLLCOUNT][NB_TOPO_LVL];
+
+    int num_available_algorithms[COLLCOUNT]; // not counting "default" behaviour
+    /* to show algorithms in ompi_info */
+    mca_base_var_enum_value_t* algorithm_enumerator[COLLCOUNT];
 
     /* Define maximum dynamic errors printed by rank 0 with a 0 verbosity level */
     int max_dynamic_errors;
@@ -247,7 +279,7 @@ typedef struct mca_coll_han_single_collective_fallback_s {
         mca_coll_base_module_gather_fn_t gather;
         mca_coll_base_module_reduce_fn_t reduce;
         mca_coll_base_module_scatter_fn_t scatter;
-    };
+    } module_fn;
     mca_coll_base_module_t* module;
 } mca_coll_han_single_collective_fallback_t;
 
@@ -316,28 +348,28 @@ OBJ_CLASS_DECLARATION(mca_coll_han_module_t);
  * Some defines to stick to the naming used in the other components in terms of
  * fallback routines
  */
-#define previous_allgather          fallback.allgather.allgather
+#define previous_allgather          fallback.allgather.module_fn.allgather
 #define previous_allgather_module   fallback.allgather.module
 
-#define previous_allgatherv         fallback.allgatherv.allgatherv
+#define previous_allgatherv         fallback.allgatherv.module_fn.allgatherv
 #define previous_allgatherv_module  fallback.allgatherv.module
 
-#define previous_allreduce          fallback.allreduce.allreduce
+#define previous_allreduce          fallback.allreduce.module_fn.allreduce
 #define previous_allreduce_module   fallback.allreduce.module
 
-#define previous_barrier            fallback.barrier.barrier
+#define previous_barrier            fallback.barrier.module_fn.barrier
 #define previous_barrier_module     fallback.barrier.module
 
-#define previous_bcast              fallback.bcast.bcast
+#define previous_bcast              fallback.bcast.module_fn.bcast
 #define previous_bcast_module       fallback.bcast.module
 
-#define previous_reduce             fallback.reduce.reduce
+#define previous_reduce             fallback.reduce.module_fn.reduce
 #define previous_reduce_module      fallback.reduce.module
 
-#define previous_gather             fallback.gather.gather
+#define previous_gather             fallback.gather.module_fn.gather
 #define previous_gather_module      fallback.gather.module
 
-#define previous_scatter            fallback.scatter.scatter
+#define previous_scatter            fallback.scatter.module_fn.scatter
 #define previous_scatter_module     fallback.scatter.module
 
 
@@ -345,9 +377,9 @@ OBJ_CLASS_DECLARATION(mca_coll_han_module_t);
 #define HAN_LOAD_FALLBACK_COLLECTIVE(HANM, COMM, COLL)                            \
     do {                                                                          \
         if ( ((COMM)->c_coll->coll_ ## COLL ## _module) == (mca_coll_base_module_t*)(HANM) ) { \
-            (COMM)->c_coll->coll_ ## COLL = (HANM)->fallback.COLL.COLL;               \
+            (COMM)->c_coll->coll_ ## COLL = (HANM)->previous_## COLL;               \
             mca_coll_base_module_t *coll_module = (COMM)->c_coll->coll_ ## COLL ## _module; \
-            (COMM)->c_coll->coll_ ## COLL ## _module = (HANM)->fallback.COLL.module;  \
+            (COMM)->c_coll->coll_ ## COLL ## _module = (HANM)->previous_ ## COLL ## _module;  \
             OBJ_RETAIN((COMM)->c_coll->coll_ ## COLL ## _module);                     \
             OBJ_RELEASE(coll_module);                                                 \
         }                                                                             \
@@ -371,7 +403,7 @@ OBJ_CLASS_DECLARATION(mca_coll_han_module_t);
 /**
  * Global component instance
  */
-OMPI_MODULE_DECLSPEC extern mca_coll_han_component_t mca_coll_han_component;
+OMPI_DECLSPEC extern mca_coll_han_component_t mca_coll_han_component;
 
 /*
  * coll module functions
@@ -380,7 +412,7 @@ int mca_coll_han_init_query(bool enable_progress_threads, bool enable_mpi_thread
 
 mca_coll_base_module_t *mca_coll_han_comm_query(struct ompi_communicator_t *comm, int *priority);
 
-int han_request_free(ompi_request_t ** request);
+int ompi_coll_han_request_free(ompi_request_t ** request);
 
 /* Subcommunicator creation */
 int mca_coll_han_comm_create(struct ompi_communicator_t *comm, mca_coll_han_module_t * han_module);
@@ -445,109 +477,7 @@ mca_coll_han_scatter_intra_dynamic(SCATTER_BASE_ARGS,
 
 int mca_coll_han_barrier_intra_simple(struct ompi_communicator_t *comm,
                                       mca_coll_base_module_t *module);
-/* Bcast */
-int mca_coll_han_bcast_intra_simple(void *buff,
-                                    int count,
-                                    struct ompi_datatype_t *dtype,
-                                    int root,
-                                    struct ompi_communicator_t *comm,
-                                    mca_coll_base_module_t *module);
-int mca_coll_han_bcast_intra(void *buff, int count, struct ompi_datatype_t *dtype, int root,
-                             struct ompi_communicator_t *comm, mca_coll_base_module_t * module);
 
-/* Reduce */
-int
-mca_coll_han_reduce_intra_simple(const void *sbuf,
-                                 void* rbuf,
-                                 int count,
-                                 struct ompi_datatype_t *dtype,
-                                 ompi_op_t *op,
-                                 int root,
-                                 struct ompi_communicator_t *comm,
-                                 mca_coll_base_module_t *module);
-int
-mca_coll_han_reduce_reproducible_decision(struct ompi_communicator_t *comm,
-                                          mca_coll_base_module_t *module);
-int
-mca_coll_han_reduce_reproducible(const void *sbuf,
-                                 void *rbuf,
-                                 int count,
-                                 struct ompi_datatype_t *dtype,
-                                 struct ompi_op_t *op,
-                                 int root,
-                                 struct ompi_communicator_t *comm,
-                                 mca_coll_base_module_t *module);
-
-int mca_coll_han_reduce_intra(const void *sbuf,
-                              void *rbuf,
-                              int count,
-                              struct ompi_datatype_t *dtype,
-                              ompi_op_t* op,
-                              int root,
-                              struct ompi_communicator_t *comm,
-                              mca_coll_base_module_t * module);
-
-/* Allreduce */
-int
-mca_coll_han_allreduce_intra_simple(const void *sbuf,
-                                    void *rbuf,
-                                    int count,
-                                    struct ompi_datatype_t *dtype,
-                                    struct ompi_op_t *op,
-                                    struct ompi_communicator_t *comm,
-                                    mca_coll_base_module_t *module);
-int
-mca_coll_han_allreduce_reproducible_decision(struct ompi_communicator_t *comm,
-                                             mca_coll_base_module_t *module);
-int
-mca_coll_han_allreduce_reproducible(const void *sbuf,
-                                    void *rbuf,
-                                    int count,
-                                    struct ompi_datatype_t *dtype,
-                                    struct ompi_op_t *op,
-                                    struct ompi_communicator_t *comm,
-                                    mca_coll_base_module_t *module);
-
-int mca_coll_han_allreduce_intra(const void *sbuf,
-                                 void *rbuf,
-                                 int count,
-                                 struct ompi_datatype_t *dtype,
-                                 struct ompi_op_t *op,
-                                 struct ompi_communicator_t *comm, mca_coll_base_module_t * module);
-
-/* Scatter */
-int
-mca_coll_han_scatter_intra(const void *sbuf, int scount,
-                           struct ompi_datatype_t *sdtype,
-                           void *rbuf, int rcount,
-                           struct ompi_datatype_t *rdtype,
-                           int root,
-                           struct ompi_communicator_t *comm, mca_coll_base_module_t * module);
-int
-mca_coll_han_scatter_intra_simple(const void *sbuf, int scount,
-                                  struct ompi_datatype_t *sdtype,
-                                  void *rbuf, int rcount,
-                                  struct ompi_datatype_t *rdtype,
-                                  int root,
-                                  struct ompi_communicator_t *comm,
-                                  mca_coll_base_module_t * module);
-
-/* Gather */
-int
-mca_coll_han_gather_intra(const void *sbuf, int scount,
-                          struct ompi_datatype_t *sdtype,
-                          void *rbuf, int rcount,
-                          struct ompi_datatype_t *rdtype,
-                          int root,
-                          struct ompi_communicator_t *comm, mca_coll_base_module_t * module);
-int
-mca_coll_han_gather_intra_simple(const void *sbuf, int scount,
-                                 struct ompi_datatype_t *sdtype,
-                                 void *rbuf, int rcount,
-                                 struct ompi_datatype_t *rdtype,
-                                 int root,
-                                 struct ompi_communicator_t *comm,
-                                 mca_coll_base_module_t *module);
 /* reordering after gather, for unordered ranks */
 void
 ompi_coll_han_reorder_gather(const void *sbuf,
@@ -556,21 +486,4 @@ ompi_coll_han_reorder_gather(const void *sbuf,
                              struct ompi_communicator_t *comm,
                              int * topo);
 
-
-
-/* Allgather */
-int
-mca_coll_han_allgather_intra(const void *sbuf, int scount,
-                             struct ompi_datatype_t *sdtype,
-                             void *rbuf, int rcount,
-                             struct ompi_datatype_t *rdtype,
-                             struct ompi_communicator_t *comm, mca_coll_base_module_t * module);
-int
-mca_coll_han_allgather_intra_simple(const void *sbuf, int scount,
-                                    struct ompi_datatype_t *sdtype,
-                                    void* rbuf, int rcount,
-                                    struct ompi_datatype_t *rdtype,
-                                    struct ompi_communicator_t *comm,
-                                    mca_coll_base_module_t *module);
-
 #endif                          /* MCA_COLL_HAN_EXPORT_H */
