@@ -4,6 +4,7 @@
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2020      Bull S.A.S. All rights reserved.
+ * Copyright (c) 2022      IBM Corporation. All rights reserved
  *
  * $COPYRIGHT$
  *
@@ -23,7 +24,6 @@
 #include "ompi/mca/coll/base/coll_base_functions.h"
 #include "ompi/mca/coll/han/coll_han.h"
 
-
 /*
  * @file
  * #################################################
@@ -54,14 +54,14 @@
  *     - MCA parameter defined rules
  *     - File defined rules
  *
- * MCA parameter defined rules are stored in mca_coll_han_component.mca_rules.
- * This is a double indexed table. The first index is the coresponding collective
+ * MCA parameter defined rules are stored in mca_coll_han_component.mca_sub_components.
+ * This is a double indexed table. The first index is the corresponding collective
  * communication and the second index is the topological level aimed by the rule.
  * These parameters define the collective component to use for a specific
  * collective communication on a specific topologic level.
  *
  * File defined rules are stored in mca_coll_han_component.dynamic_rules.
- * These structures are defined bellow. The rule storage is directy deduced
+ * These structures are defined below. The rule storage is directly deduced
  * from the rule file format.
  *
  * File defined rules precede MCA parameter defined rules.
@@ -114,7 +114,7 @@ typedef struct {
     mca_coll_base_component_t* component;
 } ompi_coll_han_components;
 
-extern ompi_coll_han_components available_components[COMPONENTS_COUNT];
+extern ompi_coll_han_components ompi_coll_han_available_components[COMPONENTS_COUNT];
 
 /* Topologic levels */
 typedef enum TOPO_LVL {
@@ -140,6 +140,7 @@ typedef struct msg_size_rule_s {
     /* Component to use on this specific configuration
      * and message size */
     COMPONENT_T component;
+    int algorithm_id;
 } msg_size_rule_t;
 
 /* Rule for a specific configuration
@@ -210,5 +211,6 @@ typedef struct mca_coll_han_collective_modules_storage_s {
 /* Tests if a dynamic collective is implemented */
 bool mca_coll_han_is_coll_dynamic_implemented(COLLTYPE_T coll_id);
 COMPONENT_T mca_coll_han_component_name_to_id(const char* name);
+int mca_coll_han_topo_lvl_name_to_id(const char *topo_level_str);
 
 #endif
