@@ -3,6 +3,10 @@
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2020      Bull S.A.S. All rights reserved.
+ * Copyright (c) 2021      Cisco Systems, Inc.  All rights reserved
+ * Copyright (c) 2021      Triad National Security, LLC. All rights
+ *                         reserved.
+ * Copyright (c) 2022      IBM Corporation. All rights reserved
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -19,7 +23,7 @@
 
 /*
  *@file
- * Coll han module managment file. Used for each new communicator.
+ * Coll han module management file. Used for each new communicator.
  */
 
 /*
@@ -32,7 +36,7 @@ static int mca_coll_han_module_disable(mca_coll_base_module_t * module,
 
 #define CLEAN_PREV_COLL(HANDLE, NAME)                    \
     do {                                                 \
-        (HANDLE)->fallback.NAME.NAME = NULL;             \
+        (HANDLE)->fallback.NAME.module_fn.NAME = NULL;   \
         (HANDLE)->fallback.NAME.module = NULL;           \
     } while (0)
 
@@ -186,21 +190,14 @@ mca_coll_han_comm_query(struct ompi_communicator_t * comm, int *priority)
      */
     if (OMPI_COMM_IS_INTER(comm)) {
         opal_output_verbose(10, ompi_coll_base_framework.framework_output,
-                            "coll:han:comm_query (%d/%s): intercomm; disqualifying myself",
-                            comm->c_contextid, comm->c_name);
+                            "coll:han:comm_query (%s/%s): intercomm; disqualifying myself",
+                            ompi_comm_print_cid(comm), comm->c_name);
         return NULL;
     }
     if (1 == ompi_comm_size(comm)) {
         opal_output_verbose(10, ompi_coll_base_framework.framework_output,
-                            "coll:han:comm_query (%d/%s): comm is too small; disqualifying myself",
-                            comm->c_contextid, comm->c_name);
-        return NULL;
-    }
-    if( !ompi_group_have_remote_peers(comm->c_local_group) ) {
-        /* The group only contains local processes. Disable HAN for now */
-        opal_output_verbose(10, ompi_coll_base_framework.framework_output,
-                            "coll:han:comm_query (%d/%s): comm has only local processes; disqualifying myself",
-                            comm->c_contextid, comm->c_name);
+                            "coll:han:comm_query (%s/%s): comm is too small; disqualifying myself",
+                            ompi_comm_print_cid(comm), comm->c_name);
         return NULL;
     }
     /* Get the priority level attached to this module. If priority is less
@@ -208,8 +205,8 @@ mca_coll_han_comm_query(struct ompi_communicator_t * comm, int *priority)
     *priority = mca_coll_han_component.han_priority;
     if (mca_coll_han_component.han_priority < 0) {
         opal_output_verbose(10, ompi_coll_base_framework.framework_output,
-                            "coll:han:comm_query (%d/%s): priority too low; disqualifying myself",
-                            comm->c_contextid, comm->c_name);
+                            "coll:han:comm_query (%s/%s): priority too low; disqualifying myself",
+                            ompi_comm_print_cid(comm), comm->c_name);
         return NULL;
     }
 
@@ -222,23 +219,33 @@ mca_coll_han_comm_query(struct ompi_communicator_t * comm, int *priority)
     han_module->topologic_level = GLOBAL_COMMUNICATOR;
 
     if (NULL != comm->super.s_info) {
-        char info_val[OPAL_MAX_INFO_VAL+1];
-
         /* Get the info value disaqualifying coll components */
+        opal_cstring_t *info_str;
         opal_info_get(comm->super.s_info, "ompi_comm_coll_han_topo_level",
-                      sizeof(info_val), info_val, &flag);
+                      &info_str, &flag);
 
         if (flag) {
-            if (0 == strcmp(info_val, "INTER_NODE")) {
+            if (0 == strcmp(info_str->string, "INTER_NODE")) {
                 han_module->topologic_level = INTER_NODE;
             } else {
                 han_module->topologic_level = INTRA_NODE;
             }
+            OBJ_RELEASE(info_str);
         }
     }
 
+    if( !ompi_group_have_remote_peers(comm->c_local_group)
+            && INTRA_NODE != han_module->topologic_level ) {
+        /* The group only contains local processes, and this is not a
+         * intra-node subcomm we created. Disable HAN for now */
+        opal_output_verbose(10, ompi_coll_base_framework.framework_output,
+                            "coll:han:comm_query (%s/%s): comm has only local processes; disqualifying myself",
+                            ompi_comm_print_cid(comm), comm->c_name);
+        OBJ_RELEASE(han_module);
+        return NULL;
+    }
+
     han_module->super.coll_module_enable = han_module_enable;
-    han_module->super.ft_event        = NULL;
     han_module->super.coll_alltoall   = NULL;
     han_module->super.coll_alltoallv  = NULL;
     han_module->super.coll_alltoallw  = NULL;
@@ -264,8 +271,8 @@ mca_coll_han_comm_query(struct ompi_communicator_t * comm, int *priority)
     }
 
     opal_output_verbose(10, ompi_coll_base_framework.framework_output,
-                        "coll:han:comm_query (%d/%s): pick me! pick me!",
-                        comm->c_contextid, comm->c_name);
+                        "coll:han:comm_query (%s/%s): pick me! pick me!",
+                        ompi_comm_print_cid(comm), comm->c_name);
     return &(han_module->super);
 }
 
@@ -280,8 +287,8 @@ mca_coll_han_comm_query(struct ompi_communicator_t * comm, int *priority)
     do {                                                                \
         if (!comm->c_coll->coll_ ## __api || !comm->c_coll->coll_ ## __api ## _module) { \
             opal_output_verbose(1, ompi_coll_base_framework.framework_output, \
-                                "(%d/%s): no underlying " # __api"; disqualifying myself", \
-                                comm->c_contextid, comm->c_name); \
+                                "(%s/%s): no underlying " # __api"; disqualifying myself", \
+                                ompi_comm_print_cid(comm), comm->c_name); \
             goto handle_error;                                  \
         }                                                       \
         han_module->previous_ ## __api            = comm->c_coll->coll_ ## __api; \
@@ -352,7 +359,7 @@ mca_coll_han_module_disable(mca_coll_base_module_t * module,
 /*
  * Free the han request
  */
-int han_request_free(ompi_request_t ** request)
+int ompi_coll_han_request_free(ompi_request_t ** request)
 {
     (*request)->req_state = OMPI_REQUEST_INVALID;
     OBJ_RELEASE(*request);
