@@ -117,7 +117,7 @@ ompi_coll_libnbc_component_t mca_coll_libnbc_component = {
         /* First, the mca_component_t struct containing meta information
          * about the component itself */
         .collm_version = {
-            MCA_COLL_BASE_VERSION_2_0_0,
+            MCA_COLL_BASE_VERSION_2_4_0,
 
             /* Component name and version */
             .mca_component_name = "libnbc",
@@ -200,7 +200,7 @@ libnbc_register(void)
      * As a result we are adding an MCA parameter to make a conservative
      * decision to avoid this issue. If the user knows that their application
      * does not use data types in this way, then they can set this parameter
-     * to get the old behavior. Once the issue is truely fixed, then this
+     * to get the old behavior. Once the issue is truly fixed, then this
      * parameter can be removed.
      */
     libnbc_ibcast_skip_dt_decision = true;
@@ -399,8 +399,6 @@ libnbc_comm_query(struct ompi_communicator_t *comm,
         module->super.coll_neighbor_alltoallw_init = ompi_coll_libnbc_neighbor_alltoallw_init;
     }
 
-    module->super.ft_event = NULL;
-
     if (OMPI_SUCCESS != NBC_Init_comm(comm, module)) {
         OBJ_RELEASE(module);
         return NULL;
