@@ -3,7 +3,7 @@
  * Copyright (c) 2006      The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2013-2018 The University of Tennessee and The University
+ * Copyright (c) 2013-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2006      The Technical University of Chemnitz. All
@@ -25,6 +25,8 @@
  * Additional copyrights may follow
  */
 #include "nbc_internal.h"
+#include "ompi/mca/coll/base/coll_tags.h"
+#include "ompi/mca/coll/base/coll_base_functions.h"
 #include "ompi/mca/coll/base/coll_base_util.h"
 #include "ompi/op/op.h"
 #include "ompi/mca/pml/pml.h"
@@ -335,8 +337,20 @@ int NBC_Progress(NBC_Handle *handle) {
     /* don't call ompi_request_test_all as it causes a recursive call into opal_progress */
     while (handle->req_count) {
         ompi_request_t *subreq = handle->req_array[handle->req_count - 1];
+#if OPAL_ENABLE_FT_MPI
+        if (REQUEST_COMPLETE(subreq)
+         || OPAL_UNLIKELY( ompi_request_is_failed(subreq) )) {
+#else
         if (REQUEST_COMPLETE(subreq)) {
+#endif /* OPAL_ENABLE_FT_MPI */
             if(OPAL_UNLIKELY( OMPI_SUCCESS != subreq->req_status.MPI_ERROR )) {
+#if OPAL_ENABLE_FT_MPI
+                if( MPI_ERR_PROC_FAILED == subreq->req_status.MPI_ERROR ||
+                    MPI_ERR_PROC_FAILED_PENDING == subreq->req_status.MPI_ERROR ||
+                    MPI_ERR_REVOKED == subreq->req_status.MPI_ERROR ) {
+                    NBC_DEBUG (1, "MPI Error in NBC subrequest %p : %d)", subreq, subreq->req_status.MPI_ERROR);
+                } else // this 'else' intentionally spills outside the ifdef
+#endif /* OPAL_ENABLE_FT_MPI */
                 NBC_Error ("MPI Error in NBC subrequest %p : %d", subreq, subreq->req_status.MPI_ERROR);
                 /* copy the error code from the underlying request and let the
                  * round finish */
