@@ -37,6 +37,19 @@
 #include "ompi/mca/fs/fs.h"
 
 
+#ifdef HAVE_SYS_STATFS_H
+#include <sys/statfs.h> /* or <sys/vfs.h> */
+#endif
+#ifdef HAVE_SYS_PARAM_H
+#include <sys/param.h>
+#endif
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 BEGIN_C_DECLS
 
 OMPI_DECLSPEC int mca_fs_base_file_select(struct ompio_file_t *file,
@@ -52,6 +65,9 @@ OMPI_DECLSPEC int mca_fs_base_init_file (struct ompio_file_t *file);
 OMPI_DECLSPEC int mca_fs_base_get_param (struct ompio_file_t *file, int keyval);
 OMPI_DECLSPEC void mca_fs_base_get_parent_dir (char *filename, char **dirnamep);
 OMPI_DECLSPEC int  mca_fs_base_get_fstype(char *fname);
+OMPI_DECLSPEC int mca_fs_base_get_mpi_err(int errno_val);
+OMPI_DECLSPEC int mca_fs_base_get_file_perm(ompio_file_t *fh);
+OMPI_DECLSPEC int mca_fs_base_get_file_amode(int rank, int access_mode);
 
 OMPI_DECLSPEC int mca_fs_base_file_delete (char* file_name, struct opal_info_t *info);
 OMPI_DECLSPEC int mca_fs_base_file_sync (ompio_file_t *fh);
@@ -59,6 +75,42 @@ OMPI_DECLSPEC int mca_fs_base_file_get_size (ompio_file_t *fh, OMPI_MPI_OFFSET_T
 OMPI_DECLSPEC int mca_fs_base_file_set_size (ompio_file_t *fh, OMPI_MPI_OFFSET_TYPE size);
 OMPI_DECLSPEC int mca_fs_base_file_close (ompio_file_t *fh);
 
+
+static inline bool mca_fs_base_is_link (const char *filename)
+{
+    int err;
+    bool ret = true;
+    struct stat statbuf;
+
+    err = lstat(filename, &statbuf);
+
+    if (err || (!S_ISLNK(statbuf.st_mode))) {
+        ret = false;
+    }
+
+    return ret;
+}
+
+static inline void mca_fs_base_get_real_filename (const char *filename, char **rfilename)
+{
+    int namelen;
+    char linkbuf[OPAL_PATH_MAX+1];
+
+    namelen = readlink(filename, linkbuf, OPAL_PATH_MAX);
+    if (namelen == -1) {
+        /* something strange has happened between the time that
+         * we determined that this was a link and the time that
+         * we attempted to read it; punt and use the old name.
+         */
+        *rfilename = strdup(filename);
+    }
+    else {
+        /* successfully read the link */
+        linkbuf[namelen] = '\0'; /* readlink doesn't null terminate */
+        *rfilename = strdup(linkbuf);
+    }
+}
+
 /*
  * Globals
  */
