@@ -38,7 +38,7 @@
  * *******************************************************************
  */
 static mca_fs_base_module_1_0_0_t lustre =  {
-    mca_fs_lustre_module_init, /* initalise after being selected */
+    mca_fs_lustre_module_init, /* initialise after being selected */
     mca_fs_lustre_module_finalize, /* close a module on a communicator */
     mca_fs_lustre_file_open,
     mca_fs_base_file_close,
