@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2016-2017 Inria.  All rights reserved.
- * Copyright (c) 2017      Research Organization for Information Science
- *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2017-2019 Research Organization for Information Science
+ *                         and Technology (RIST).  All rights reserved.
  * Copyright (c) 2017      Amazon.com, Inc. or its affiliates.  All Rights
  *                         reserved.
  * $COPYRIGHT$
@@ -14,9 +14,9 @@
 #ifndef MCA_OSC_MONITORING_MODULE_H
 #define MCA_OSC_MONITORING_MODULE_H
 
-#include <ompi/info/info.h>
-#include <ompi/win/win.h>
-#include <ompi/mca/osc/osc.h>
+#include "ompi/info/info.h"
+#include "ompi/win/win.h"
+#include "ompi/mca/osc/osc.h"
 
 /* Define once and for all the module_template variable name */
 #define OMPI_OSC_MONITORING_MODULE_VARIABLE(template)	\
@@ -50,7 +50,7 @@
     OSC_MONITORING_SET_TEMPLATE_FCT_NAME(template) (ompi_osc_base_module_t*module) \
     {                                                                   \
         /* Define the ompi_osc_monitoring_module_## template ##_init_done variable */ \
-        static int32_t init_done = 0;                                   \
+        opal_atomic_int32_t init_done = 0;                              \
         /* Define and set the ompi_osc_monitoring_## template           \
          * ##_template variable. The functions recorded here are        \
          * linked to the original functions of the original             \
