@@ -2,7 +2,8 @@
 /*
  * Copyright (c) 2014-2016 Los Alamos National Security, LLC.  All rights
  *                         reserved.
- * Copyright (c) 2020      Google, LLC. All rights reserved.
+ * Copyright (c) 2020-2021 Google, LLC. All rights reserved.
+ * Copyright (c) 2020      Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -100,7 +101,7 @@ static ompi_osc_rdma_region_t *find_insertion_point (ompi_osc_rdma_region_t *reg
         return (ompi_osc_rdma_region_t *)((intptr_t) regions + min_index * region_size);
     }
 
-    if (region->base > base || (region->base == base && region->len > region_size)) {
+    if (region->base > base || (region->base == base && (size_t)region->len > region_size)) {
         return find_insertion_point (regions, min_index, mid_index-1, base, region_size, region_index);
     }
 
@@ -114,7 +115,7 @@ static bool ompi_osc_rdma_find_conflicting_attachment (ompi_osc_rdma_handle_t *h
     OPAL_LIST_FOREACH(attachment, &handle->attachments, ompi_osc_rdma_attachment_t) {
         intptr_t region_bound = attachment->base + attachment->len;
         if ((base >= attachment->base && base < region_bound) ||
-             (bound > attachment->base && bound <= region_bound)) {
+            (bound > attachment->base && bound <= region_bound)) {
             OSC_RDMA_VERBOSE(MCA_BASE_VERBOSE_TRACE, "existing region {%p, %p} overlaps region {%p, %p}",
                              (void *) attachment->base, (void *) region_bound, (void *) base, (void *) bound);
             return true;
@@ -171,6 +172,8 @@ int ompi_osc_rdma_attach (struct ompi_win_t *win, void *base, size_t len)
     intptr_t page_size = opal_getpagesize ();
     int region_index, ret;
     size_t aligned_len;
+    int dev_id;
+    uint64_t flags;
 
     if (module->flavor != MPI_WIN_FLAVOR_DYNAMIC) {
         return OMPI_ERR_RMA_FLAVOR;
@@ -181,6 +184,16 @@ int ompi_osc_rdma_attach (struct ompi_win_t *win, void *base, size_t len)
         return OMPI_SUCCESS;
     }
 
+    /* Component query check doesn't apply for dynamic window types.
+     * We have to check whether a GPU window is supported here. */
+    if (opal_accelerator.check_addr(base, &dev_id, &flags)) {
+        if (!osc_rdma_btl_accel_support(&mca_btl_base_modules_initialized)) {
+            OSC_RDMA_VERBOSE(MCA_BASE_VERBOSE_TRACE, "Failed to attach %p. No components capable of attaching accelerator buffers.",
+                             base);
+            return OMPI_ERR_NOT_SUPPORTED;
+        }
+    }
+
     OSC_RDMA_VERBOSE(MCA_BASE_VERBOSE_TRACE, "attach: %s, %p, %lu", win->w_name, base, (unsigned long) len);
 
     OPAL_THREAD_LOCK(&module->lock);
@@ -213,7 +226,7 @@ int ompi_osc_rdma_attach (struct ompi_win_t *win, void *base, size_t len)
         return ret;
     }
 
-    /* do a binary seach for where the region should be inserted */
+    /* do a binary search for where the region should be inserted */
     if (region_count) {
         region = find_insertion_point ((ompi_osc_rdma_region_t *) module->state->regions, 0, region_count - 1,
                                        (intptr_t) base, module->region_size, &region_index);
@@ -239,7 +252,7 @@ int ompi_osc_rdma_attach (struct ompi_win_t *win, void *base, size_t len)
     rdma_region_handle = OBJ_NEW(ompi_osc_rdma_handle_t);
     assert (NULL != rdma_region_handle);
 
-    if (module->selected_btl->btl_register_mem) {
+    if (module->use_memory_registration) {
         mca_btl_base_registration_handle_t *handle;
 
         ret = ompi_osc_rdma_register (module, MCA_BTL_ENDPOINT_ANY, (void *) region->base, region->len,
@@ -251,7 +264,8 @@ int ompi_osc_rdma_attach (struct ompi_win_t *win, void *base, size_t len)
             return OMPI_ERR_RMA_ATTACH;
         }
 
-        memcpy (region->btl_handle_data, handle, module->selected_btl->btl_registration_handle_size);
+        assert(module->use_accelerated_btl);
+        memcpy(region->btl_handle_data, handle, module->accelerated_btl->btl_registration_handle_size);
         rdma_region_handle->btl_handle = handle;
     } else {
         rdma_region_handle->btl_handle = NULL;
@@ -311,10 +325,9 @@ int ompi_osc_rdma_detach (struct ompi_win_t *win, const void *base)
         rdma_region_handle = module->dynamic_handles[region_index];
         region = (ompi_osc_rdma_region_t *) ((intptr_t) module->state->regions + region_index * module->region_size);
         OSC_RDMA_VERBOSE(MCA_BASE_VERBOSE_INFO, "checking attachments at index %d {.base=%p, len=%lu} for attachment %p"
-                         ", region handle=%p", region_index, (void *) region->base,
-                         (unsigned long) region->len, base, (void *) rdma_region_handle);
+                         ", region handle=%p", region_index, (void *) region->base, (unsigned long)region->len, base, (void*)rdma_region_handle);
 
-        if (region->base > (intptr_t) base || (region->base + region->len) < (uintptr_t) base) {
+        if ((uintptr_t)region->base > (uintptr_t) base || (uintptr_t)(region->base + region->len) < (uintptr_t) base) {
             continue;
         }
 
@@ -340,7 +353,7 @@ int ompi_osc_rdma_detach (struct ompi_win_t *win, const void *base)
     OSC_RDMA_VERBOSE(MCA_BASE_VERBOSE_DEBUG, "detaching dynamic memory region {%p, %p} from index %d",
                      base, (void *)((intptr_t) base + region->len), region_index);
 
-    if (module->selected_btl->btl_register_mem) {
+    if (module->use_memory_registration) {
         ompi_osc_rdma_deregister (module, rdma_region_handle->btl_handle);
     }
 
@@ -392,8 +405,9 @@ static int ompi_osc_rdma_refresh_dynamic_region (ompi_osc_rdma_module_t *module,
         osc_rdma_counter_t remote_value;
 
         source_address = (uint64_t)(intptr_t) peer->super.state + offsetof (ompi_osc_rdma_state_t, region_count);
-        ret = ompi_osc_get_data_blocking (module, peer->super.state_endpoint, source_address, peer->super.state_handle,
-                                          &remote_value, sizeof (remote_value));
+        ret = ompi_osc_get_data_blocking (module, peer->super.state_btl_index, peer->super.state_endpoint,
+                                          source_address, peer->super.state_handle, &remote_value,
+                                          sizeof (remote_value));
         if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
             return ret;
         }
@@ -432,8 +446,8 @@ static int ompi_osc_rdma_refresh_dynamic_region (ompi_osc_rdma_module_t *module,
                                            OMPI_OSC_RDMA_LOCK_EXCLUSIVE);
 
         source_address = (uint64_t)(intptr_t) peer->super.state + offsetof (ompi_osc_rdma_state_t, regions);
-        ret = ompi_osc_get_data_blocking (module, peer->super.state_endpoint, source_address, peer->super.state_handle,
-                                          peer->regions, region_len);
+        ret = ompi_osc_get_data_blocking (module, peer->super.state_btl_index, peer->super.state_endpoint,
+                                          source_address, peer->super.state_handle, peer->regions, region_len);
         if (OPAL_UNLIKELY(OMPI_SUCCESS != ret)) {
             OPAL_THREAD_UNLOCK(&module->lock);
             return ret;
