@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2016 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2007 High Performance Computing Center Stuttgart,
@@ -11,10 +11,11 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008      UT-Battelle, LLC. All rights reserved.
- * Copyright (c) 2011-2015 Los Alamos National Security, LLC. All rights
+ * Copyright (c) 2011-2018 Los Alamos National Security, LLC. All rights
  *                         reserved.
  * Copyright (c) 2014      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2020      Google, LLC. All rights reserved.
  *
  * $COPYRIGHT$
  *
@@ -41,9 +42,9 @@ BEGIN_C_DECLS
 struct mca_pml_ob1_recv_request_t {
     mca_pml_base_recv_request_t req_recv;
     opal_ptr_t remote_req_send;
-    int32_t  req_lock;
-    int32_t  req_pipeline_depth;
-    size_t   req_bytes_received;  /**< amount of data transferred into the user buffer */
+    opal_atomic_int32_t  req_lock;
+    opal_atomic_int32_t  req_pipeline_depth;
+    opal_atomic_size_t   req_bytes_received;  /**< amount of data transferred into the user buffer */
     size_t   req_bytes_expected; /**< local size of the data as suggested by the user */
     size_t   req_rdma_offset;
     size_t   req_send_offset;
@@ -178,7 +179,10 @@ recv_request_pml_complete(mca_pml_ob1_recv_request_t *recvreq)
 
         if(true == recvreq->req_recv.req_base.req_free_called) {
             if( MPI_SUCCESS != recvreq->req_recv.req_base.req_ompi.req_status.MPI_ERROR ) {
-                ompi_mpi_abort(&ompi_mpi_comm_world.comm, MPI_ERR_REQUEST);
+                /* An error after freeing the request MUST be fatal
+                 * MPI3 ch3.7: MPI_REQUEST_FREE */
+                int err = MPI_ERR_REQUEST;
+                ompi_mpi_errors_are_fatal_comm_handler(NULL, &err, "Recv error after request freed");
             }
             MCA_PML_OB1_RECV_REQUEST_RETURN(recvreq);
         } else {
@@ -205,9 +209,8 @@ recv_request_pml_complete(mca_pml_ob1_recv_request_t *recvreq)
 static inline bool
 recv_request_pml_complete_check(mca_pml_ob1_recv_request_t *recvreq)
 {
-#if OPAL_ENABLE_MULTI_THREADS
     opal_atomic_rmb();
-#endif
+
     if(recvreq->req_match_received &&
             recvreq->req_bytes_received >= recvreq->req_recv.req_bytes_packed &&
             lock_recv_request(recvreq)) {
@@ -240,14 +243,14 @@ static inline void prepare_recv_req_converter(mca_pml_ob1_recv_request_t *req)
     recv_req_matched(request, hdr)
 
 static inline void recv_req_matched(mca_pml_ob1_recv_request_t *req,
-                                    mca_pml_ob1_match_hdr_t *hdr)
+                                    const mca_pml_ob1_match_hdr_t *hdr)
 {
     req->req_recv.req_base.req_ompi.req_status.MPI_SOURCE = hdr->hdr_src;
     req->req_recv.req_base.req_ompi.req_status.MPI_TAG = hdr->hdr_tag;
     req->req_match_received = true;
-#if OPAL_ENABLE_MULTI_THREADS
+
     opal_atomic_wmb();
-#endif
+
     if(req->req_recv.req_bytes_packed > 0) {
 #if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
         if(MPI_ANY_SOURCE == req->req_recv.req_base.req_peer) {
@@ -279,7 +282,7 @@ do {
         uint32_t iov_count = 0;                                                   \
         size_t max_data = bytes_received;                                         \
         size_t n, offset = seg_offset;                                            \
-        mca_btl_base_segment_t* segment = segments;                               \
+        const mca_btl_base_segment_t *segment = segments;                         \
                                                                                   \
         for( n = 0; n < num_segments; n++, segment++ ) {                          \
             if(offset >= segment->seg_len) {                                      \
@@ -315,7 +318,7 @@ do {
 void mca_pml_ob1_recv_request_progress_match(
     mca_pml_ob1_recv_request_t* req,
     struct mca_btl_base_module_t* btl,
-    mca_btl_base_segment_t* segments,
+    const mca_btl_base_segment_t* segments,
     size_t num_segments);
 
 /**
@@ -325,14 +328,13 @@ void mca_pml_ob1_recv_request_progress_match(
 void mca_pml_ob1_recv_request_progress_frag(
     mca_pml_ob1_recv_request_t* req,
     struct mca_btl_base_module_t* btl,
-    mca_btl_base_segment_t* segments,
+    const mca_btl_base_segment_t* segments,
     size_t num_segments);
 
-#if OPAL_CUDA_SUPPORT
 void mca_pml_ob1_recv_request_frag_copy_start(
     mca_pml_ob1_recv_request_t* req,
     struct mca_btl_base_module_t* btl,
-    mca_btl_base_segment_t* segments,
+    const mca_btl_base_segment_t* segments,
     size_t num_segments,
     mca_btl_base_descriptor_t* des);
 
@@ -340,7 +342,6 @@ void mca_pml_ob1_recv_request_frag_copy_finished(struct mca_btl_base_module_t* b
     struct mca_btl_base_endpoint_t* ep,
     struct mca_btl_base_descriptor_t* des,
     int status );
-#endif /* OPAL_CUDA_SUPPORT */
 /**
  *
  */
@@ -348,7 +349,7 @@ void mca_pml_ob1_recv_request_frag_copy_finished(struct mca_btl_base_module_t* b
 void mca_pml_ob1_recv_request_progress_rndv(
     mca_pml_ob1_recv_request_t* req,
     struct mca_btl_base_module_t* btl,
-    mca_btl_base_segment_t* segments,
+    const mca_btl_base_segment_t* segments,
     size_t num_segments);
 
 /**
@@ -358,7 +359,7 @@ void mca_pml_ob1_recv_request_progress_rndv(
 void mca_pml_ob1_recv_request_progress_rget(
     mca_pml_ob1_recv_request_t* req,
     struct mca_btl_base_module_t* btl,
-    mca_btl_base_segment_t* segments,
+    const mca_btl_base_segment_t* segments,
     size_t num_segments);
 
 /**
@@ -368,7 +369,7 @@ void mca_pml_ob1_recv_request_progress_rget(
 void mca_pml_ob1_recv_request_matched_probe(
     mca_pml_ob1_recv_request_t* req,
     struct mca_btl_base_module_t* btl,
-    mca_btl_base_segment_t* segments,
+    const mca_btl_base_segment_t* segments,
     size_t num_segments);
 
 /**
