@@ -11,6 +11,8 @@
  *                         All rights reserved.
  * Copyright (c) 2013-2015 University of Houston. All rights reserved.
  * Copyright (c) 2016-2017 IBM Corporation. All rights reserved.
+ * Copyright (c) 2018      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -37,7 +39,7 @@
  */
  /* IMPORTANT: Update here when adding sharedfp component interface functions*/
 static mca_sharedfp_base_module_1_0_0_t individual =  {
-    mca_sharedfp_individual_module_init, /* initalise after being selected */
+    mca_sharedfp_individual_module_init, /* initialise after being selected */
     mca_sharedfp_individual_module_finalize, /* close a module on a communicator */
     mca_sharedfp_individual_seek,
     mca_sharedfp_individual_get_position,
@@ -68,15 +70,14 @@ int mca_sharedfp_individual_component_init_query(bool enable_progress_threads,
    return OMPI_SUCCESS;
 }
 
-struct mca_sharedfp_base_module_1_0_0_t * mca_sharedfp_individual_component_file_query (mca_io_ompio_file_t *fh, int *priority) {
+struct mca_sharedfp_base_module_1_0_0_t * mca_sharedfp_individual_component_file_query (ompio_file_t *fh, int *priority) {
 
     int amode;
     bool wronly_flag=false;
     bool relaxed_order_flag=false;
     opal_info_t *info;
     int flag;
-    int valuelen;
-    char value[MPI_MAX_INFO_VAL+1];
+    opal_cstring_t *info_str;
     *priority = 0;
 
     /*test, and update priority*/
@@ -103,16 +104,16 @@ struct mca_sharedfp_base_module_1_0_0_t * mca_sharedfp_individual_component_file
     /* 2. Did the user specify MPI_INFO relaxed ordering flag? */
     info = fh->f_info;
     if ( info != &(MPI_INFO_NULL->super) ){
-        valuelen = MPI_MAX_INFO_VAL;
-        opal_info_get ( info,"OMPIO_SHAREDFP_RELAXED_ORDERING", valuelen, value, &flag);
+        opal_info_get ( info,"OMPIO_SHAREDFP_RELAXED_ORDERING", &info_str, &flag);
         if ( flag ) {
            if ( mca_sharedfp_individual_verbose ) {
                 opal_output(ompi_sharedfp_base_framework.framework_output,
                         "mca_sharedfp_individual_component_file_query: "
-                        "OMPIO_SHAREDFP_RELAXED_ORDERING=%s\n",value);
+                        "OMPIO_SHAREDFP_RELAXED_ORDERING=%s\n", info_str->string);
 	    }
             /* flag - Returns true if key defined, false if not (boolean). */
             relaxed_order_flag=true;
+            OBJ_RELEASE(info_str);
         }
         else {
             if ( mca_sharedfp_individual_verbose ) {
@@ -152,7 +153,7 @@ struct mca_sharedfp_base_module_1_0_0_t * mca_sharedfp_individual_component_file
     return NULL;
 }
 
-int mca_sharedfp_individual_component_file_unquery (mca_io_ompio_file_t *file)
+int mca_sharedfp_individual_component_file_unquery (ompio_file_t *file)
 {
    /* This function might be needed for some purposes later. for now it
     * does not have anything to do since there are no steps which need
@@ -161,13 +162,13 @@ int mca_sharedfp_individual_component_file_unquery (mca_io_ompio_file_t *file)
    return OMPI_SUCCESS;
 }
 
-int mca_sharedfp_individual_module_init (mca_io_ompio_file_t *file)
+int mca_sharedfp_individual_module_init (ompio_file_t *file)
 {
     return OMPI_SUCCESS;
 }
 
 
-int mca_sharedfp_individual_module_finalize (mca_io_ompio_file_t *file)
+int mca_sharedfp_individual_module_finalize (ompio_file_t *file)
 {
     return OMPI_SUCCESS;
 }
