@@ -27,7 +27,13 @@
  */
 
 #include "ompi_config.h"
+
+#if HAVE_SYS_STAT_H
+#    include <sys/stat.h>
+#endif /* HAVE_SYS_STAT_H */
+
 #include "mpi.h"
+#include "opal/util/printf.h"
 #include "ompi/mca/sharedfp/sharedfp.h"
 #include "ompi/mca/sharedfp/base/base.h"
 #include "ompi/mca/sharedfp/sm/sharedfp_sm.h"
@@ -41,7 +47,7 @@
  */
  /* IMPORTANT: Update here when adding sharedfp component interface functions*/
 static mca_sharedfp_base_module_1_0_0_t sm =  {
-    mca_sharedfp_sm_module_init, /* initalise after being selected */
+    mca_sharedfp_sm_module_init, /* initialise after being selected */
     mca_sharedfp_sm_module_finalize, /* close a module on a communicator */
     mca_sharedfp_sm_seek,
     mca_sharedfp_sm_get_position,
@@ -91,9 +97,9 @@ struct mca_sharedfp_base_module_1_0_0_t * mca_sharedfp_sm_component_file_query(o
         proc = ompi_group_peer_lookup(group,i);
         if (!OPAL_PROC_ON_LOCAL_NODE(proc->super.proc_flags)){
             opal_output_verbose(10, ompi_sharedfp_base_framework.framework_output,
-                        "mca_sharedfp_sm_component_file_query: Disqualifying myself: (%d/%s) "
+                        "mca_sharedfp_sm_component_file_query: Disqualifying myself: (%s/%s) "
                         "not all processes are on the same node.",
-                        comm->c_contextid, comm->c_name);
+                        ompi_comm_print_cid (comm), comm->c_name);
             return NULL;
         }
     }
@@ -105,7 +111,7 @@ struct mca_sharedfp_base_module_1_0_0_t * mca_sharedfp_sm_component_file_query(o
     int comm_cid = -1;
     int pid = ompi_comm_rank (comm);
     
-    asprintf(&sm_filename, "%s/%s_cid-%d-%d.sm", ompi_process_info.job_session_dir,
+    opal_asprintf(&sm_filename, "%s/%s_cid-%d-%d.sm", ompi_process_info.job_session_dir,
              filename_basename, comm_cid, pid);
     free(filename_basename);
 
