@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2018 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart,
@@ -55,7 +55,7 @@ int MPI_Cancel(MPI_Request *request)
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
         if (NULL == request || NULL == *request ||
             MPI_REQUEST_NULL == *request) {
-            OMPI_ERRHANDLER_RETURN(MPI_ERR_REQUEST, MPI_COMM_WORLD,
+            OMPI_ERRHANDLER_NOHANDLE_RETURN(MPI_ERR_REQUEST, 
                                    MPI_ERR_REQUEST, FUNC_NAME);
         }
     }
@@ -64,8 +64,7 @@ int MPI_Cancel(MPI_Request *request)
         return MPI_SUCCESS;
     }
 
-    OPAL_CR_ENTER_LIBRARY();
     rc = ompi_request_cancel(*request);
-    OMPI_ERRHANDLER_RETURN(rc, MPI_COMM_WORLD, rc, FUNC_NAME);
+    OMPI_ERRHANDLER_NOHANDLE_RETURN(rc, rc, FUNC_NAME);
 }
 
