@@ -52,10 +52,8 @@ int MPI_File_close(MPI_File *fh)
         }
     }
 
-    OPAL_CR_ENTER_LIBRARY();
-
     /* Release the MPI_File; the destructor releases the component,
-       zeroes out fiels, etc. */
+       zeroes out fields, etc. */
 
     rc = ompi_file_close(fh);
     OMPI_ERRHANDLER_RETURN(rc, *fh, rc, FUNC_NAME);
