@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2010-2012 Oak Ridge National Labs.  All rights reserved.
  * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
@@ -77,7 +78,19 @@ int MPI_Probe(int source, int tag, MPI_Comm comm, MPI_Status *status)
         return MPI_SUCCESS;
     }
 
-    OPAL_CR_ENTER_LIBRARY();
+#if OPAL_ENABLE_FT_MPI
+    /*
+     * Check here for issues with the peer, so we do not have to duplicate the
+     * functionality in the PML.
+     */
+    if( OPAL_UNLIKELY(!ompi_comm_iface_p2p_check_proc(comm, source, &rc)) ) {
+        if (MPI_STATUS_IGNORE != status) {
+            status->MPI_SOURCE = source;
+            status->MPI_TAG    = tag;
+        }
+        OMPI_ERRHANDLER_RETURN(rc, comm, rc, FUNC_NAME);
+    }
+#endif
 
     rc = MCA_PML_CALL(probe(source, tag, comm, status));
     /*
