@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart,
@@ -15,6 +15,9 @@
  *                         reserved.
  * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2021      Triad National Security, LLC. All rights
+ *                         reserved.
+ * Copyright (c) 2024      NVIDIA Corporation.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -30,6 +33,7 @@
 #include "ompi/errhandler/errhandler.h"
 #include "ompi/request/request.h"
 #include "ompi/memchecker.h"
+#include "ompi/runtime/ompi_spc.h"
 
 #if OMPI_BUILD_MPI_PROFILING
 #if OPAL_HAVE_WEAK_SYMBOLS
@@ -44,6 +48,8 @@ static const char FUNC_NAME[] = "MPI_Testall";
 int MPI_Testall(int count, MPI_Request requests[], int *flag,
                 MPI_Status statuses[])
 {
+    SPC_RECORD(OMPI_SPC_TESTALL, 1);
+
     MEMCHECKER(
         int j;
         for (j = 0; j < count; j++){
@@ -52,22 +58,19 @@ int MPI_Testall(int count, MPI_Request requests[], int *flag,
     );
 
     if ( MPI_PARAM_CHECK ) {
-        int i, rc = MPI_SUCCESS;
+        int rc = MPI_SUCCESS;
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
         if( (NULL == requests) && (0 != count) ) {
             rc = MPI_ERR_REQUEST;
         } else {
-            for (i = 0; i < count; i++) {
-                if (NULL == requests[i]) {
-                    rc = MPI_ERR_REQUEST;
-                    break;
-                }
+            if(!ompi_request_check_same_instance(requests, count) ) {
+                rc = MPI_ERR_REQUEST;
             }
         }
         if ((NULL == flag) || (count < 0)) {
             rc = MPI_ERR_ARG;
         }
-        OMPI_ERRHANDLER_CHECK(rc, MPI_COMM_WORLD, rc, FUNC_NAME);
+        OMPI_ERRHANDLER_NOHANDLE_CHECK(rc, rc, FUNC_NAME);
     }
 
     if (OPAL_UNLIKELY(0 == count)) {
@@ -75,20 +78,15 @@ int MPI_Testall(int count, MPI_Request requests[], int *flag,
         return MPI_SUCCESS;
     }
 
-    OPAL_CR_ENTER_LIBRARY();
-
     if (OMPI_SUCCESS == ompi_request_test_all(count, requests, flag,
                                               statuses)) {
-        OPAL_CR_EXIT_LIBRARY();
         return MPI_SUCCESS;
     }
 
     if (MPI_SUCCESS !=
         ompi_errhandler_request_invoke(count, requests, FUNC_NAME)) {
-        OPAL_CR_EXIT_LIBRARY();
         return MPI_ERR_IN_STATUS;
     }
 
-    OPAL_CR_EXIT_LIBRARY();
     return MPI_SUCCESS;
 }
