@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart,
@@ -50,20 +50,17 @@ int MPI_Type_free(MPI_Datatype *type)
       OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
       if (NULL == type || NULL == *type || MPI_DATATYPE_NULL == *type ||
           ompi_datatype_is_predefined(*type)) {
-        return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_TYPE,
+        return OMPI_ERRHANDLER_NOHANDLE_INVOKE(MPI_ERR_TYPE,
                                       FUNC_NAME );
       }
    }
 
-   OPAL_CR_ENTER_LIBRARY();
-
    rc = ompi_datatype_destroy( type );
    if( rc != MPI_SUCCESS ) {
-      OMPI_ERRHANDLER_RETURN( MPI_ERR_INTERN, MPI_COMM_WORLD,
+      OMPI_ERRHANDLER_NOHANDLE_RETURN( MPI_ERR_INTERN, 
                              MPI_ERR_INTERN, FUNC_NAME );
    }
    *type = MPI_DATATYPE_NULL;
 
-   OPAL_CR_EXIT_LIBRARY();
    return MPI_SUCCESS;
 }
