@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -11,9 +11,9 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008-2009 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2015      Research Organization for Information Science
+ * Copyright (c) 2015-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
- * Copyright (c) 2016      Los Alamos National Security, LLC. All rights
+ * Copyright (c) 2016-2017 Los Alamos National Security, LLC. All rights
  *                         reserved.
  * $COPYRIGHT$
  *
@@ -44,13 +44,11 @@ int MPI_Win_set_errhandler(MPI_Win win, MPI_Errhandler errhandler)
 {
     MPI_Errhandler tmp;
 
-    OPAL_CR_NOOP_PROGRESS();
-
     if (MPI_PARAM_CHECK) {
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
 
         if (ompi_win_invalid(win)) {
-            return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_WIN,
+            return OMPI_ERRHANDLER_NOHANDLE_INVOKE(MPI_ERR_WIN,
                                           FUNC_NAME);
         } else if (NULL == errhandler ||
                    MPI_ERRHANDLER_NULL == errhandler ||
@@ -63,11 +61,12 @@ int MPI_Win_set_errhandler(MPI_Win win, MPI_Errhandler errhandler)
     /* Prepare the new error handler */
     OBJ_RETAIN(errhandler);
 
-    /* Ditch the old errhandler, and decrement its refcount.  On 64
-       bits environments we have to make sure the reading of the
-       error_handler became atomic. */
-    tmp = OPAL_ATOMIC_SWAP_PTR(&win->error_handler, errhandler);
+    OPAL_THREAD_LOCK(&win->w_lock);
+    /* Ditch the old errhandler, and decrement its refcount. */
+    tmp = win->error_handler;
+    win->error_handler = errhandler;
     OBJ_RELEASE(tmp);
+    OPAL_THREAD_UNLOCK(&win->w_lock);
 
     /* All done */
     return MPI_SUCCESS;
