@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -34,13 +34,13 @@
 #pragma weak PMPI_Type_create_keyval_f = ompi_type_create_keyval_f
 #pragma weak PMPI_Type_create_keyval_f08 = ompi_type_create_keyval_f
 #else
-OMPI_GENERATE_F77_BINDINGS (PMPI_TYPE_CREATE_KEYVAL,
-                           pmpi_type_create_keyval,
-                           pmpi_type_create_keyval_,
-                           pmpi_type_create_keyval__,
+OMPI_GENERATE_F77_BINDINGS(PMPI_TYPE_CREATE_KEYVAL, pmpi_type_create_keyval,
+                           pmpi_type_create_keyval_, pmpi_type_create_keyval__,
                            pompi_type_create_keyval_f,
-                           (ompi_aint_copy_attr_function* type_copy_attr_fn, ompi_aint_delete_attr_function* type_delete_attr_fn, MPI_Fint *type_keyval, MPI_Aint *extra_state, MPI_Fint *ierr),
-                           (type_copy_attr_fn, type_delete_attr_fn, type_keyval, extra_state, ierr) )
+                           (ompi_aint_copy_attr_function type_copy_attr_fn,
+                            ompi_aint_delete_attr_function type_delete_attr_fn,
+                            MPI_Fint *type_keyval, MPI_Aint *extra_state, MPI_Fint *ierr),
+                           (type_copy_attr_fn, type_delete_attr_fn, type_keyval, extra_state, ierr))
 #endif
 #endif
 
@@ -54,13 +54,12 @@ OMPI_GENERATE_F77_BINDINGS (PMPI_TYPE_CREATE_KEYVAL,
 #pragma weak MPI_Type_create_keyval_f08 = ompi_type_create_keyval_f
 #else
 #if ! OMPI_BUILD_MPI_PROFILING
-OMPI_GENERATE_F77_BINDINGS (MPI_TYPE_CREATE_KEYVAL,
-                           mpi_type_create_keyval,
-                           mpi_type_create_keyval_,
-                           mpi_type_create_keyval__,
-                           ompi_type_create_keyval_f,
-                           (ompi_aint_copy_attr_function* type_copy_attr_fn, ompi_aint_delete_attr_function* type_delete_attr_fn, MPI_Fint *type_keyval, MPI_Aint *extra_state, MPI_Fint *ierr),
-                           (type_copy_attr_fn, type_delete_attr_fn, type_keyval, extra_state, ierr) )
+OMPI_GENERATE_F77_BINDINGS(MPI_TYPE_CREATE_KEYVAL, mpi_type_create_keyval, mpi_type_create_keyval_,
+                           mpi_type_create_keyval__, ompi_type_create_keyval_f,
+                           (ompi_aint_copy_attr_function type_copy_attr_fn,
+                            ompi_aint_delete_attr_function type_delete_attr_fn,
+                            MPI_Fint *type_keyval, MPI_Aint *extra_state, MPI_Fint *ierr),
+                           (type_copy_attr_fn, type_delete_attr_fn, type_keyval, extra_state, ierr))
 #else
 #define ompi_type_create_keyval_f pompi_type_create_keyval_f
 #endif
@@ -68,8 +67,8 @@ OMPI_GENERATE_F77_BINDINGS (MPI_TYPE_CREATE_KEYVAL,
 
 static char FUNC_NAME[] = "MPI_Type_create_keyval_f";
 
-void ompi_type_create_keyval_f(ompi_aint_copy_attr_function* type_copy_attr_fn,
-                               ompi_aint_delete_attr_function* type_delete_attr_fn,
+void ompi_type_create_keyval_f(ompi_aint_copy_attr_function type_copy_attr_fn,
+                               ompi_aint_delete_attr_function type_delete_attr_fn,
                                MPI_Fint *type_keyval, MPI_Aint *extra_state, MPI_Fint *ierr)
 {
     int ret, c_ierr;
@@ -90,7 +89,7 @@ void ompi_type_create_keyval_f(ompi_aint_copy_attr_function* type_copy_attr_fn,
                                        NULL);
 
     if (MPI_SUCCESS != ret) {
-        c_ierr = OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD,
+        c_ierr = OMPI_ERRHANDLER_NOHANDLE_INVOKE(
                                         MPI_ERR_OTHER,
                                         FUNC_NAME);
         if (NULL != ierr) *ierr = OMPI_INT_2_FINT(c_ierr);
