@@ -10,12 +10,15 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2007-2012 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2007-2018 Cisco Systems, Inc.  All rights reserved
  * Copyright (c) 2009      Sun Microsystems, Inc.  All rights reserved.
  * Copyright (c) 2013      Los Alamos National Security, LLC. All rights
  *                         reserved.
  * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2018      FUJITSU LIMITED.  All rights reserved.
+ * Copyright (c) 2018      Triad National Security, LLC. All rights
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -26,11 +29,13 @@
 #include "ompi_config.h"
 
 #include "opal/class/opal_pointer_array.h"
+#include "opal/util/string_copy.h"
 
 #include "ompi/constants.h"
 #include "ompi/op/op.h"
 #include "ompi/mca/op/base/base.h"
 #include "ompi/datatype/ompi_datatype_internal.h"
+#include "ompi/instance/instance.h"
 
 
 /*
@@ -45,6 +50,7 @@ opal_pointer_array_t *ompi_op_f_to_c_table = {0};
 static int add_intrinsic(ompi_op_t *op, int fort_handle, int flags,
                          const char *name);
 
+static int ompi_op_finalize (void);
 
 /*
  * Class information
@@ -143,6 +149,10 @@ int ompi_op_init(void)
     ompi_op_ddt_map[OMPI_DATATYPE_MPI_DOUBLE] = OMPI_OP_BASE_TYPE_DOUBLE;
     ompi_op_ddt_map[OMPI_DATATYPE_MPI_LONG_DOUBLE] = OMPI_OP_BASE_TYPE_LONG_DOUBLE;
 
+#if OMPI_HAVE_FORTRAN_COMPLEX4
+    ompi_op_ddt_map[OMPI_DATATYPE_MPI_COMPLEX4] = FORTRAN_COMPLEX_OP_TYPE(OMPI_KIND_FORTRAN_COMPLEX4);
+#endif
+
 #if OMPI_HAVE_FORTRAN_COMPLEX8
     ompi_op_ddt_map[OMPI_DATATYPE_MPI_COMPLEX8] = FORTRAN_COMPLEX_OP_TYPE(OMPI_KIND_FORTRAN_COMPLEX8);
 #endif
@@ -216,6 +226,12 @@ int ompi_op_init(void)
 #warning Unsupported definition for MPI_COUNT
 #endif
 
+    /* Datatypes proposed to the MPI Forum in June 2017 for proposal in
+     * the MPI 4.0 standard. As of February 2019, it is not accepted yet.
+     * See https://github.com/mpi-forum/mpi-issues/issues/65 */
+    ompi_op_ddt_map[OMPI_DATATYPE_MPI_SHORT_FLOAT] = OMPI_OP_BASE_TYPE_SHORT_FLOAT;
+    ompi_op_ddt_map[OMPI_DATATYPE_MPI_C_SHORT_FLOAT_COMPLEX] = OMPI_OP_BASE_TYPE_C_SHORT_FLOAT_COMPLEX;
+
     ompi_op_ddt_map[OMPI_DATATYPE_MPI_LONG] = OMPI_OP_BASE_TYPE_LONG;
     ompi_op_ddt_map[OMPI_DATATYPE_MPI_UNSIGNED_LONG] = OMPI_OP_BASE_TYPE_UNSIGNED_LONG;
 
@@ -288,15 +304,23 @@ int ompi_op_init(void)
         ompi_mpi_op_replace.op.op_type = OMPI_OP_REPLACE;
     }
 
+    ompi_mpi_instance_append_finalize (ompi_op_finalize);
+
     /* All done */
     return OMPI_SUCCESS;
 }
 
 
-/*
- * Clean up the op resources
+
+/**
+ * Finalize the op interface.
+ *
+ * @returns OMPI_SUCCESS Always
+ *
+ * Invokes on instance teardown if ompi_op_init() was called; tears down the op interface, and
+ * destroys the F2C translation table.
  */
-int ompi_op_finalize(void)
+static int ompi_op_finalize (void)
 {
     /* clean up the intrinsic ops */
     OBJ_DESTRUCT(&ompi_mpi_op_no_op);
@@ -359,12 +383,12 @@ ompi_op_t *ompi_op_create_user(bool commute,
         new_op->o_flags |= OMPI_OP_FLAGS_COMMUTE;
     }
 
-    strncpy(new_op->o_name, "USER OP", sizeof(new_op->o_name) - 1);
+    opal_string_copy(new_op->o_name, "USER OP", sizeof(new_op->o_name));
     new_op->o_name[sizeof(new_op->o_name) - 1] = '\0';
 
     /* Set the user-defined callback function.  The "fort_fn" member
        is part of a union, so it doesn't matter if this is a C or
-       Fortan callback; we'll call the right flavor (per o_flags) at
+       Fortran callback; we'll call the right flavor (per o_flags) at
        invocation time. */
     new_op->o_func.fort_fn = func;
 
@@ -374,22 +398,6 @@ error:
 }
 
 
-/*
- * See lengthy comment in mpi/cxx/intercepts.cc for how the C++ MPI::Op
- * callbacks work.
- */
-void ompi_op_set_cxx_callback(ompi_op_t *op, MPI_User_function *fn)
-{
-    op->o_flags |= OMPI_OP_FLAGS_CXX_FUNC;
-    /* The OMPI C++ intercept was previously stored in
-       op->o_func.fort_fn by ompi_op_create_user().  So save that in
-       cxx.intercept_fn and put the user's fn in cxx.user_fn. */
-    op->o_func.cxx_data.intercept_fn =
-        (ompi_op_cxx_handler_fn_t *) op->o_func.fort_fn;
-    op->o_func.cxx_data.user_fn = fn;
-}
-
-
 /*
  * See lengthy comment in mpi/cxx/intercepts.cc for how the C++ MPI::Op
  * callbacks work.
@@ -426,7 +434,7 @@ static int add_intrinsic(ompi_op_t *op, int fort_handle, int flags,
 
     /* Set the members */
     op->o_flags = flags;
-    strncpy(op->o_name, name, sizeof(op->o_name) - 1);
+    opal_string_copy(op->o_name, name, sizeof(op->o_name));
     op->o_name[sizeof(op->o_name) - 1] = '\0';
 
     /* Perform the selection on this op to fill in the function
