@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2010 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2018 The University of Tennessee and The University
+ * Copyright (c) 2004-2021 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart,
@@ -31,8 +31,6 @@
 #include "ompi/request/request_default.h"
 #include "ompi/request/grequest.h"
 
-#include "ompi/mca/crcp/crcp.h"
-
 int ompi_request_default_wait(
     ompi_request_t ** req_ptr,
     ompi_status_public_t * status)
@@ -41,10 +39,18 @@ int ompi_request_default_wait(
 
     ompi_request_wait_completion(req);
 
-    OMPI_CRCP_REQUEST_COMPLETE(req);
+#if OPAL_ENABLE_FT_MPI
+    /* Special case for MPI_ANY_SOURCE */
+    if( MPI_ERR_PROC_FAILED_PENDING == req->req_status.MPI_ERROR ) {
+        if( MPI_STATUS_IGNORE != status ) {
+            OMPI_COPY_STATUS(status, req->req_status, false);
+        }
+        return MPI_ERR_PROC_FAILED_PENDING;
+    }
+#endif /* OPAL_ENABLE_FT_MPI */
 
     /* return status.  If it's a generalized request, we *have* to
-       invoke the query_fn, even if the user procided STATUS_IGNORE.
+       invoke the query_fn, even if the user provided STATUS_IGNORE.
        MPI-2:8.2. */
     if (OMPI_REQUEST_GEN == req->req_type) {
         ompi_grequest_invoke_query(req, &req->req_status);
@@ -91,6 +97,7 @@ int ompi_request_default_wait_any(size_t count,
         return OMPI_SUCCESS;
     }
 
+recheck:
     WAIT_SYNC_INIT(&sync, 1);
 
     num_requests_null_inactive = 0;
@@ -108,11 +115,20 @@ int ompi_request_default_wait_any(size_t count,
         }
 
         if( !OPAL_ATOMIC_COMPARE_EXCHANGE_STRONG_PTR(&request->req_complete, &_tmp_ptr, &sync) ) {
-            assert(REQUEST_COMPLETE(request));
+            if(OPAL_LIKELY( REQUEST_COMPLETE(request) )) {
+                completed = i;
+                *index = i;
+                goto after_sync_wait;
+            }
+        }
+
+#if OPAL_ENABLE_FT_MPI
+        if(OPAL_UNLIKELY( ompi_request_is_failed(request) )) {
             completed = i;
             *index = i;
             goto after_sync_wait;
         }
+#endif /* OPAL_ENABLE_FT_MPI */
     }
 
     if(num_requests_null_inactive == count) {
@@ -125,7 +141,7 @@ int ompi_request_default_wait_any(size_t count,
         return rc;
     }
 
-    SYNC_WAIT(&sync);
+    rc = SYNC_WAIT(&sync);
 
   after_sync_wait:
     /* recheck the complete status and clean up the sync primitives.
@@ -150,6 +166,15 @@ int ompi_request_default_wait_any(size_t count,
         }
     }
 
+    /* Error path: SYNC_WAIT was interrupted by an error
+     * We do this after the cleanup loop to make sure nobody is updating the
+     * sync again while we are rearming it */
+    if(OPAL_UNLIKELY( OMPI_SUCCESS != rc )) {
+        rc = OMPI_SUCCESS;
+        WAIT_SYNC_RELEASE(&sync);
+        goto recheck;
+    }
+
     if( *index == (int)completed ) {
         /* Only one request has triggered. There was no in-flight
          * completions. Drop the signalled flag so we won't block
@@ -159,12 +184,14 @@ int ompi_request_default_wait_any(size_t count,
     }
 
     request = requests[*index];
-    assert( REQUEST_COMPLETE(request) );
-#if OPAL_ENABLE_FT_CR == 1
-    if( opal_cr_is_enabled ) {
-        OMPI_CRCP_REQUEST_COMPLETE(request);
+#if OPAL_ENABLE_FT_MPI
+    /* Special case for MPI_ANY_SOURCE */
+    if( MPI_ERR_PROC_FAILED == request->req_status.MPI_ERROR ) {
+        WAIT_SYNC_RELEASE(&sync);
+        return MPI_ERR_PROC_FAILED_PENDING;
     }
-#endif
+#endif  /* OPAL_ENABLE_FT_MPI */
+    assert( REQUEST_COMPLETE(request) );
     /* Per note above, we have to call gen request query_fn even
        if STATUS_IGNORE was provided */
     if (OMPI_REQUEST_GEN == request->req_type) {
@@ -203,6 +230,7 @@ int ompi_request_default_wait_all( size_t count,
         return OMPI_SUCCESS;
     }
 
+recheck:
     WAIT_SYNC_INIT(&sync, count);
     rptr = requests;
     for (i = 0; i < count; i++) {
@@ -216,13 +244,25 @@ int ompi_request_default_wait_all( size_t count,
         }
 
         if (REQUEST_COMPLETE(request) || !OPAL_ATOMIC_COMPARE_EXCHANGE_STRONG_PTR(&request->req_complete, &_tmp_ptr, &sync)) {
-            if( OPAL_UNLIKELY( MPI_SUCCESS != request->req_status.MPI_ERROR ) ) {
-                failed++;
+            if( OPAL_LIKELY( REQUEST_COMPLETE(request) ) ) {
+                if( OPAL_UNLIKELY( MPI_SUCCESS != request->req_status.MPI_ERROR ) ) {
+                    failed++;
+                }
+                completed++;
             }
-            completed++;
         }
+
+#if OPAL_ENABLE_FT_MPI
+        if(OPAL_UNLIKELY( ompi_request_is_failed(request) )) {
+            failed++;
+            continue;
+        }
+#endif /* OPAL_ENABLE_FT_MPI */
     }
     if( failed > 0 ) {
+        /* We are completing only one here, lets prevent blocking in the
+         * SYNC_RELEASE by marking the sync as SIGNALED */
+        WAIT_SYNC_SIGNALLED(&sync);
         goto finish;
     }
 
@@ -233,9 +273,30 @@ int ompi_request_default_wait_all( size_t count,
     /* wait until all requests complete or until an error is triggered. */
     mpi_error = SYNC_WAIT(&sync);
     if( OPAL_SUCCESS != mpi_error ) {
-        /* if we are in an error case, increase the failed to ensure
-           proper cleanup during the requests completion. */
-        failed++;
+        /* The sync triggered because of an error. The error may be for us, but
+         * it may be for some other pending wait, so we have to recheck
+         * our request status.
+         *
+         * We are going to rearm the sync, but first make sure it is not
+         * updated by any progress thread meanwhile by removing it from all
+         * requests it has been attached to.
+         */
+        rptr = requests;
+        for (i = 0; i < count; i++) {
+            void *_tmp_ptr = &sync;
+
+            request = *rptr++;
+
+            if( request->req_state == OMPI_REQUEST_INACTIVE ) {
+                continue;
+            }
+
+            OPAL_ATOMIC_COMPARE_EXCHANGE_STRONG_PTR(&request->req_complete, &_tmp_ptr, REQUEST_PENDING);
+        }
+        /* The sync is now ready for rearming */
+        WAIT_SYNC_RELEASE(&sync);
+        failed = completed = 0;
+        goto recheck;
     }
 
  finish:
@@ -255,7 +316,7 @@ int ompi_request_default_wait_all( size_t count,
             if( OPAL_UNLIKELY(0 < failed) ) {
                 /* if we have failed requests we skipped the waiting on the sync. Thus,
                  * some of the requests might not be properly completed, in which case
-                 * we must detach all requests from the sync. However, if we can succesfully
+                 * we must detach all requests from the sync. However, if we can successfully
                  * mark the request as pending then it is neither failed nor complete, and
                  * we must stop altering it.
                  */
@@ -267,16 +328,18 @@ int ompi_request_default_wait_all( size_t count,
                      * there was an error in one of the other requests.
                      */
                     statuses[i].MPI_ERROR = MPI_ERR_PENDING;
+#if OPAL_ENABLE_FT_MPI
+                    /* PROC_FAILED_PENDING errors are also not completed yet */
+                    if( MPI_ERR_PROC_FAILED_PENDING == requests[i]->req_status.MPI_ERROR ) {
+                        statuses[i].MPI_ERROR = MPI_ERR_PROC_FAILED_PENDING;
+                    }
+#endif /* OPAL_ENABLE_FT_MPI */
                     mpi_error = MPI_ERR_IN_STATUS;
                     continue;
                 }
             }
             assert( REQUEST_COMPLETE(request) );
 
-            if( opal_cr_is_enabled) {
-                OMPI_CRCP_REQUEST_COMPLETE(request);
-            }
-
             if (OMPI_REQUEST_GEN == request->req_type) {
                 ompi_grequest_invoke_query(request, &request->req_status);
             }
@@ -329,15 +392,17 @@ int ompi_request_default_wait_all( size_t count,
                      * there was an error in one of the other requests.
                      */
                     rc = MPI_ERR_PENDING;
+#if OPAL_ENABLE_FT_MPI
+                    /* PROC_FAILED_PENDING errors are also not completed yet */
+                    if( MPI_ERR_PROC_FAILED_PENDING == requests[i]->req_status.MPI_ERROR ) {
+                        rc = MPI_ERR_PROC_FAILED_PENDING;
+                    }
+#endif  /* OPAL_ENABLE_FT_MPI */
                     goto absorb_error_and_continue;
                  }
             }
             assert( REQUEST_COMPLETE(request) );
 
-            if( opal_cr_is_enabled) {
-                OMPI_CRCP_REQUEST_COMPLETE(request);
-            }
-
             /* Per note above, we have to call gen request query_fn
                even if STATUSES_IGNORE was provided */
             if (OMPI_REQUEST_GEN == request->req_type) {
@@ -356,6 +421,11 @@ int ompi_request_default_wait_all( size_t count,
                 }
             }
     absorb_error_and_continue:
+#if OPAL_ENABLE_FT_MPI
+            if( (MPI_ERR_PROC_FAILED == rc) || (MPI_ERR_REVOKED == rc) ) {
+                mpi_error = rc;
+            }
+#endif  /* OPAL_ENABLE_FT_MPI */
             /*
              * Per MPI 2.2 p34:
              * "It is possible for an MPI function to return MPI_ERR_IN_STATUS
@@ -391,6 +461,7 @@ int ompi_request_default_wait_some(size_t count,
         return OMPI_SUCCESS;
     }
 
+  recheck:
     WAIT_SYNC_INIT(&sync, 1);
 
     *outcount = 0;
@@ -414,15 +485,23 @@ int ompi_request_default_wait_some(size_t count,
         indices[num_active_reqs] = OPAL_ATOMIC_COMPARE_EXCHANGE_STRONG_PTR(&request->req_complete, &_tmp_ptr, &sync);
         if( !indices[num_active_reqs] ) {
             /* If the request is completed go ahead and mark it as such */
-            assert( REQUEST_COMPLETE(request) );
-            num_requests_done++;
+            if( REQUEST_COMPLETE(request) ) {
+                num_requests_done++;
+            }
         }
+
+#if OPAL_ENABLE_FT_MPI
+        if(OPAL_UNLIKELY( ompi_request_is_failed(request) )) {
+            num_requests_done++;
+            continue;
+        }
+#endif /* OPAL_ENABLE_FT_MPI */
         num_active_reqs++;
     }
 
     if(num_requests_null_inactive == count) {
         *outcount = MPI_UNDEFINED;
-        /* nobody will signall us */
+        /* nobody will signal us */
         WAIT_SYNC_RELEASE_NOWAIT(&sync);
         return rc;
     }
@@ -465,6 +544,13 @@ int ompi_request_default_wait_some(size_t count,
         } else if( !OPAL_ATOMIC_COMPARE_EXCHANGE_STRONG_PTR(&request->req_complete, &_tmp_ptr, REQUEST_PENDING) ) {
             indices[num_requests_done++] = i;
         }
+#if OPAL_ENABLE_FT_MPI
+        /* Special case for MPI_ANY_SOURCE - Error managed below */
+        else if(OPAL_UNLIKELY( ompi_request_is_failed(request) &&
+                               MPI_ERR_PROC_FAILED_PENDING == request->req_status.MPI_ERROR )) {
+            indices[num_requests_done++] = i;
+        }
+#endif /* OPAL_ENABLE_FT_MPI */
         num_active_reqs++;
     }
     sync_unsets = num_active_reqs - num_requests_done;
@@ -478,17 +564,35 @@ int ompi_request_default_wait_some(size_t count,
 
     WAIT_SYNC_RELEASE(&sync);
 
+    /* error path: no requests are done because the sync got triggered
+     * We have nothing more to do here besides rearming the sync and trying
+     * again */
+    if(OPAL_UNLIKELY( 0 == num_requests_done )) {
+        assert(OMPI_SUCCESS != sync.status);
+        goto recheck;
+    }
+
     *outcount = num_requests_done;
 
     for (size_t i = 0; i < num_requests_done; i++) {
         request = requests[indices[i]];
-        assert( REQUEST_COMPLETE(request) );
-
-#if OPAL_ENABLE_FT_CR == 1
-        if( opal_cr_is_enabled) {
-            OMPI_CRCP_REQUEST_COMPLETE(request);
+#if OPAL_ENABLE_FT_MPI
+        /* Special case for MPI_ANY_SOURCE */
+        if( MPI_ERR_PROC_FAILED_PENDING == request->req_status.MPI_ERROR ) {
+            rc = MPI_ERR_IN_STATUS;
+            if (MPI_STATUSES_IGNORE != statuses) {
+                OMPI_COPY_STATUS(&statuses[i], request->req_status, true);
+                statuses[i].MPI_ERROR = MPI_ERR_PROC_FAILED_PENDING;
+            } else {
+                if( (MPI_ERR_PROC_FAILED == request->req_status.MPI_ERROR) ||
+                    (MPI_ERR_REVOKED == request->req_status.MPI_ERROR) ) {
+                    rc = request->req_status.MPI_ERROR;
+                }
+            }
+            continue;
         }
-#endif
+#endif /* OPAL_ENABLE_FT_MPI */
+        assert( REQUEST_COMPLETE(request) );
 
         /* Per note above, we have to call gen request query_fn even
            if STATUS_IGNORE was provided */
