@@ -9,6 +9,8 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2022      Amazon.com, Inc. or its affiliates.
+ *                         All Rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -19,34 +21,29 @@
 #ifndef OPAL_SYS_ARCH_TIMER_H
 #define OPAL_SYS_ARCH_TIMER_H 1
 
-
 typedef uint64_t opal_timer_t;
 
+#if OPAL_C_GCC_INLINE_ASSEMBLY
 
-#if OPAL_GCC_INLINE_ASSEMBLY
-
-static inline opal_timer_t
-opal_sys_timer_get_cycles(void)
+static inline opal_timer_t opal_sys_timer_get_cycles(void)
 {
     unsigned int tbl, tbu0, tbu1;
 
     do {
-        __asm__ __volatile__ ("mftbu %0" : "=r"(tbu0));
-        __asm__ __volatile__ ("mftb %0" : "=r"(tbl));
-        __asm__ __volatile__ ("mftbu %0" : "=r"(tbu1));
+        __asm__ __volatile__("mftbu %0" : "=r"(tbu0));
+        __asm__ __volatile__("mftb %0" : "=r"(tbl));
+        __asm__ __volatile__("mftbu %0" : "=r"(tbu1));
     } while (tbu0 != tbu1);
 
-    return (((unsigned long long)tbu0) << 32) | tbl;
+    return (((unsigned long long) tbu0) << 32) | tbl;
 }
 
-#define OPAL_HAVE_SYS_TIMER_GET_CYCLES 1
+#    define OPAL_HAVE_SYS_TIMER_GET_CYCLES 1
 
 #else
 
-opal_timer_t opal_sys_timer_get_cycles(void);
+#    define OPAL_HAVE_SYS_TIMER_GET_CYCLES 0
 
-#define OPAL_HAVE_SYS_TIMER_GET_CYCLES 1
-
-#endif /* OPAL_GCC_INLINE_ASSEMBLY */
+#endif /* OPAL_C_GCC_INLINE_ASSEMBLY */
 
 #endif /* ! OPAL_SYS_ARCH_TIMER_H */
