@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2015 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2013-2019 Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -14,9 +14,6 @@
 
 #include "btl_usnic_module.h"
 
-
-#if OPAL_HAVE_HWLOC
 int opal_btl_usnic_hwloc_distance(opal_btl_usnic_module_t *module);
-#endif
 
 #endif /* BTL_USNIC_HWLOC_H */
