@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2013-2017 Cisco Systems, Inc.  All rights reserved
  * $COPYRIGHT$
@@ -11,17 +12,14 @@
 #define BTL_USNIC_RECV_H
 
 #include "btl_usnic.h"
-#include "btl_usnic_util.h"
 #include "btl_usnic_frag.h"
 #include "btl_usnic_proc.h"
+#include "btl_usnic_util.h"
 
-
-void opal_btl_usnic_recv_call(opal_btl_usnic_module_t *module,
-                              opal_btl_usnic_recv_segment_t *rseg,
+void opal_btl_usnic_recv_call(opal_btl_usnic_module_t *module, opal_btl_usnic_recv_segment_t *rseg,
                               opal_btl_usnic_channel_t *channel);
 
-static inline int
-opal_btl_usnic_post_recv_list(opal_btl_usnic_channel_t *channel)
+static inline int opal_btl_usnic_post_recv_list(opal_btl_usnic_channel_t *channel)
 {
     struct iovec iov;
     struct fi_msg msg;
@@ -56,35 +54,32 @@ opal_btl_usnic_post_recv_list(opal_btl_usnic_channel_t *channel)
 /*
  * Given an incoming segment, lookup the endpoint that sent it
  */
-static inline opal_btl_usnic_endpoint_t *
-lookup_sender(opal_btl_usnic_module_t *module, opal_btl_usnic_segment_t *seg)
+static inline opal_btl_usnic_endpoint_t *lookup_sender(opal_btl_usnic_module_t *module,
+                                                       opal_btl_usnic_segment_t *seg)
 {
     int ret;
     opal_btl_usnic_endpoint_t *sender;
 
-    /* Use the hashed ORTE process name in the BTL header to uniquely
+    /* Use the hashed PRTE process name in the BTL header to uniquely
        identify the sending process (using the MAC/hardware address
-       only identifies the sending server -- not the sending ORTE
+       only identifies the sending server -- not the sending PRTE
        process). */
     /* JMS We've experimented with using a handshake before sending
        any data so that instead of looking up a hash on the
        btl_header->sender, echo back the ptr to the sender's
        ompi_proc.  There was limited speedup with this scheme; more
        investigation is required. */
-    ret = opal_hash_table_get_value_uint64(&module->senders,
-                                           seg->us_btl_header->sender,
-                                           (void**) &sender);
+    ret = opal_hash_table_get_value_uint64(&module->senders, seg->us_btl_header->sender,
+                                           (void **) &sender);
     if (OPAL_LIKELY(OPAL_SUCCESS == ret)) {
         return sender;
     }
 
     /* The sender wasn't in the hash table, so do a slow lookup and
        put the result in the hash table */
-    sender = opal_btl_usnic_proc_lookup_endpoint(module,
-                                                 seg->us_btl_header->sender);
+    sender = opal_btl_usnic_proc_lookup_endpoint(module, seg->us_btl_header->sender);
     if (NULL != sender) {
-        opal_hash_table_set_value_uint64(&module->senders,
-                                         seg->us_btl_header->sender, sender);
+        opal_hash_table_set_value_uint64(&module->senders, seg->us_btl_header->sender, sender);
         return sender;
     }
 
@@ -97,27 +92,25 @@ lookup_sender(opal_btl_usnic_module_t *module, opal_btl_usnic_segment_t *seg)
  * to indicate that it and possible following contiguous sequence
  * numbers have been received.
  */
-static inline void
-opal_btl_usnic_update_window(
-    opal_btl_usnic_endpoint_t *endpoint,
-    uint32_t window_index)
+static inline void opal_btl_usnic_update_window(opal_btl_usnic_endpoint_t *endpoint,
+                                                uint32_t window_index)
 {
     uint32_t i;
 
     /* Enable ACK reply if not enabled */
 #if MSGDEBUG1
-    opal_output(0, "ep: %p, ack_needed = %s\n", (void*)endpoint, endpoint->endpoint_ack_needed?"true":"false");
+    opal_output(0, "ep: %p, ack_needed = %s\n", (void *) endpoint,
+                endpoint->endpoint_ack_needed ? "true" : "false");
 #endif
     if (!endpoint->endpoint_ack_needed) {
         opal_btl_usnic_add_to_endpoints_needing_ack(endpoint);
     }
 
-    /* A hueristic: set to send this ACK after we have checked our
+    /* A heuristic: set to send this ACK after we have checked our
        incoming DATA_CHANNEL component.act_iteration_delay times
        (i.e., so we can piggyback an ACK on an outgoing send) */
     if (0 == endpoint->endpoint_acktime) {
-        endpoint->endpoint_acktime =
-            get_ticks() + mca_btl_usnic_component.ack_iteration_delay;
+        endpoint->endpoint_acktime = get_ticks() + mca_btl_usnic_component.ack_iteration_delay;
     }
 
     /* Save this incoming segment in the received segmentss array on the
@@ -143,11 +136,9 @@ opal_btl_usnic_update_window(
     endpoint->endpoint_rfstart = i;
 }
 
-static inline int
-opal_btl_usnic_check_rx_seq(
-    opal_btl_usnic_endpoint_t *endpoint,
-    opal_btl_usnic_recv_segment_t *seg,
-    uint32_t *window_index)
+static inline int opal_btl_usnic_check_rx_seq(opal_btl_usnic_endpoint_t *endpoint,
+                                              opal_btl_usnic_recv_segment_t *seg,
+                                              uint32_t *window_index)
 {
     uint32_t i;
     opal_btl_usnic_seq_t seq;
@@ -158,11 +149,11 @@ opal_btl_usnic_check_rx_seq(
      */
     if (seg->rs_base.us_btl_header->ack_present) {
 #if MSGDEBUG1
-        opal_output(0, "Handle piggy-packed ACK seq %"UDSEQ"\n", seg->rs_base.us_btl_header->ack_seq);
+        opal_output(0, "Handle piggy-packed ACK seq %" UDSEQ "\n",
+                    seg->rs_base.us_btl_header->ack_seq);
 #endif
         OPAL_THREAD_LOCK(&btl_usnic_lock);
-        opal_btl_usnic_handle_ack(endpoint,
-                seg->rs_base.us_btl_header->ack_seq);
+        opal_btl_usnic_handle_ack(endpoint, seg->rs_base.us_btl_header->ack_seq);
         OPAL_THREAD_UNLOCK(&btl_usnic_lock);
     }
 
@@ -193,13 +184,13 @@ opal_btl_usnic_check_rx_seq(
     delta = SEQ_DIFF(seq, endpoint->endpoint_next_contig_seq_to_recv);
     if (delta < 0 || delta >= WINDOW_SIZE) {
 #if MSGDEBUG1
-            opal_output(0, "<-- Received FRAG/CHUNK ep %p, seq %" UDSEQ " outside of window (%" UDSEQ " - %" UDSEQ "), %p, module %p -- DROPPED\n",
-                        (void*)endpoint, seg->rs_base.us_btl_header->pkt_seq,
-                        endpoint->endpoint_next_contig_seq_to_recv,
-                        (endpoint->endpoint_next_contig_seq_to_recv +
-                         WINDOW_SIZE - 1),
-                        (void*) seg,
-                        (void*) endpoint->endpoint_module);
+        opal_output(0,
+                    "<-- Received FRAG/CHUNK ep %p, seq %" UDSEQ " outside of window (%" UDSEQ
+                    " - %" UDSEQ "), %p, module %p -- DROPPED\n",
+                    (void *) endpoint, seg->rs_base.us_btl_header->pkt_seq,
+                    endpoint->endpoint_next_contig_seq_to_recv,
+                    (endpoint->endpoint_next_contig_seq_to_recv + WINDOW_SIZE - 1), (void *) seg,
+                    (void *) endpoint->endpoint_module);
 #endif
 
         /* Stats */
@@ -217,7 +208,7 @@ opal_btl_usnic_check_rx_seq(
        ACKed).
 
        We have saved all un-ACKed segment in an array on the
-       endpoint that is the same legnth as the receiver's window
+       endpoint that is the same length as the receiver's window
        (i.e., WINDOW_SIZE).  We can use the incoming segment sequence
        number to find its position in the array.  It's a little
        tricky because the left edge of the receiver window keeps
@@ -238,15 +229,16 @@ opal_btl_usnic_check_rx_seq(
     i = WINDOW_SIZE_MOD(i + endpoint->endpoint_rfstart);
     if (endpoint->endpoint_rcvd_segs[i]) {
 #if MSGDEBUG1
-        opal_output(0, "<-- Received FRAG/CHUNK ep %p, seq %" UDSEQ ", seg %p: duplicate -- DROPPED\n",
-            (void*) endpoint, seg->rs_base.us_btl_header->pkt_seq, (void*) seg);
+        opal_output(0,
+                    "<-- Received FRAG/CHUNK ep %p, seq %" UDSEQ ", seg %p: duplicate -- DROPPED\n",
+                    (void *) endpoint, seg->rs_base.us_btl_header->pkt_seq, (void *) seg);
 #endif
         /* highest_seq_rcvd is for debug stats only; it's not used
            in any window calculations */
         assert(SEQ_LE(seq, endpoint->endpoint_highest_seq_rcvd));
         /* next_contig_seq_to_recv-1 is the ack number we'll
            send */
-        assert (SEQ_GT(seq, endpoint->endpoint_next_contig_seq_to_recv - 1));
+        assert(SEQ_GT(seq, endpoint->endpoint_next_contig_seq_to_recv - 1));
 
         /* Stats */
         ++endpoint->endpoint_module->stats.num_dup_recvs;
@@ -275,13 +267,12 @@ dup_needs_ack:
  * possible.
  * See README.txt for a discussion of receive fastpath
  */
-static inline void
-opal_btl_usnic_recv_fast(opal_btl_usnic_module_t *module,
-                         opal_btl_usnic_recv_segment_t *seg,
-                         opal_btl_usnic_channel_t *channel)
+static inline void opal_btl_usnic_recv_fast(opal_btl_usnic_module_t *module,
+                                            opal_btl_usnic_recv_segment_t *seg,
+                                            opal_btl_usnic_channel_t *channel)
 {
     opal_btl_usnic_segment_t *bseg;
-    mca_btl_active_message_callback_t* reg;
+    mca_btl_active_message_callback_t *reg;
     opal_btl_usnic_seq_t seq;
     opal_btl_usnic_endpoint_t *endpoint;
     int delta;
@@ -304,10 +295,9 @@ opal_btl_usnic_dump_hex(15, USNIC_OUT, bseg->us_btl_header, bseg->us_btl_header-
        wholly contained in this one message -- it is not chunked
        across multiple messages), and it's not a PUT from the sender,
        then just handle it here. */
-    if (endpoint != NULL && !endpoint->endpoint_exiting &&
-            (OPAL_BTL_USNIC_PAYLOAD_TYPE_FRAG ==
-                bseg->us_btl_header->payload_type) &&
-            seg->rs_base.us_btl_header->put_addr == NULL) {
+    if (endpoint != NULL && !endpoint->endpoint_exiting
+        && (OPAL_BTL_USNIC_PAYLOAD_TYPE_FRAG == bseg->us_btl_header->payload_type)
+        && seg->rs_base.us_btl_header->put_addr == NULL) {
 
         seq = seg->rs_base.us_btl_header->pkt_seq;
         delta = SEQ_DIFF(seq, endpoint->endpoint_next_contig_seq_to_recv);
@@ -329,10 +319,12 @@ opal_btl_usnic_dump_hex(15, USNIC_OUT, bseg->us_btl_header, bseg->us_btl_header-
          */
         reg = mca_btl_base_active_message_trigger + bseg->us_btl_header->tag;
         seg->rs_segment.seg_len = bseg->us_btl_header->payload_len;
-        reg->cbfunc(&module->super, bseg->us_btl_header->tag,
-                    &seg->rs_desc, reg->cbdata);
+        seg->rs_desc.endpoint = endpoint;
+        seg->rs_desc.tag = bseg->us_btl_header->tag;
+        seg->rs_desc.cbdata = reg->cbdata;
+        reg->cbfunc(&module->super, &seg->rs_desc);
 
-drop:
+    drop:
         channel->chan_deferred_recv = seg;
     }
 
@@ -344,21 +336,18 @@ drop:
 
 /*
  */
-static inline int
-opal_btl_usnic_recv_frag_bookkeeping(
-    opal_btl_usnic_module_t* module,
-    opal_btl_usnic_recv_segment_t *seg,
-    opal_btl_usnic_channel_t *channel)
+static inline int opal_btl_usnic_recv_frag_bookkeeping(opal_btl_usnic_module_t *module,
+                                                       opal_btl_usnic_recv_segment_t *seg,
+                                                       opal_btl_usnic_channel_t *channel)
 {
-    opal_btl_usnic_endpoint_t* endpoint;
+    opal_btl_usnic_endpoint_t *endpoint;
     uint32_t window_index;
     int rc;
 
     endpoint = seg->rs_endpoint;
 
     /* Valgrind help */
-    opal_memchecker_base_mem_defined(
-                (void*)(seg->rs_protocol_header), seg->rs_len);
+    opal_memchecker_base_mem_defined((void *) (seg->rs_protocol_header), seg->rs_len);
 
     ++module->stats.num_total_recvs;
 
@@ -391,13 +380,12 @@ repost:
  * We have received a segment, take action based on the
  * packet type in the BTL header
  */
-static inline void
-opal_btl_usnic_recv(opal_btl_usnic_module_t *module,
-                    opal_btl_usnic_recv_segment_t *seg,
-                    opal_btl_usnic_channel_t *channel)
+static inline void opal_btl_usnic_recv(opal_btl_usnic_module_t *module,
+                                       opal_btl_usnic_recv_segment_t *seg,
+                                       opal_btl_usnic_channel_t *channel)
 {
     opal_btl_usnic_segment_t *bseg;
-    mca_btl_active_message_callback_t* reg;
+    mca_btl_active_message_callback_t *reg;
     opal_btl_usnic_endpoint_t *endpoint;
     int rc;
 
@@ -414,13 +402,12 @@ opal_btl_usnic_recv(opal_btl_usnic_module_t *module,
        wholly contained in this one message -- it is not chunked
        across multiple messages), and it's not a PUT from the sender,
        then just handle it here. */
-    if (endpoint != NULL && !endpoint->endpoint_exiting &&
-            (OPAL_BTL_USNIC_PAYLOAD_TYPE_FRAG ==
-                bseg->us_btl_header->payload_type) &&
-            seg->rs_base.us_btl_header->put_addr == NULL) {
+    if (endpoint != NULL && !endpoint->endpoint_exiting
+        && (OPAL_BTL_USNIC_PAYLOAD_TYPE_FRAG == bseg->us_btl_header->payload_type)
+        && seg->rs_base.us_btl_header->put_addr == NULL) {
 
         MSGDEBUG1_OUT("<-- Received FRAG (fastpath) ep %p, seq %" UDSEQ ", len=%" PRIu16 "\n",
-                      (void*) endpoint, bseg->us_btl_header->pkt_seq,
+                      (void *) endpoint, bseg->us_btl_header->pkt_seq,
                       bseg->us_btl_header->payload_len);
 
         /* do the receive bookkeeping */
@@ -437,8 +424,9 @@ opal_btl_usnic_recv(opal_btl_usnic_module_t *module,
          */
         reg = mca_btl_base_active_message_trigger + bseg->us_btl_header->tag;
         seg->rs_segment.seg_len = bseg->us_btl_header->payload_len;
-        reg->cbfunc(&module->super, bseg->us_btl_header->tag,
-                    &seg->rs_desc, reg->cbdata);
+        seg->rs_desc.tag = bseg->us_btl_header->tag;
+        seg->rs_desc.cbdata = reg->cbdata;
+        reg->cbfunc(&module->super, &seg->rs_desc);
 
     }
 
