@@ -27,9 +27,7 @@
 BEGIN_C_DECLS
 
 #define OPAL_ERROR_LOG(r) \
-    opal_output(0, "OPAL ERROR: %s in file %s at line %d", \
-                opal_strerror((r)), __FILE__, __LINE__);
-
+    opal_output(0, "OPAL ERROR: %s in file %s at line %d", opal_strerror((r)), __FILE__, __LINE__);
 
 /**
  * Prints error message for errnum on stderr
@@ -72,7 +70,6 @@ OPAL_DECLSPEC const char *opal_strerror(int errnum);
  */
 OPAL_DECLSPEC int opal_strerror_r(int errnum, char *strerrbuf, size_t buflen);
 
-
 typedef int (*opal_err2str_fn_t)(int errnum, const char **str);
 
 /**
@@ -86,8 +83,7 @@ typedef int (*opal_err2str_fn_t)(int errnum, const char **str);
  * \note A maximum of 5 converters can be registered.  The 6th
  * converter registration attempt will return OPAL_ERR_OUT_OF_RESOURCE
  */
-OPAL_DECLSPEC int opal_error_register(const char *project,
-                                      int err_base, int err_max,
+OPAL_DECLSPEC int opal_error_register(const char *project, int err_base, int err_max,
                                       opal_err2str_fn_t converter);
 
 /**
