@@ -9,7 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2009-2014 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2009-2018 Cisco Systems, Inc.  All rights reserved
  * Copyright (c) 2010      IBM Corporation.  All rights reserved.
  * Copyright (c) 2012-2013 Los Alamos National Security, LLC.
  *                         All rights reserved.
@@ -25,64 +25,64 @@
  */
 
 #include "opal_config.h"
+#include <errno.h>
 #include <stdlib.h>
 #include <string.h>
-#include <errno.h>
 #ifdef HAVE_UNISTD_H
-#include <unistd.h>
+#    include <unistd.h>
 #endif
 #ifdef HAVE_SHLWAPI_H
-#include <shlwapi.h>
+#    include <shlwapi.h>
 #endif
 #ifdef HAVE_SYS_PARAM_H
-#include <sys/param.h>
+#    include <sys/param.h>
 #endif
 #ifdef HAVE_SYS_MOUNT_H
-#include <sys/mount.h>
+#    include <sys/mount.h>
 #endif
 #ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
+#    include <sys/types.h>
 #endif
 #ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
+#    include <sys/stat.h>
 #endif
 #ifdef HAVE_SYS_VFS_H
-#include <sys/vfs.h>
+#    include <sys/vfs.h>
 #endif
 #ifdef HAVE_SYS_STATFS_H
-#include <sys/statfs.h>
+#    include <sys/statfs.h>
 #endif
 #ifdef HAVE_SYS_STATVFS_H
-#include <sys/statvfs.h>
+#    include <sys/statvfs.h>
 #endif
 #ifdef HAVE_SYS_MOUNT_H
-#include <sys/mount.h>
+#    include <sys/mount.h>
 #endif
 #ifdef HAVE_MNTENT_H
-#include <mntent.h>
+#    include <mntent.h>
 #endif
 #ifdef HAVE_PATHS_H
-#include <paths.h>
+#    include <paths.h>
 #endif
 
 #ifdef _PATH_MOUNTED
-#define MOUNTED_FILE _PATH_MOUNTED
+#    define MOUNTED_FILE _PATH_MOUNTED
 #else
-#define MOUNTED_FILE "/etc/mtab"
+#    define MOUNTED_FILE "/etc/mtab"
 #endif
 
-
-#include "opal_stdint.h"
+#include "opal/util/argv.h"
+#include "opal/util/os_path.h"
 #include "opal/util/output.h"
 #include "opal/util/path.h"
-#include "opal/util/os_path.h"
-#include "opal/util/argv.h"
+#include "opal/util/printf.h"
+#include "opal_stdint.h"
 
 /*
  * Sanity check to ensure we have either statfs or statvfs
  */
 #if !defined(HAVE_STATFS) && !defined(HAVE_STATVFS)
-#error Must have either statfs() or statvfs()
+#    error Must have either statfs() or statvfs()
 #endif
 
 /*
@@ -90,17 +90,17 @@
  * no struct statfs (!).  So check to make sure we have struct statfs
  * before allowing the use of statfs().
  */
-#if defined(HAVE_STATFS) && (defined(HAVE_STRUCT_STATFS_F_FSTYPENAME) || \
-                             defined(HAVE_STRUCT_STATFS_F_TYPE))
-#define USE_STATFS 1
+#if defined(HAVE_STATFS) \
+    && (defined(HAVE_STRUCT_STATFS_F_FSTYPENAME) || defined(HAVE_STRUCT_STATFS_F_TYPE))
+#    define USE_STATFS 1
 #endif
 
 static void path_env_load(char *path, int *pargc, char ***pargv);
 static char *list_env_get(char *var, char **list);
 
-bool opal_path_is_absolute( const char *path )
+bool opal_path_is_absolute(const char *path)
 {
-    if( OPAL_PATH_SEP[0] == *path ) {
+    if (OPAL_PATH_SEP[0] == *path) {
         return true;
     }
     return false;
@@ -118,7 +118,7 @@ char *opal_path_find(char *fname, char **pathv, int mode, char **envv)
     int i;
 
     /* If absolute path is given, return it without searching. */
-    if( opal_path_is_absolute(fname) ) {
+    if (opal_path_is_absolute(fname)) {
         return opal_path_access(fname, NULL, mode);
     }
 
@@ -138,7 +138,7 @@ char *opal_path_find(char *fname, char **pathv, int mode, char **envv)
             if (delimit) {
                 *delimit = '\0';
             }
-            env = list_env_get(pathv[i]+1, envv);
+            env = list_env_get(pathv[i] + 1, envv);
             if (delimit) {
                 *delimit = OPAL_PATH_SEP[0];
             }
@@ -146,18 +146,12 @@ char *opal_path_find(char *fname, char **pathv, int mode, char **envv)
                 if (!delimit) {
                     fullpath = opal_path_access(fname, env, mode);
                 } else {
-                    pfix = (char*) malloc(strlen(env) + strlen(delimit) + 1);
-                    if (NULL == pfix) {
-                        return NULL;
-                    }
-                    strcpy(pfix, env);
-                    strcat(pfix, delimit);
+                    opal_asprintf(&pfix, "%s%s", env, delimit);
                     fullpath = opal_path_access(fname, pfix, mode);
                     free(pfix);
                 }
             }
-        }
-        else {
+        } else {
             fullpath = opal_path_access(fname, pathv[i], mode);
         }
         i++;
@@ -194,7 +188,7 @@ char *opal_path_findv(char *fname, int mode, char **envv, char *wrkdir)
                 found_dot = true;
                 free(dirv[i]);
                 dirv[i] = strdup(wrkdir);
-                if (NULL == dirv[i]){
+                if (NULL == dirv[i]) {
                     return NULL;
                 }
             }
@@ -208,16 +202,16 @@ char *opal_path_findv(char *fname, int mode, char **envv, char *wrkdir)
         opal_argv_append(&dirc, &dirv, wrkdir);
     }
 
-    if(NULL == dirv)
+    if (NULL == dirv) {
         return NULL;
+    }
     fullpath = opal_path_find(fname, dirv, mode, envv);
     opal_argv_free(dirv);
     return fullpath;
 }
 
-
 /**
- *  Forms a complete pathname and checks it for existance and
+ *  Forms a complete pathname and checks it for existence and
  *  permissions
  *
  *  Accepts:
@@ -256,8 +250,7 @@ char *opal_path_access(char *fname, char *path, int mode)
         return NULL;
     }
 
-    if (!(S_IFREG & buf.st_mode) &&
-        !(S_IFLNK & buf.st_mode)) {
+    if (!(S_IFREG & buf.st_mode) && !(S_IFLNK & buf.st_mode)) {
         /* this isn't a regular file or a symbolic link, so
          * ignore it
          */
@@ -292,7 +285,6 @@ char *opal_path_access(char *fname, char *path, int mode)
     return fullpath;
 }
 
-
 /**
  *
  *  Loads argument array with $PATH env var.
@@ -341,7 +333,6 @@ static void path_env_load(char *path, int *pargc, char ***pargv)
     }
 }
 
-
 /**
  *  Gets value of variable in list or environment. Looks in the list first
  *
@@ -380,31 +371,37 @@ static char *list_env_get(char *var, char **list)
  * function will return NULL. Otherwise, an newly allocated string
  * will be returned.
  */
-char* opal_find_absolute_path( char* app_name )
+char *opal_find_absolute_path(char *app_name)
 {
-    char* abs_app_name;
+    char *abs_app_name;
     char cwd[OPAL_PATH_MAX], *pcwd;
 
-    if( opal_path_is_absolute(app_name) ) { /* already absolute path */
+    if (opal_path_is_absolute(app_name)) { /* already absolute path */
         abs_app_name = app_name;
-    } else if ( '.' == app_name[0] ||
-               NULL != strchr(app_name, OPAL_PATH_SEP[0])) {
+    } else if ('.' == app_name[0] || NULL != strchr(app_name, OPAL_PATH_SEP[0])) {
         /* the app is in the current directory or below it */
-        pcwd = getcwd( cwd, OPAL_PATH_MAX );
-        if( NULL == pcwd ) {
+        pcwd = getcwd(cwd, OPAL_PATH_MAX);
+        if (NULL == pcwd) {
             /* too bad there is no way we can get the app absolute name */
             return NULL;
         }
-        abs_app_name = opal_os_path( false, pcwd, app_name, NULL );
+        abs_app_name = opal_os_path(false, pcwd, app_name, NULL);
     } else {
         /* Otherwise try to search for the application in the PATH ... */
-        abs_app_name = opal_path_findv( app_name, X_OK, NULL, NULL );
+        abs_app_name = opal_path_findv(app_name, X_OK, NULL, NULL);
     }
 
-    if( NULL != abs_app_name ) {
-        char* resolved_path = (char*)malloc(OPAL_PATH_MAX);
-        realpath( abs_app_name, resolved_path );
-        if( abs_app_name != app_name ) free(abs_app_name);
+    if (NULL != abs_app_name) {
+        char *resolved_path = (char *) malloc(OPAL_PATH_MAX);
+        char *ret;
+        ret = realpath(abs_app_name, resolved_path);
+        if (abs_app_name != app_name) {
+            free(abs_app_name);
+        }
+        if (NULL == ret) {
+            free(resolved_path);
+            resolved_path = NULL;
+        }
         return resolved_path;
     }
     return NULL;
@@ -420,15 +417,13 @@ static char *opal_check_mtab(char *dev_path)
 {
 
 #ifdef HAVE_MNTENT_H
-    FILE * mtab = NULL;
-    struct mntent * part = NULL;
+    FILE *mtab = NULL;
+    struct mntent *part = NULL;
 
     if ((mtab = setmntent(MOUNTED_FILE, "r")) != NULL) {
         while (NULL != (part = getmntent(mtab))) {
-            if ((NULL != part->mnt_dir) &&
-                (NULL != part->mnt_type) &&
-                (0 == strcmp(part->mnt_dir, dev_path)))
-            {
+            if ((NULL != part->mnt_dir) && (NULL != part->mnt_type)
+                && (0 == strcmp(part->mnt_dir, dev_path))) {
                 endmntent(mtab);
                 return strdup(part->mnt_type);
             }
@@ -439,7 +434,6 @@ static char *opal_check_mtab(char *dev_path)
     return NULL;
 }
 
-
 /**
  * @brief Figure out, whether fname is on network file system
  *
@@ -485,26 +479,26 @@ static char *opal_check_mtab(char *dev_path)
  *          return 0 success, -1 on failure with errno set.
  */
 #ifndef LL_SUPER_MAGIC
-#define LL_SUPER_MAGIC                    0x0BD00BD0     /* Lustre magic number */
+#    define LL_SUPER_MAGIC 0x0BD00BD0 /* Lustre magic number */
 #endif
 #ifndef NFS_SUPER_MAGIC
-#define NFS_SUPER_MAGIC                   0x6969
+#    define NFS_SUPER_MAGIC 0x6969
 #endif
 #ifndef PAN_KERNEL_FS_CLIENT_SUPER_MAGIC
-#define PAN_KERNEL_FS_CLIENT_SUPER_MAGIC  0xAAD7AAEA     /* Panasas FS */
+#    define PAN_KERNEL_FS_CLIENT_SUPER_MAGIC 0xAAD7AAEA /* Panasas FS */
 #endif
 #ifndef GPFS_SUPER_MAGIC
-#define GPFS_SUPER_MAGIC  0x47504653    /* Thats GPFS in ASCII */
+#    define GPFS_SUPER_MAGIC 0x47504653 /* That's GPFS in ASCII */
 #endif
 #ifndef AUTOFS_SUPER_MAGIC
-#define AUTOFS_SUPER_MAGIC 0x0187
+#    define AUTOFS_SUPER_MAGIC 0x0187
 #endif
 #ifndef PVFS2_SUPER_MAGIC
-#define PVFS2_SUPER_MAGIC 0x20030528
+#    define PVFS2_SUPER_MAGIC 0x20030528
 #endif
 
-#define MASK2        0xffff
-#define MASK4    0xffffffff
+#define MASK2 0xffff
+#define MASK4 0xffffffff
 
 bool opal_path_nfs(char *fname, char **ret_fstype)
 {
@@ -512,7 +506,7 @@ bool opal_path_nfs(char *fname, char **ret_fstype)
     int fsrc = -1;
     int vfsrc = -1;
     int trials;
-    char * file = strdup (fname);
+    char *file = strdup(fname);
 #if defined(USE_STATFS)
     struct statfs fsbuf;
 #endif
@@ -526,16 +520,14 @@ bool opal_path_nfs(char *fname, char **ret_fstype)
     static struct fs_types_t {
         unsigned long long f_fsid;
         unsigned long long f_mask;
-        const char * f_fsname;
-    } fs_types[] = {
-        {LL_SUPER_MAGIC,                   MASK4, "lustre"},
-        {NFS_SUPER_MAGIC,                  MASK2, "nfs"},
-        {AUTOFS_SUPER_MAGIC,               MASK2, "autofs"},
-        {PAN_KERNEL_FS_CLIENT_SUPER_MAGIC, MASK4, "panfs"},
-        {GPFS_SUPER_MAGIC,                 MASK4, "gpfs"},
-        {PVFS2_SUPER_MAGIC,                MASK4, "pvfs2"}
-    };
-#define FS_TYPES_NUM (int)(sizeof (fs_types)/sizeof (fs_types[0]))
+        const char *f_fsname;
+    } fs_types[] = {{LL_SUPER_MAGIC, MASK4, "lustre"},
+                    {NFS_SUPER_MAGIC, MASK2, "nfs"},
+                    {AUTOFS_SUPER_MAGIC, MASK2, "autofs"},
+                    {PAN_KERNEL_FS_CLIENT_SUPER_MAGIC, MASK4, "panfs"},
+                    {GPFS_SUPER_MAGIC, MASK4, "gpfs"},
+                    {PVFS2_SUPER_MAGIC, MASK4, "pvfs2"}};
+#define FS_TYPES_NUM (int) (sizeof(fs_types) / sizeof(fs_types[0]))
 
     /*
      * First, get the OS-dependent struct stat(v)fs buf.  This may
@@ -559,13 +551,14 @@ again:
     /* In case some error with the current filename, try the parent
        directory */
     if (-1 == fsrc && -1 == vfsrc) {
-        char * last_sep;
+        char *last_sep;
 
-        OPAL_OUTPUT_VERBOSE((10, 0, "opal_path_nfs: stat(v)fs on file:%s failed errno:%d directory:%s\n",
+        OPAL_OUTPUT_VERBOSE((10, 0,
+                             "opal_path_nfs: stat(v)fs on file:%s failed errno:%d directory:%s\n",
                              fname, errno, file));
         if (EPERM == errno) {
             free(file);
-            if ( NULL != ret_fstype ) {
+            if (NULL != ret_fstype) {
                 *ret_fstype = NULL;
             }
             return false;
@@ -573,11 +566,10 @@ again:
 
         last_sep = strrchr(file, OPAL_PATH_SEP[0]);
         /* Stop the search, when we have searched past root '/' */
-        if (NULL == last_sep || (1 == strlen(last_sep) &&
-            OPAL_PATH_SEP[0] == *last_sep)) {
-            free (file);
-            if ( NULL != ret_fstype ) {
-                *ret_fstype=NULL;
+        if (NULL == last_sep || (1 == strlen(last_sep) && OPAL_PATH_SEP[0] == *last_sep)) {
+            free(file);
+            if (NULL != ret_fstype) {
+                *ret_fstype = NULL;
             }
             return false;
         }
@@ -591,15 +583,15 @@ again:
 #if defined(USE_STATFS)
         /* These are uses of struct statfs */
 #    if defined(HAVE_STRUCT_STATFS_F_FSTYPENAME)
-        if (0 == fsrc &&
-            0 == strncasecmp(fs_types[i].f_fsname, fsbuf.f_fstypename,
-                             sizeof(fsbuf.f_fstypename))) {
+        if (0 == fsrc
+            && 0
+                   == strncasecmp(fs_types[i].f_fsname, fsbuf.f_fstypename,
+                                  sizeof(fsbuf.f_fstypename))) {
             goto found;
         }
 #    endif
 #    if defined(HAVE_STRUCT_STATFS_F_TYPE)
-        if (0 == fsrc &&
-            fs_types[i].f_fsid == (fsbuf.f_type & fs_types[i].f_mask)) {
+        if (0 == fsrc && fs_types[i].f_fsid == (fsbuf.f_type & fs_types[i].f_mask)) {
             goto found;
         }
 #    endif
@@ -608,31 +600,33 @@ again:
 #if defined(HAVE_STATVFS)
         /* These are uses of struct statvfs */
 #    if defined(HAVE_STRUCT_STATVFS_F_BASETYPE)
-        if (0 == vfsrc &&
-            0 == strncasecmp(fs_types[i].f_fsname, vfsbuf.f_basetype,
-                             sizeof(vfsbuf.f_basetype))) {
+        if (0 == vfsrc
+            && 0
+                   == strncasecmp(fs_types[i].f_fsname, vfsbuf.f_basetype,
+                                  sizeof(vfsbuf.f_basetype))) {
             goto found;
         }
 #    endif
 #    if defined(HAVE_STRUCT_STATVFS_F_FSTYPENAME)
-        if (0 == vfsrc &&
-            0 == strncasecmp(fs_types[i].f_fsname, vfsbuf.f_fstypename,
-                             sizeof(vfsbuf.f_fstypename))) {
+        if (0 == vfsrc
+            && 0
+                   == strncasecmp(fs_types[i].f_fsname, vfsbuf.f_fstypename,
+                                  sizeof(vfsbuf.f_fstypename))) {
             goto found;
         }
 #    endif
 #endif
     }
 
-    free (file);
-    if ( NULL != ret_fstype ) {
-        *ret_fstype=NULL;
+    free(file);
+    if (NULL != ret_fstype) {
+        *ret_fstype = NULL;
     }
     return false;
 
 found:
 
-    free (file);
+    free(file);
     if (AUTOFS_SUPER_MAGIC == fs_types[i].f_fsid) {
         char *fs_type = opal_check_mtab(fname);
         int x;
@@ -642,35 +636,33 @@ found:
                     continue;
                 }
                 if (0 == strcasecmp(fs_types[x].f_fsname, fs_type)) {
-                    OPAL_OUTPUT_VERBOSE((10, 0, "opal_path_nfs: file:%s on fs:%s\n", fname, fs_type));
+                    OPAL_OUTPUT_VERBOSE(
+                        (10, 0, "opal_path_nfs: file:%s on fs:%s\n", fname, fs_type));
                     free(fs_type);
-                    if ( NULL != ret_fstype ) {
+                    if (NULL != ret_fstype) {
                         *ret_fstype = strdup(fs_types[x].f_fsname);
                     }
                     return true;
                 }
             }
             free(fs_type);
-            if ( NULL != ret_fstype ) {
-                *ret_fstype=NULL;
+            if (NULL != ret_fstype) {
+                *ret_fstype = NULL;
             }
             return false;
         }
     }
 
-    OPAL_OUTPUT_VERBOSE((10, 0, "opal_path_nfs: file:%s on fs:%s\n",
-                fname, fs_types[i].f_fsname));
-    if ( NULL != ret_fstype ) {
-        *ret_fstype = strdup (fs_types[i].f_fsname);
+    OPAL_OUTPUT_VERBOSE((10, 0, "opal_path_nfs: file:%s on fs:%s\n", fname, fs_types[i].f_fsname));
+    if (NULL != ret_fstype) {
+        *ret_fstype = strdup(fs_types[i].f_fsname);
     }
     return true;
 
 #undef FS_TYPES_NUM
 }
 
-int
-opal_path_df(const char *path,
-             uint64_t *out_avail)
+int opal_path_df(const char *path, uint64_t *out_avail)
 {
     int rc = -1;
     int trials = 5;
@@ -696,18 +688,20 @@ opal_path_df(const char *path,
     } while (-1 == rc && ESTALE == err && (--trials > 0));
 
     if (-1 == rc) {
-        OPAL_OUTPUT_VERBOSE((10, 2, "opal_path_df: stat(v)fs on "
+        OPAL_OUTPUT_VERBOSE((10, 2,
+                             "opal_path_df: stat(v)fs on "
                              "path: %s failed with errno: %d (%s)\n",
                              path, err, strerror(err)));
         return OPAL_ERROR;
     }
 
     /* now set the amount of free space available on path */
-                               /* sometimes buf.f_bavail is negative */
-    *out_avail = (uint64_t)buf.f_bsize * (uint64_t)((long)buf.f_bavail < 0 ? 0 : buf.f_bavail);
+    /* sometimes buf.f_bavail is negative */
+    *out_avail = (uint64_t) buf.f_bsize * (uint64_t)((long) buf.f_bavail < 0 ? 0 : buf.f_bavail);
 
-    OPAL_OUTPUT_VERBOSE((10, 2, "opal_path_df: stat(v)fs states "
-                         "path: %s has %"PRIu64 " B of free space.",
+    OPAL_OUTPUT_VERBOSE((10, 2,
+                         "opal_path_df: stat(v)fs states "
+                         "path: %s has %" PRIu64 " B of free space.",
                          path, *out_avail));
 
     return OPAL_SUCCESS;
