@@ -1,10 +1,12 @@
-/**
-  Copyright (c) 2011 Mellanox Technologies. All rights reserved.
-  $COPYRIGHT$
-
-  Additional copyrights may follow
-
-  $HEADER$
+/*
+ * Copyright (c) 2011      Mellanox Technologies. All rights reserved.
+ * Copyright (c) 2019      Research Organization for Information Science
+ *                         and Technology (RIST).  All rights reserved.
+ * $COPYRIGHT$
+ *
+ * Additional copyrights may follow
+ *
+ * $HEADER$
  */
 
 #ifndef MCA_SCOLL_MPI_H
@@ -20,8 +22,6 @@
 #include "oshmem/proc/proc.h"
 #include "ompi/communicator/communicator.h"
 
-#include "orte/runtime/orte_globals.h"
-
 #include "scoll_mpi_debug.h"
 BEGIN_C_DECLS
 
@@ -49,7 +49,7 @@ struct mca_scoll_mpi_component_t {
 };
 typedef struct mca_scoll_mpi_component_t mca_scoll_mpi_component_t;
 
-OMPI_MODULE_DECLSPEC extern mca_scoll_mpi_component_t mca_scoll_mpi_component;
+OMPI_DECLSPEC extern mca_scoll_mpi_component_t mca_scoll_mpi_component;
 
 
 /**
