@@ -1,9 +1,10 @@
 /*
  * Copyright (c) 2013-2018 Mellanox Technologies, Inc.
  *                         All rights reserved.
- * Copyright (c) 2016      Research Organization for Information Science
+ * Copyright (c) 2016-2019 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2017      Cisco Systems, Inc.  All rights reserved
+ * Copyright (c) 2021      Nanook Consulting.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -18,13 +19,11 @@
 #include "oshmem/constants.h"
 
 #include "opal/class/opal_list.h"
+#include "opal/class/opal_bitmap.h"
 #include "opal/util/proc.h"
-#include "opal/dss/dss_types.h"
+#include "opal/util/argv.h"
 #include "opal/mca/hwloc/hwloc-internal.h"
 
-#include "orte/types.h"
-#include "orte/runtime/orte_globals.h"
-
 #include "ompi/proc/proc.h"
 #include "ompi/communicator/communicator.h"
 
@@ -40,19 +39,6 @@ struct oshmem_group_t;
 
 #define OSHMEM_PE_INVALID   (-1)
 
-/* This struct will be copied into the padding field of an ompi_proc_t
- * so the size of oshmem_proc_data_t must be less or equal than
- * OMPI_PROC_PADDING_SIZE */
-struct oshmem_proc_data_t {
-    char * transport_ids;
-    int num_transports;
-};
-
-typedef struct oshmem_proc_data_t oshmem_proc_data_t;
-
-#define OSHMEM_PROC_DATA(proc) \
-    ((oshmem_proc_data_t *)(proc)->padding)
-
 /**
  * Group of Open SHMEM processes structure
  *
@@ -64,9 +50,7 @@ struct oshmem_group_t {
     int                         my_pe;
     int                         proc_count;     /**< number of processes in group */
     int                         is_member;   /* true if my_pe is part of the group, participate in collectives */
-    struct ompi_proc_t          **proc_array; /**< list of pointers to ompi_proc_t structures
-                                                   for each process in the group */
-    opal_list_t                 peer_list;
+    opal_vpid_t                 *proc_vpids; /* vpids of each process in group */
 
     /* Collectives module interface and data */
     mca_scoll_base_group_scoll_t g_scoll;
@@ -138,28 +122,35 @@ static inline ompi_proc_t *oshmem_proc_local(void)
  *
  * @return Pointer to the process instance for \c name
  */
-static inline ompi_proc_t *oshmem_proc_for_find(const orte_process_name_t name)
+static inline ompi_proc_t *oshmem_proc_for_find(const ompi_process_name_t name)
 {
     return (ompi_proc_t *)ompi_proc_for_name(name);
 }
 
 static inline ompi_proc_t *oshmem_proc_find(int pe)
 {
-    orte_process_name_t name;
+    ompi_process_name_t name;
 
-    name.jobid = ORTE_PROC_MY_NAME->jobid;
+    name.jobid = OMPI_PROC_MY_NAME->jobid;
     name.vpid = pe;
     return oshmem_proc_for_find(name);
 }
 
-static inline int oshmem_proc_pe(ompi_proc_t *proc)
+static inline int oshmem_proc_pe_vpid(oshmem_group_t *group, int pe)
 {
-    return (proc ? (int) ((orte_process_name_t*)&proc->super.proc_name)->vpid : -1);
+    if (OPAL_LIKELY(pe < group->proc_count)) {
+        return (group->proc_vpids[pe]);
+    } else {
+        return -1;
+    }
 }
 
-#define OSHMEM_PROC_JOBID(PROC)    (((orte_process_name_t*)&((PROC)->super.proc_name))->jobid)
-#define OSHMEM_PROC_VPID(PROC)     (((orte_process_name_t*)&((PROC)->super.proc_name))->vpid)
+static inline int oshmem_proc_pe(ompi_proc_t *proc)
+{
+    return (proc ? (int) ((ompi_process_name_t*)&proc->super.proc_name)->vpid : -1);
+}
 
+bool oshmem_proc_on_local_node(int pe);
 /**
  * Initialize the OSHMEM process predefined groups
  *
@@ -235,40 +226,6 @@ fatal:
  */
 OSHMEM_DECLSPEC void oshmem_proc_group_destroy(oshmem_group_t* group);
 
-static inline ompi_proc_t *oshmem_proc_group_all(int pe)
-{
-    return oshmem_group_all->proc_array[pe];
-}
-
-static inline ompi_proc_t *oshmem_proc_group_find(oshmem_group_t* group,
-                                                    int pe)
-{
-    int i = 0;
-    ompi_proc_t* proc = NULL;
-
-    if (OPAL_LIKELY(group)) {
-        if (OPAL_LIKELY(group == oshmem_group_all)) {
-            /* To improve performance use direct index. It is feature of oshmem_group_all */
-            proc = group->proc_array[pe];
-        } else {
-            for (i = 0; i < group->proc_count; i++) {
-                if (pe == oshmem_proc_pe(group->proc_array[i])) {
-                    proc = group->proc_array[i];
-                    break;
-                }
-            }
-        }
-    } else {
-        orte_process_name_t name;
-
-        name.jobid = ORTE_PROC_MY_NAME->jobid;
-        name.vpid = pe;
-        proc = oshmem_proc_for_find(name);
-    }
-
-    return proc;
-}
-
 static inline int oshmem_proc_group_find_id(oshmem_group_t* group, int pe)
 {
     int i = 0;
@@ -276,7 +233,7 @@ static inline int oshmem_proc_group_find_id(oshmem_group_t* group, int pe)
 
     if (group) {
         for (i = 0; i < group->proc_count; i++) {
-            if (pe == oshmem_proc_pe(group->proc_array[i])) {
+            if (pe == oshmem_proc_pe_vpid(group, i)) {
                 id = i;
                 break;
             }
@@ -302,22 +259,6 @@ static inline int oshmem_my_proc_id(void)
     return oshmem_group_self->my_pe;
 }
 
-static inline int oshmem_get_transport_id(int pe)
-{
-    ompi_proc_t *proc;
-
-    proc = oshmem_proc_group_find(oshmem_group_all, pe);
-
-    return (int) OSHMEM_PROC_DATA(proc)->transport_ids[0];
-}
-
-static inline int oshmem_get_transport_count(int pe)
-{
-    ompi_proc_t *proc;
-    proc = oshmem_proc_group_find(oshmem_group_all, pe);
-    return OSHMEM_PROC_DATA(proc)->num_transports;
-}
-
 END_C_DECLS
 
 #endif /* OSHMEM_PROC_PROC_H */
