@@ -1,6 +1,8 @@
 /*
  * Copyright (c) 2012-2015 Mellanox Technologies, Inc.
  *                         All rights reserved.
+ * Copyright (c) 2019      Research Organization for Information Science
+ *                         and Technology (RIST).  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -11,13 +13,12 @@
 
 #include "oshmem/include/shmem.h"
 #include "oshmem/runtime/runtime.h"
-
-#include "orte/mca/errmgr/errmgr.h"
+#include "oshmem/proc/proc.h"
 
 #if OSHMEM_PROFILING
 #include "oshmem/include/pshmem.h"
 #pragma weak shmem_global_exit = pshmem_global_exit
-#include "oshmem/shmem/c/profile/defines.h"
+#include "oshmem/shmem/c/profile-defines.h"
 #endif
 
 extern int oshmem_shmem_inglobalexit;
@@ -38,7 +39,7 @@ static inline void _globalexit(int status)
 {
     oshmem_shmem_inglobalexit++;
 
-    orte_errmgr.abort(status, NULL);
+    ompi_rte_abort(status, NULL);
 
     oshmem_shmem_aborted = true;
     exit(status);
