@@ -1,6 +1,8 @@
 /*
  * Copyright (c) 2013      Mellanox Technologies, Inc.
  *                         All rights reserved.
+ * Copyright (c) 2019      Research Organization for Information Science
+ *                         and Technology (RIST).  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -12,8 +14,6 @@
 
 #include <stdlib.h>
 
-#include "orte/util/show_help.h"
-
 #include "oshmem/constants.h"
 #include "oshmem/include/shmem.h"
 
@@ -22,7 +22,7 @@
 #if OSHMEM_PROFILING
 #include "oshmem/include/pshmem.h"
 #pragma weak shmem_pe_accessible = pshmem_pe_accessible
-#include "oshmem/shmem/c/profile/defines.h"
+#include "oshmem/shmem/c/profile-defines.h"
 #endif
 
 int shmem_pe_accessible(int pe)
