@@ -1,6 +1,8 @@
 /*
  * Copyright (c) 2013      Mellanox Technologies, Inc.
  *                         All rights reserved.
+ * Copyright (c) 2019      Research Organization for Information Science
+ *                         and Technology (RIST).  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -12,8 +14,6 @@
 
 #include <stdlib.h>
 
-#include "orte/util/show_help.h"
-
 #include "oshmem/constants.h"
 #include "oshmem/include/shmem.h"
 #include "oshmem/shmem/shmem_api_logger.h"
@@ -26,7 +26,7 @@
 #if OSHMEM_PROFILING
 #include "oshmem/include/pshmem.h"
 #pragma weak shmem_ptr = pshmem_ptr
-#include "oshmem/shmem/c/profile/defines.h"
+#include "oshmem/shmem/c/profile-defines.h"
 #endif
 
 void *shmem_ptr(const void *dst_addr, int pe)
