@@ -162,7 +162,7 @@ static int pack_unpack_datatype( void* send_data, ompi_datatype_t *datatype, int
 
 int main(int argc, char *argv[])
 {
-    opal_init_util(&argc, &argv);
+    opal_init(&argc, &argv);
     ompi_datatype_init();
 
     /* Simple contiguous data: MPI_INT32_T */
@@ -245,7 +245,7 @@ int main(int argc, char *argv[])
         }
     }
 
-    ompi_datatype_finalize();
+    opal_finalize_util();
 
     return 0;
 }
